@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1999-2019 Douglas Gilbert.
+ * Copyright (c) 1999-2023 Douglas Gilbert.
  * All rights reserved.
  * Use of this source code is governed by a BSD-style
  * license that can be found in the BSD_LICENSE file.
@@ -42,7 +42,7 @@
 #endif
 
 
-static const char * const version_str = "1.95 20191219";
+static const char * const version_str = "2.02 20230311";
 
 
 #define SENSE_BUFF_LEN 64       /* Arbitrary, could be larger */
@@ -93,15 +93,14 @@ sg_cmds_close_device(int device_fd)
     return scsi_pt_close_device(device_fd);
 }
 
-static const char * const pass_through_s = "pass-through";
+static const char * const pt_s = "pass-through";
 
 static void
 sg_cmds_resid_print(const char * leadin, bool is_din, int num_req,
                     int num_got)
 {
-    pr2ws("    %s: %s requested %d bytes (data-%s  got %d "
-          "bytes%s\n", leadin, pass_through_s,num_req,
-          (is_din ? "in), got" : "out) but reported"), num_got,
+    pr2ws("    %s: %s requested %d bytes (data-%s %d bytes%s\n", leadin, pt_s,
+          num_req, (is_din ? "in), got" : "out) but reported"), num_got,
           (is_din ? "" : " sent"));
 }
 
@@ -110,16 +109,16 @@ sg_cmds_process_helper(const char * leadin, int req_din_x, int act_din_x,
                        int req_dout_x, int act_dout_x, const uint8_t * sbp,
                        int slen, bool noisy, int verbose, int * o_sense_cat)
 {
-    int scat;
     bool n = false;
     bool check_data_in = false;
-    char b[512];
+    int scat;
 
     scat = sg_err_category_sense(sbp, slen);
     switch (scat) {
     case SG_LIB_CAT_NOT_READY:
     case SG_LIB_CAT_INVALID_OP:
     case SG_LIB_CAT_ILLEGAL_REQ:
+    case SG_LIB_CAT_INVALID_PARAM:
     case SG_LIB_LBA_OUT_OF_RANGE:
     case SG_LIB_CAT_ABORTED_COMMAND:
     case SG_LIB_CAT_COPY_ABORTED:
@@ -127,6 +126,8 @@ sg_cmds_process_helper(const char * leadin, int req_din_x, int act_din_x,
     case SG_LIB_CAT_PROTECTION:
     case SG_LIB_CAT_NO_SENSE:
     case SG_LIB_CAT_MISCOMPARE:
+    case SG_LIB_CAT_STANDBY:
+    case SG_LIB_CAT_UNAVAILABLE:
         n = false;
         break;
     case SG_LIB_CAT_RECOVERED:
@@ -145,10 +146,16 @@ sg_cmds_process_helper(const char * leadin, int req_din_x, int act_din_x,
         break;
     }
     if (verbose || n) {
+        uint32_t pg_sz = sg_get_page_size();
+        char * b;
+        uint8_t *free_b;
+
+        b = (char *)sg_memalign(pg_sz, pg_sz, &free_b, false);
+        if (NULL == b)
+            return -2;
         if (leadin && (strlen(leadin) > 0))
             pr2ws("%s:\n", leadin);
-        sg_get_sense_str(NULL, sbp, slen, (verbose > 1),
-                         sizeof(b), b);
+        sg_get_sense_str(NULL, sbp, slen, (verbose > 1), pg_sz, b);
         pr2ws("%s", b);
         if (req_din_x > 0) {
             if (act_din_x != req_din_x) {
@@ -157,7 +164,7 @@ sg_cmds_process_helper(const char * leadin, int req_din_x, int act_din_x,
                 if (act_din_x < 0) {
                     if (verbose)
                         pr2ws("    %s: %s can't get negative bytes, say it "
-                              "got none\n", leadin, pass_through_s);
+                              "got none\n", leadin, pt_s);
                 }
             }
         }
@@ -168,10 +175,12 @@ sg_cmds_process_helper(const char * leadin, int req_din_x, int act_din_x,
                 if (act_dout_x < 0) {
                     if (verbose)
                         pr2ws("    %s: %s can't send negative bytes, say it "
-                              "sent none\n", leadin, pass_through_s);
+                              "sent none\n", leadin, pt_s);
                 }
             }
         }
+        if (free_b)
+            free(free_b);
     }
     if (o_sense_cat)
         *o_sense_cat = scat;
@@ -182,27 +191,27 @@ sg_cmds_process_helper(const char * leadin, int req_din_x, int act_din_x,
  * call to the pass-through. pt_res is returned from do_scsi_pt(). If valid
  * sense data is found it is decoded and output to sg_warnings_strm (def:
  * stderr); depending on the 'noisy' and 'verbose' settings. Returns -2 for
- * "sense" category (may not be fatal), -1 for failed, 0, or a positive
- * number. If din type command (or bidi) returns actual number of bytes read
- * (din_len - resid); otherwise returns 0. If -2 returned then sense category
- * output via 'o_sense_cat' pointer (if not NULL). Note that several sense
- * categories also have data in bytes received; -2 is still returned. */
+ * o_sense_cat (sense category) written which may not be fatal. Returns
+ * -1 for other types of failure. Returns 0, or a positive number. If data-in
+ * type command (or bidi) then returns actual number of bytes read
+ * (din_len - resid); otherwise returns 0. Note that several sense categories
+ * also have data in bytes received; -2 is still returned. */
 int
 sg_cmds_process_resp(struct sg_pt_base * ptvp, const char * leadin,
                      int pt_res, bool noisy, int verbose, int * o_sense_cat)
 {
-    bool transport_sense;
-    int cat, slen, resp_code, sstat, req_din_x, req_dout_x;
+    int cat, slen, sstat, req_din_x, req_dout_x;
     int act_din_x, act_dout_x;
     const uint8_t * sbp;
-    char b[1024];
+    char d[256];
+    static const int dlen = sizeof(d);
 
     if (NULL == leadin)
         leadin = "";
     if (pt_res < 0) {
 #ifdef SG_LIB_LINUX
         if (verbose)
-            pr2ws("%s: %s os error: %s\n", leadin, pass_through_s,
+            pr2ws("%s: %s os error: %s\n", leadin, pt_s,
                   safe_strerror(-pt_res));
         if ((-ENXIO == pt_res) && o_sense_cat) {
             if (verbose > 2)
@@ -210,19 +219,19 @@ sg_cmds_process_resp(struct sg_pt_base * ptvp, const char * leadin,
             *o_sense_cat = SG_LIB_CAT_NOT_READY;
             return -2;
         } else if (noisy && (0 == verbose))
-            pr2ws("%s: %s os error: %s\n", leadin, pass_through_s,
+            pr2ws("%s: %s os error: %s\n", leadin, pt_s,
                   safe_strerror(-pt_res));
 #else
         if (noisy || verbose)
-            pr2ws("%s: %s os error: %s\n", leadin, pass_through_s,
+            pr2ws("%s: %s os error: %s\n", leadin, pt_s,
                   safe_strerror(-pt_res));
 #endif
         return -1;
     } else if (SCSI_PT_DO_BAD_PARAMS == pt_res) {
-        pr2ws("%s: bad %s setup\n", leadin, pass_through_s);
+        pr2ws("%s: bad %s setup\n", leadin, pt_s);
         return -1;
     } else if (SCSI_PT_DO_TIMEOUT == pt_res) {
-        pr2ws("%s: %s timeout\n", leadin, pass_through_s);
+        pr2ws("%s: %s timeout\n", leadin, pt_s);
         return -1;
     }
     if (verbose > 2) {
@@ -231,10 +240,10 @@ sg_cmds_process_resp(struct sg_pt_base * ptvp, const char * leadin,
         if (duration > 0)
             pr2ws("      duration=%" PRIu64 " ns\n", duration);
         else {
-            int d = get_scsi_pt_duration_ms(ptvp);
+            int dur = get_scsi_pt_duration_ms(ptvp);
 
-            if (d != -1)
-                pr2ws("      duration=%u ms\n", (uint32_t)d);
+            if (dur != -1)
+                pr2ws("      duration=%u ms\n", (uint32_t)dur);
         }
     }
     get_pt_req_lengths(ptvp, &req_din_x, &req_dout_x);
@@ -244,7 +253,8 @@ sg_cmds_process_resp(struct sg_pt_base * ptvp, const char * leadin,
     switch ((cat = get_scsi_pt_result_category(ptvp))) {
     case SCSI_PT_RESULT_GOOD:
         if (sbp && (slen > 7)) {
-            resp_code = sbp[0] & 0x7f;
+            int resp_code = sbp[0] & 0x7f;
+
             /* SBC referrals can have status=GOOD and sense_key=COMPLETED */
             if (resp_code >= 0x70) {
                 if (resp_code < 0x72) {
@@ -263,7 +273,7 @@ sg_cmds_process_resp(struct sg_pt_base * ptvp, const char * leadin,
                 if (act_din_x < 0) {
                     if (verbose)
                         pr2ws("    %s: %s can't get negative bytes, say it "
-                              "got none\n", leadin, pass_through_s);
+                              "got none\n", leadin, pt_s);
                     act_din_x = 0;
                 }
             }
@@ -275,7 +285,7 @@ sg_cmds_process_resp(struct sg_pt_base * ptvp, const char * leadin,
                 if (act_dout_x < 0) {
                     if (verbose)
                         pr2ws("    %s: %s can't send negative bytes, say it "
-                              "sent none\n", leadin, pass_through_s);
+                              "sent none\n", leadin, pt_s);
                     act_dout_x = 0;
                 }
             }
@@ -308,8 +318,8 @@ sg_cmds_process_resp(struct sg_pt_base * ptvp, const char * leadin,
             }
         }
         if (verbose || noisy) {
-            sg_get_scsi_status_str(sstat, sizeof(b), b);
-            pr2ws("%s: scsi status: %s\n", leadin, b);
+            sg_get_scsi_status_str(sstat, dlen, d);
+            pr2ws("%s: scsi status: %s\n", leadin, d);
         }
         return -1;
     case SCSI_PT_RESULT_SENSE:
@@ -318,30 +328,34 @@ sg_cmds_process_resp(struct sg_pt_base * ptvp, const char * leadin,
                                       noisy, verbose, o_sense_cat);
     case SCSI_PT_RESULT_TRANSPORT_ERR:
         if (verbose || noisy) {
-            get_scsi_pt_transport_err_str(ptvp, sizeof(b), b);
-            pr2ws("%s: transport: %s\n", leadin, b);
+            get_scsi_pt_transport_err_str(ptvp, dlen, d);
+            pr2ws("%s: transport: %s\n", leadin, d);
         }
 #ifdef SG_LIB_LINUX
-        transport_sense = (slen > 0);
+        return -1;      /* DRIVER_SENSE is not passed through */
 #else
-        transport_sense = ((SAM_STAT_CHECK_CONDITION ==
-                            get_scsi_pt_status_response(ptvp)) && (slen > 0));
+        /* Shall we favour sense data over a transport error (given both) */
+        {
+            bool favour_sense = ((SAM_STAT_CHECK_CONDITION ==
+                    get_scsi_pt_status_response(ptvp)) && (slen > 0));
+
+            if (favour_sense)
+                return sg_cmds_process_helper(leadin, req_din_x, act_din_x,
+                                              req_dout_x, act_dout_x, sbp,
+                                              slen, noisy, verbose,
+                                              o_sense_cat);
+            else
+                return -1;
+        }
 #endif
-        if (transport_sense)
-            return sg_cmds_process_helper(leadin, req_din_x, act_din_x,
-                                          req_dout_x, act_dout_x, sbp, slen,
-                                          noisy, verbose, o_sense_cat);
-        else
-            return -1;
     case SCSI_PT_RESULT_OS_ERR:
         if (verbose || noisy) {
-            get_scsi_pt_os_err_str(ptvp, sizeof(b), b);
-            pr2ws("%s: os: %s\n", leadin, b);
+            get_scsi_pt_os_err_str(ptvp, dlen, d);
+            pr2ws("%s: os: %s\n", leadin, d);
         }
         return -1;
     default:
-        pr2ws("%s: unknown %s result category (%d)\n", leadin, pass_through_s,
-               cat);
+        pr2ws("%s: unknown %s result category (%d)\n", leadin, pt_s, cat);
         return -1;
     }
 }
@@ -367,13 +381,16 @@ static const char * const inquiry_s = "inquiry";
 /* Returns 0 on success, while positive values are SG_LIB_CAT_* errors
  * (e.g. SG_LIB_CAT_MALFORMED). If OS error, returns negated errno or -1. */
 static int
-sg_ll_inquiry_com(struct sg_pt_base * ptvp, bool cmddt, bool evpd, int pg_op,
-                  void * resp, int mx_resp_len, int timeout_secs,
+sg_ll_inquiry_com(struct sg_pt_base * ptvp, int sg_fd, bool cmddt, bool evpd,
+                  int pg_op, void * resp, int mx_resp_len, int timeout_secs,
                   int * residp, bool noisy, int verbose)
 {
+    bool ptvp_given = false;
+    bool local_sense = true;
+    bool local_cdb = true;
     int res, ret, sense_cat, resid;
     uint8_t inq_cdb[INQUIRY_CMDLEN] = {INQUIRY_CMD, 0, 0, 0, 0, 0};
-    uint8_t sense_b[SENSE_BUFF_LEN];
+    uint8_t sense_b[SENSE_BUFF_LEN] SG_C_CPP_ZERO_INIT;
     uint8_t * up;
 
     if (resp == NULL) {
@@ -395,7 +412,7 @@ sg_ll_inquiry_com(struct sg_pt_base * ptvp, bool cmddt, bool evpd, int pg_op,
               sg_get_command_str(inq_cdb, INQUIRY_CMDLEN, false, sizeof(b),
                                  b));
     }
-    if (resp && (mx_resp_len > 0)) {
+    if (mx_resp_len > 0) {
         up = (uint8_t *)resp;
         up[0] = 0x7f;   /* defensive prefill */
         if (mx_resp_len > 4)
@@ -403,8 +420,24 @@ sg_ll_inquiry_com(struct sg_pt_base * ptvp, bool cmddt, bool evpd, int pg_op,
     }
     if (timeout_secs <= 0)
         timeout_secs = DEF_PT_TIMEOUT;
-    set_scsi_pt_cdb(ptvp, inq_cdb, sizeof(inq_cdb));
-    set_scsi_pt_sense(ptvp, sense_b, sizeof(sense_b));
+    if (ptvp) {
+        ptvp_given = true;
+        partial_clear_scsi_pt_obj(ptvp);
+        if (get_scsi_pt_cdb_buf(ptvp))
+            local_cdb = false; /* N.B. Ignores locally built cdb */
+        else
+            set_scsi_pt_cdb(ptvp, inq_cdb, sizeof(inq_cdb));
+        if (get_scsi_pt_sense_buf(ptvp))
+            local_sense = false;
+        else
+            set_scsi_pt_sense(ptvp, sense_b, sizeof(sense_b));
+    } else {
+        ptvp = construct_scsi_pt_obj_with_fd(sg_fd, verbose);
+        if (NULL == ptvp)
+            return sg_convert_errno(ENOMEM);
+        set_scsi_pt_cdb(ptvp, inq_cdb, sizeof(inq_cdb));
+        set_scsi_pt_sense(ptvp, sense_b, sizeof(sense_b));
+    }
     set_scsi_pt_data_in(ptvp, (uint8_t *)resp, mx_resp_len);
     res = do_scsi_pt(ptvp, -1, timeout_secs, verbose);
     ret = sg_cmds_process_resp(ptvp, inquiry_s, res, noisy, verbose,
@@ -412,9 +445,12 @@ sg_ll_inquiry_com(struct sg_pt_base * ptvp, bool cmddt, bool evpd, int pg_op,
     resid = get_scsi_pt_resid(ptvp);
     if (residp)
         *residp = resid;
-    if (-1 == ret)
-        ret = sg_convert_errno(get_scsi_pt_os_err(ptvp));
-    else if (-2 == ret) {
+    if (-1 == ret) {
+        if (get_scsi_pt_transport_err(ptvp))
+            ret = SG_LIB_TRANSPORT_ERROR;
+        else
+            ret = sg_convert_errno(get_scsi_pt_os_err(ptvp));
+    } else if (-2 == ret) {
         switch (sense_cat) {
         case SG_LIB_CAT_RECOVERED:
         case SG_LIB_CAT_NO_SENSE:
@@ -435,11 +471,23 @@ sg_ll_inquiry_com(struct sg_pt_base * ptvp, bool cmddt, bool evpd, int pg_op,
         if (resid > mx_resp_len) {
             pr2ws("%s resid (%d) should never exceed requested "
                     "len=%d\n", inquiry_s, resid, mx_resp_len);
-            return ret ? ret : SG_LIB_CAT_MALFORMED;
+            if (0 == ret)
+                ret = SG_LIB_CAT_MALFORMED;
+            goto fini;
         }
         /* zero unfilled section of response buffer, based on resid */
         memset((uint8_t *)resp + (mx_resp_len - resid), 0, resid);
     }
+fini:
+    if (ptvp_given) {
+        if (local_sense)    /* stop caller trying to access local sense */
+            set_scsi_pt_sense(ptvp, NULL, 0);
+        if (local_cdb)
+            set_scsi_pt_cdb(ptvp, NULL, 0);
+    } else {
+        if (ptvp)
+            destruct_scsi_pt_obj(ptvp);
+    }
     return ret;
 }
 
@@ -450,16 +498,9 @@ int
 sg_ll_inquiry(int sg_fd, bool cmddt, bool evpd, int pg_op, void * resp,
               int mx_resp_len, bool noisy, int verbose)
 {
-    int ret;
-    struct sg_pt_base * ptvp;
-
-    ptvp = construct_scsi_pt_obj_with_fd(sg_fd, verbose);
-    if (NULL == ptvp)
-        return sg_convert_errno(ENOMEM);
-    ret = sg_ll_inquiry_com(ptvp, cmddt, evpd, pg_op, resp, mx_resp_len,
-                            0 /* timeout_sec */, NULL, noisy, verbose);
-    destruct_scsi_pt_obj(ptvp);
-    return ret;
+    return sg_ll_inquiry_com(NULL, sg_fd, cmddt, evpd, pg_op, resp,
+                             mx_resp_len, 0 /* timeout_sec */, NULL, noisy,
+                             verbose);
 }
 
 /* Invokes a SCSI INQUIRY command and yields the response. Returns 0 when
@@ -478,16 +519,9 @@ sg_ll_inquiry_v2(int sg_fd, bool evpd, int pg_op, void * resp,
                  int mx_resp_len, int timeout_secs, int * residp,
                  bool noisy, int verbose)
 {
-    int ret;
-    struct sg_pt_base * ptvp;
-
-    ptvp = construct_scsi_pt_obj_with_fd(sg_fd, verbose);
-    if (NULL == ptvp)
-        return sg_convert_errno(ENOMEM);
-    ret = sg_ll_inquiry_com(ptvp, false, evpd, pg_op, resp, mx_resp_len,
-                            timeout_secs, residp, noisy, verbose);
-    destruct_scsi_pt_obj(ptvp);
-    return ret;
+    return sg_ll_inquiry_com(NULL, sg_fd, false, evpd, pg_op, resp,
+                             mx_resp_len, timeout_secs, residp, noisy,
+                             verbose);
 }
 
 /* Similar to _v2 but takes a pointer to an object (derived from) sg_pt_base.
@@ -498,10 +532,8 @@ sg_ll_inquiry_pt(struct sg_pt_base * ptvp, bool evpd, int pg_op, void * resp,
                  int mx_resp_len, int timeout_secs, int * residp, bool noisy,
                  int verbose)
 {
-    clear_scsi_pt_obj(ptvp);
-    return sg_ll_inquiry_com(ptvp, false, evpd, pg_op, resp, mx_resp_len,
+    return sg_ll_inquiry_com(ptvp, -1, false, evpd, pg_op, resp, mx_resp_len,
                              timeout_secs, residp, noisy, verbose);
-
 }
 
 /* Yields most of first 36 bytes of a standard INQUIRY (evpd==0) response.
@@ -518,19 +550,19 @@ sg_simple_inquiry(int sg_fd, struct sg_simple_inquiry_resp * inq_data,
     if (inq_data) {
         memset(inq_data, 0, sizeof(* inq_data));
         inq_data->peripheral_qualifier = 0x3;
-        inq_data->peripheral_type = 0x1f;
+        inq_data->peripheral_type = PDT_UNKNOWN;
     }
     inq_resp = sg_memalign(SAFE_STD_INQ_RESP_LEN, 0, &free_irp, false);
     if (NULL == inq_resp) {
         pr2ws("%s: out of memory\n", __func__);
         return sg_convert_errno(ENOMEM);
     }
-    ret = sg_ll_inquiry_v2(sg_fd, false, 0, inq_resp, SAFE_STD_INQ_RESP_LEN,
-                           0, NULL, noisy, verbose);
+    ret = sg_ll_inquiry_com(NULL, sg_fd, false, false, 0, inq_resp,
+                            SAFE_STD_INQ_RESP_LEN, 0, NULL, noisy, verbose);
 
     if (inq_data && (0 == ret)) {
         inq_data->peripheral_qualifier = (inq_resp[0] >> 5) & 0x7;
-        inq_data->peripheral_type = inq_resp[0] & 0x1f;
+        inq_data->peripheral_type = inq_resp[0] & PDT_MASK;
         inq_data->byte_1 = inq_resp[1];
         inq_data->version = inq_resp[2];
         inq_data->byte_3 = inq_resp[3];
@@ -560,19 +592,19 @@ sg_simple_inquiry_pt(struct sg_pt_base * ptvp,
     if (inq_data) {
         memset(inq_data, 0, sizeof(* inq_data));
         inq_data->peripheral_qualifier = 0x3;
-        inq_data->peripheral_type = 0x1f;
+        inq_data->peripheral_type = PDT_MASK;
     }
     inq_resp = sg_memalign(SAFE_STD_INQ_RESP_LEN, 0, &free_irp, false);
     if (NULL == inq_resp) {
         pr2ws("%s: out of memory\n", __func__);
         return sg_convert_errno(ENOMEM);
     }
-    ret = sg_ll_inquiry_pt(ptvp, false, 0, inq_resp, SAFE_STD_INQ_RESP_LEN,
-                           0, NULL, noisy, verbose);
+    ret = sg_ll_inquiry_com(ptvp, -1, false, false, 0, inq_resp,
+                            SAFE_STD_INQ_RESP_LEN, 0, NULL, noisy, verbose);
 
     if (inq_data && (0 == ret)) {
         inq_data->peripheral_qualifier = (inq_resp[0] >> 5) & 0x7;
-        inq_data->peripheral_type = inq_resp[0] & 0x1f;
+        inq_data->peripheral_type = inq_resp[0] & PDT_MASK;
         inq_data->byte_1 = inq_resp[1];
         inq_data->version = inq_resp[2];
         inq_data->byte_3 = inq_resp[3];
@@ -589,19 +621,24 @@ sg_simple_inquiry_pt(struct sg_pt_base * ptvp,
 }
 
 /* Invokes a SCSI TEST UNIT READY command.
+ * N.B. To access the sense buffer outside this routine then one be
+ * provided by the caller.
  * 'pack_id' is just for diagnostics, safe to set to 0.
  * Looks for progress indicator if 'progress' non-NULL;
  * if found writes value [0..65535] else write -1.
  * Returns 0 when successful, various SG_LIB_CAT_* positive values or
  * -1 -> other errors */
-int
-sg_ll_test_unit_ready_progress_pt(struct sg_pt_base * ptvp, int pack_id,
-                                  int * progress, bool noisy, int verbose)
+static int
+sg_ll_test_unit_ready_com(struct sg_pt_base * ptvp, int sg_fd, int pack_id,
+                          int * progress, bool noisy, int verbose)
 {
     static const char * const tur_s = "test unit ready";
+    bool ptvp_given = false;
+    bool local_sense = true;
+    bool local_cdb = true;
     int res, ret, sense_cat;
     uint8_t tur_cdb[TUR_CMDLEN] = {TUR_CMD, 0, 0, 0, 0, 0};
-    uint8_t sense_b[SENSE_BUFF_LEN];
+    uint8_t sense_b[SENSE_BUFF_LEN] SG_C_CPP_ZERO_INIT;
 
     if (verbose) {
         char b[128];
@@ -609,16 +646,33 @@ sg_ll_test_unit_ready_progress_pt(struct sg_pt_base * ptvp, int pack_id,
         pr2ws("    %s cdb: %s\n", tur_s,
               sg_get_command_str(tur_cdb, TUR_CMDLEN, false, sizeof(b), b));
     }
-
-    clear_scsi_pt_obj(ptvp);
-    set_scsi_pt_cdb(ptvp, tur_cdb, sizeof(tur_cdb));
-    set_scsi_pt_sense(ptvp, sense_b, sizeof(sense_b));
+    if (ptvp) {
+        ptvp_given = true;
+        partial_clear_scsi_pt_obj(ptvp);
+        if (get_scsi_pt_cdb_buf(ptvp))
+            local_cdb = false; /* N.B. Ignores locally built cdb */
+        else
+            set_scsi_pt_cdb(ptvp, tur_cdb, sizeof(tur_cdb));
+        if (get_scsi_pt_sense_buf(ptvp))
+            local_sense = false;
+        else
+            set_scsi_pt_sense(ptvp, sense_b, sizeof(sense_b));
+    } else {
+        ptvp = construct_scsi_pt_obj_with_fd(sg_fd, verbose);
+        if (NULL == ptvp)
+            return sg_convert_errno(ENOMEM);
+        set_scsi_pt_cdb(ptvp, tur_cdb, sizeof(tur_cdb));
+        set_scsi_pt_sense(ptvp, sense_b, sizeof(sense_b));
+    }
     set_scsi_pt_packet_id(ptvp, pack_id);
     res = do_scsi_pt(ptvp, -1, DEF_PT_TIMEOUT, verbose);
     ret = sg_cmds_process_resp(ptvp, tur_s, res, noisy, verbose, &sense_cat);
-    if (-1 == ret)
-        ret = sg_convert_errno(get_scsi_pt_os_err(ptvp));
-    else if (-2 == ret) {
+    if (-1 == ret) {
+        if (get_scsi_pt_transport_err(ptvp))
+            ret = SG_LIB_TRANSPORT_ERROR;
+        else
+            ret = sg_convert_errno(get_scsi_pt_os_err(ptvp));
+    } else if (-2 == ret) {
         if (progress) {
             int slen = get_scsi_pt_sense_len(ptvp);
 
@@ -636,23 +690,32 @@ sg_ll_test_unit_ready_progress_pt(struct sg_pt_base * ptvp, int pack_id,
         }
     } else
         ret = 0;
+    if (ptvp_given) {
+        if (local_sense)    /* stop caller trying to access local sense */
+            set_scsi_pt_sense(ptvp, NULL, 0);
+        if (local_cdb)
+            set_scsi_pt_cdb(ptvp, NULL, 0);
+    } else {
+        if (ptvp)
+            destruct_scsi_pt_obj(ptvp);
+    }
     return ret;
 }
 
+int
+sg_ll_test_unit_ready_progress_pt(struct sg_pt_base * ptvp, int pack_id,
+                                  int * progress, bool noisy, int verbose)
+{
+    return sg_ll_test_unit_ready_com(ptvp, -1, pack_id, progress, noisy,
+                                     verbose);
+}
+
 int
 sg_ll_test_unit_ready_progress(int sg_fd, int pack_id, int * progress,
                                bool noisy, int verbose)
 {
-    int ret;
-    struct sg_pt_base * ptvp;
-
-    ptvp = construct_scsi_pt_obj_with_fd(sg_fd, verbose);
-    if (NULL == ptvp)
-        return sg_convert_errno(ENOMEM);
-    ret = sg_ll_test_unit_ready_progress_pt(ptvp, pack_id, progress, noisy,
-                                            verbose);
-    destruct_scsi_pt_obj(ptvp);
-    return ret;
+    return sg_ll_test_unit_ready_com(NULL, sg_fd, pack_id, progress, noisy,
+                                     verbose);
 }
 
 /* Invokes a SCSI TEST UNIT READY command.
@@ -662,24 +725,15 @@ sg_ll_test_unit_ready_progress(int sg_fd, int pack_id, int * progress,
 int
 sg_ll_test_unit_ready(int sg_fd, int pack_id, bool noisy, int verbose)
 {
-    int ret;
-    struct sg_pt_base * ptvp;
-
-    ptvp = construct_scsi_pt_obj_with_fd(sg_fd, verbose);
-    if (NULL == ptvp)
-        return sg_convert_errno(ENOMEM);
-    ret = sg_ll_test_unit_ready_progress_pt(ptvp, pack_id, NULL, noisy,
-                                            verbose);
-    destruct_scsi_pt_obj(ptvp);
-    return ret;
+    return sg_ll_test_unit_ready_com(NULL, sg_fd, pack_id, NULL, noisy,
+                                     verbose);
 }
 
 int
 sg_ll_test_unit_ready_pt(struct sg_pt_base * ptvp, int pack_id, bool noisy,
                          int verbose)
 {
-    return sg_ll_test_unit_ready_progress_pt(ptvp, pack_id, NULL, noisy,
-                                             verbose);
+    return sg_ll_test_unit_ready_com(ptvp, -1, pack_id, NULL, noisy, verbose);
 }
 
 /* Invokes a SCSI REQUEST SENSE command. Returns 0 when successful, various
@@ -689,11 +743,13 @@ sg_ll_request_sense_com(struct sg_pt_base * ptvp, int sg_fd, bool desc,
                         void * resp, int mx_resp_len, bool noisy, int verbose)
 {
     bool ptvp_given = false;
+    bool local_cdb = true;
+    bool local_sense = true;
     int ret, res, sense_cat;
     static const char * const rq_s = "request sense";
     uint8_t rs_cdb[REQUEST_SENSE_CMDLEN] =
         {REQUEST_SENSE_CMD, 0, 0, 0, 0, 0};
-    uint8_t sense_b[SENSE_BUFF_LEN];
+    uint8_t sense_b[SENSE_BUFF_LEN] SG_C_CPP_ZERO_INIT;
 
     if (desc)
         rs_cdb[1] |= 0x1;
@@ -709,21 +765,32 @@ sg_ll_request_sense_com(struct sg_pt_base * ptvp, int sg_fd, bool desc,
               sg_get_command_str(rs_cdb, REQUEST_SENSE_CMDLEN, false,
                                  sizeof(b), b));
     }
-    if (ptvp)
+    if (ptvp) {
         ptvp_given = true;
-    else {
+        if (get_scsi_pt_cdb_buf(ptvp))
+            local_cdb = false;
+        else
+            set_scsi_pt_cdb(ptvp, rs_cdb, sizeof(rs_cdb));
+        if (get_scsi_pt_sense_buf(ptvp))
+            local_sense = false;
+        else
+            set_scsi_pt_sense(ptvp, sense_b, sizeof(sense_b));
+    } else {
         ptvp = construct_scsi_pt_obj_with_fd(sg_fd, verbose);
         if (NULL == ptvp)
             return sg_convert_errno(ENOMEM);
+        set_scsi_pt_cdb(ptvp, rs_cdb, sizeof(rs_cdb));
+        set_scsi_pt_sense(ptvp, sense_b, sizeof(sense_b));
     }
-    set_scsi_pt_cdb(ptvp, rs_cdb, sizeof(rs_cdb));
-    set_scsi_pt_sense(ptvp, sense_b, sizeof(sense_b));
     set_scsi_pt_data_in(ptvp, (uint8_t *)resp, mx_resp_len);
     res = do_scsi_pt(ptvp, -1, DEF_PT_TIMEOUT, verbose);
     ret = sg_cmds_process_resp(ptvp, rq_s, res, noisy, verbose, &sense_cat);
-    if (-1 == ret)
-        ret = sg_convert_errno(get_scsi_pt_os_err(ptvp));
-    else if (-2 == ret) {
+    if (-1 == ret) {
+        if (get_scsi_pt_transport_err(ptvp))
+            ret = SG_LIB_TRANSPORT_ERROR;
+        else
+            ret = sg_convert_errno(get_scsi_pt_os_err(ptvp));
+    } else if (-2 == ret) {
         switch (sense_cat) {
         case SG_LIB_CAT_RECOVERED:
         case SG_LIB_CAT_NO_SENSE:
@@ -742,7 +809,12 @@ sg_ll_request_sense_com(struct sg_pt_base * ptvp, int sg_fd, bool desc,
         } else
             ret = 0;
     }
-    if ((! ptvp_given) && ptvp)
+    if (ptvp_given) {
+        if (local_sense)        /* stop caller accessing local sense */
+        set_scsi_pt_sense(ptvp, NULL, 0);
+        if (local_cdb)  /* stop caller accessing local sense */
+        set_scsi_pt_cdb(ptvp, NULL, 0);
+    } else if (ptvp)
         destruct_scsi_pt_obj(ptvp);
     return ret;
 }
@@ -759,7 +831,6 @@ int
 sg_ll_request_sense_pt(struct sg_pt_base * ptvp, bool desc, void * resp,
                        int mx_resp_len, bool noisy, int verbose)
 {
-    clear_scsi_pt_obj(ptvp);
     return sg_ll_request_sense_com(ptvp, -1, desc, resp, mx_resp_len,
                                    noisy, verbose);
 }
@@ -772,10 +843,12 @@ sg_ll_report_luns_com(struct sg_pt_base * ptvp, int sg_fd, int select_report,
 {
     static const char * const report_luns_s = "report luns";
     bool ptvp_given = false;
+    bool local_cdb = true;
+    bool local_sense = true;
     int ret, res, sense_cat;
     uint8_t rl_cdb[REPORT_LUNS_CMDLEN] =
                          {REPORT_LUNS_CMD, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0};
-    uint8_t sense_b[SENSE_BUFF_LEN];
+    uint8_t sense_b[SENSE_BUFF_LEN] SG_C_CPP_ZERO_INIT;
 
     rl_cdb[2] = select_report & 0xff;
     sg_put_unaligned_be32((uint32_t)mx_resp_len, rl_cdb + 6);
@@ -786,19 +859,33 @@ sg_ll_report_luns_com(struct sg_pt_base * ptvp, int sg_fd, int select_report,
               sg_get_command_str(rl_cdb, REPORT_LUNS_CMDLEN, false,
                                  sizeof(b), b));
     }
-    if (ptvp)
+    if (ptvp) {
         ptvp_given = true;
-    else if (NULL == ((ptvp = create_pt_obj(report_luns_s))))
-        return sg_convert_errno(ENOMEM);
-    set_scsi_pt_cdb(ptvp, rl_cdb, sizeof(rl_cdb));
-    set_scsi_pt_sense(ptvp, sense_b, sizeof(sense_b));
+        partial_clear_scsi_pt_obj(ptvp);
+        if (get_scsi_pt_cdb_buf(ptvp))
+            local_cdb = false;
+        else
+            set_scsi_pt_cdb(ptvp, rl_cdb, sizeof(rl_cdb));
+        if (get_scsi_pt_sense_buf(ptvp))
+            local_sense = false;
+        else
+            set_scsi_pt_sense(ptvp, sense_b, sizeof(sense_b));
+    } else {
+        if (NULL == ((ptvp = create_pt_obj(report_luns_s))))
+            return sg_convert_errno(ENOMEM);
+        set_scsi_pt_cdb(ptvp, rl_cdb, sizeof(rl_cdb));
+        set_scsi_pt_sense(ptvp, sense_b, sizeof(sense_b));
+    }
     set_scsi_pt_data_in(ptvp, (uint8_t *)resp, mx_resp_len);
     res = do_scsi_pt(ptvp, sg_fd, DEF_PT_TIMEOUT, verbose);
     ret = sg_cmds_process_resp(ptvp, report_luns_s, res, noisy, verbose,
                                &sense_cat);
-    if (-1 == ret)
-        ret = sg_convert_errno(get_scsi_pt_os_err(ptvp));
-    else if (-2 == ret) {
+    if (-1 == ret) {
+        if (get_scsi_pt_transport_err(ptvp))
+            ret = SG_LIB_TRANSPORT_ERROR;
+        else
+            ret = sg_convert_errno(get_scsi_pt_os_err(ptvp));
+    } else if (-2 == ret) {
         switch (sense_cat) {
         case SG_LIB_CAT_RECOVERED:
         case SG_LIB_CAT_NO_SENSE:
@@ -810,8 +897,15 @@ sg_ll_report_luns_com(struct sg_pt_base * ptvp, int sg_fd, int select_report,
         }
     } else
         ret = 0;
-    if ((! ptvp_given) && ptvp)
-        destruct_scsi_pt_obj(ptvp);
+    if (ptvp_given) {
+        if (local_sense)        /* stop caller accessing local sense */
+            set_scsi_pt_sense(ptvp, NULL, 0);
+        if (local_cdb)
+            set_scsi_pt_cdb(ptvp, NULL, 0);
+    } else {
+        if (ptvp)
+            destruct_scsi_pt_obj(ptvp);
+    }
     return ret;
 }
 
@@ -834,7 +928,6 @@ int
 sg_ll_report_luns_pt(struct sg_pt_base * ptvp, int select_report,
                      void * resp, int mx_resp_len, bool noisy, int verbose)
 {
-    clear_scsi_pt_obj(ptvp);
     return sg_ll_report_luns_com(ptvp, -1, select_report, resp,
                                  mx_resp_len, noisy, verbose);
 }
