@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1999-2020 Douglas Gilbert.
+ * Copyright (c) 1999-2023 Douglas Gilbert.
  * All rights reserved.
  * Use of this source code is governed by a BSD-style
  * license that can be found in the BSD_LICENSE file.
@@ -68,7 +68,7 @@
 
 #define MODE6_RESP_HDR_LEN 4
 #define MODE10_RESP_HDR_LEN 8
-#define MODE_RESP_ARB_LEN 1024
+#define MODE_RESP_ARB_LEN 8192
 
 #define INQUIRY_RESP_INITIAL_LEN 36
 
@@ -93,7 +93,7 @@ sg_ll_sync_cache_10(int sg_fd, bool sync_nv, bool immed, int group,
     int res, ret, sense_cat;
     uint8_t sc_cdb[SYNCHRONIZE_CACHE_CMDLEN] =
                 {SYNCHRONIZE_CACHE_CMD, 0, 0, 0, 0, 0, 0, 0, 0, 0};
-    uint8_t sense_b[SENSE_BUFF_LEN];
+    uint8_t sense_b[SENSE_BUFF_LEN] SG_C_CPP_ZERO_INIT;
     struct sg_pt_base * ptvp;
 
     if (sync_nv)
@@ -101,7 +101,7 @@ sg_ll_sync_cache_10(int sg_fd, bool sync_nv, bool immed, int group,
     if (immed)
         sc_cdb[1] |= 2;
     sg_put_unaligned_be32((uint32_t)lba, sc_cdb + 2);
-    sc_cdb[6] = group & 0x1f;
+    sc_cdb[6] = group & GRPNUM_MASK;
     if (count > 0xffff) {
         pr2ws("count too big\n");
         return -1;
@@ -121,9 +121,12 @@ sg_ll_sync_cache_10(int sg_fd, bool sync_nv, bool immed, int group,
     set_scsi_pt_sense(ptvp, sense_b, sizeof(sense_b));
     res = do_scsi_pt(ptvp, sg_fd, DEF_PT_TIMEOUT, verbose);
     ret = sg_cmds_process_resp(ptvp, cdb_s, res, noisy, verbose, &sense_cat);
-    if (-1 == ret)
-        ret = sg_convert_errno(get_scsi_pt_os_err(ptvp));
-    else if (-2 == ret) {
+    if (-1 == ret) {
+        if (get_scsi_pt_transport_err(ptvp))
+            ret = SG_LIB_TRANSPORT_ERROR;
+        else
+            ret = sg_convert_errno(get_scsi_pt_os_err(ptvp));
+    } else if (-2 == ret) {
         switch (sense_cat) {
         case SG_LIB_CAT_RECOVERED:
         case SG_LIB_CAT_NO_SENSE:
@@ -151,7 +154,7 @@ sg_ll_readcap_16(int sg_fd, bool pmi, uint64_t llba, void * resp,
     uint8_t rc_cdb[SERVICE_ACTION_IN_16_CMDLEN] =
                         {SERVICE_ACTION_IN_16_CMD, READ_CAPACITY_16_SA,
                          0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0};
-    uint8_t sense_b[SENSE_BUFF_LEN];
+    uint8_t sense_b[SENSE_BUFF_LEN] SG_C_CPP_ZERO_INIT;
     struct sg_pt_base * ptvp;
 
     if (pmi) { /* lbs only valid when pmi set */
@@ -174,9 +177,12 @@ sg_ll_readcap_16(int sg_fd, bool pmi, uint64_t llba, void * resp,
     set_scsi_pt_data_in(ptvp, (uint8_t *)resp, mx_resp_len);
     res = do_scsi_pt(ptvp, sg_fd, DEF_PT_TIMEOUT, verbose);
     ret = sg_cmds_process_resp(ptvp, cdb_s, res, noisy, verbose, &sense_cat);
-    if (-1 == ret)
-        ret = sg_convert_errno(get_scsi_pt_os_err(ptvp));
-    else if (-2 == ret) {
+    if (-1 == ret) {
+        if (get_scsi_pt_transport_err(ptvp))
+            ret = SG_LIB_TRANSPORT_ERROR;
+        else
+            ret = sg_convert_errno(get_scsi_pt_os_err(ptvp));
+    } else if (-2 == ret) {
         switch (sense_cat) {
         case SG_LIB_CAT_RECOVERED:
         case SG_LIB_CAT_NO_SENSE:
@@ -203,7 +209,7 @@ sg_ll_readcap_10(int sg_fd, bool pmi, unsigned int lba, void * resp,
     int ret, res, sense_cat;
     uint8_t rc_cdb[READ_CAPACITY_10_CMDLEN] =
                          {READ_CAPACITY_10_CMD, 0, 0, 0, 0, 0, 0, 0, 0, 0};
-    uint8_t sense_b[SENSE_BUFF_LEN];
+    uint8_t sense_b[SENSE_BUFF_LEN] SG_C_CPP_ZERO_INIT;
     struct sg_pt_base * ptvp;
 
     if (pmi) { /* lbs only valid when pmi set */
@@ -224,9 +230,12 @@ sg_ll_readcap_10(int sg_fd, bool pmi, unsigned int lba, void * resp,
     set_scsi_pt_data_in(ptvp, (uint8_t *)resp, mx_resp_len);
     res = do_scsi_pt(ptvp, sg_fd, DEF_PT_TIMEOUT, verbose);
     ret = sg_cmds_process_resp(ptvp, cdb_s, res, noisy, verbose, &sense_cat);
-    if (-1 == ret)
-        ret = sg_convert_errno(get_scsi_pt_os_err(ptvp));
-    else if (-2 == ret) {
+    if (-1 == ret) {
+        if (get_scsi_pt_transport_err(ptvp))
+            ret = SG_LIB_TRANSPORT_ERROR;
+        else
+            ret = sg_convert_errno(get_scsi_pt_os_err(ptvp));
+    } else if (-2 == ret) {
         switch (sense_cat) {
         case SG_LIB_CAT_RECOVERED:
         case SG_LIB_CAT_NO_SENSE:
@@ -253,7 +262,7 @@ sg_ll_mode_sense6(int sg_fd, bool dbd, int pc, int pg_code, int sub_pg_code,
     int res, ret, sense_cat, resid;
     uint8_t modes_cdb[MODE_SENSE6_CMDLEN] =
         {MODE_SENSE6_CMD, 0, 0, 0, 0, 0};
-    uint8_t sense_b[SENSE_BUFF_LEN];
+    uint8_t sense_b[SENSE_BUFF_LEN] SG_C_CPP_ZERO_INIT;
     struct sg_pt_base * ptvp;
 
     modes_cdb[1] = (uint8_t)(dbd ? 0x8 : 0);
@@ -279,9 +288,12 @@ sg_ll_mode_sense6(int sg_fd, bool dbd, int pc, int pg_code, int sub_pg_code,
     res = do_scsi_pt(ptvp, sg_fd, DEF_PT_TIMEOUT, verbose);
     ret = sg_cmds_process_resp(ptvp, cdb_s, res, noisy, verbose, &sense_cat);
     resid = get_scsi_pt_resid(ptvp);
-    if (-1 == ret)
-        ret = sg_convert_errno(get_scsi_pt_os_err(ptvp));
-    else if (-2 == ret) {
+    if (-1 == ret) {
+        if (get_scsi_pt_transport_err(ptvp))
+            ret = SG_LIB_TRANSPORT_ERROR;
+        else
+            ret = sg_convert_errno(get_scsi_pt_os_err(ptvp));
+    } else if (-2 == ret) {
         switch (sense_cat) {
         case SG_LIB_CAT_RECOVERED:
         case SG_LIB_CAT_NO_SENSE:
@@ -349,7 +361,7 @@ sg_ll_mode_sense10_v2(int sg_fd, bool llbaa, bool dbd, int pc, int pg_code,
     struct sg_pt_base * ptvp;
     uint8_t modes_cdb[MODE_SENSE10_CMDLEN] =
         {MODE_SENSE10_CMD, 0, 0, 0, 0, 0, 0, 0, 0, 0};
-    uint8_t sense_b[SENSE_BUFF_LEN];
+    uint8_t sense_b[SENSE_BUFF_LEN] SG_C_CPP_ZERO_INIT;
 
     modes_cdb[1] = (uint8_t)((dbd ? 0x8 : 0) | (llbaa ? 0x10 : 0));
     modes_cdb[2] = (uint8_t)(((pc << 6) & 0xc0) | (pg_code & 0x3f));
@@ -379,9 +391,12 @@ sg_ll_mode_sense10_v2(int sg_fd, bool llbaa, bool dbd, int pc, int pg_code,
     resid = get_scsi_pt_resid(ptvp);
     if (residp)
         *residp = resid;
-    if (-1 == ret)
-        ret = sg_convert_errno(get_scsi_pt_os_err(ptvp));
-    else if (-2 == ret) {
+    if (-1 == ret) {
+        if (get_scsi_pt_transport_err(ptvp))
+            ret = SG_LIB_TRANSPORT_ERROR;
+        else
+            ret = sg_convert_errno(get_scsi_pt_os_err(ptvp));
+    } else if (-2 == ret) {
         switch (sense_cat) {
         case SG_LIB_CAT_RECOVERED:
         case SG_LIB_CAT_NO_SENSE:
@@ -433,7 +448,7 @@ sg_ll_mode_select6_v2(int sg_fd, bool pf, bool rtd, bool sp, void * paramp,
     int res, ret, sense_cat;
     uint8_t modes_cdb[MODE_SELECT6_CMDLEN] =
         {MODE_SELECT6_CMD, 0, 0, 0, 0, 0};
-    uint8_t sense_b[SENSE_BUFF_LEN];
+    uint8_t sense_b[SENSE_BUFF_LEN] SG_C_CPP_ZERO_INIT;
     struct sg_pt_base * ptvp;
 
     modes_cdb[1] = (uint8_t)((pf ? 0x10 : 0x0) | (sp ? 0x1 : 0x0));
@@ -463,9 +478,12 @@ sg_ll_mode_select6_v2(int sg_fd, bool pf, bool rtd, bool sp, void * paramp,
     set_scsi_pt_data_out(ptvp, (uint8_t *)paramp, param_len);
     res = do_scsi_pt(ptvp, sg_fd, DEF_PT_TIMEOUT, verbose);
     ret = sg_cmds_process_resp(ptvp, cdb_s, res, noisy, verbose, &sense_cat);
-    if (-1 == ret)
-        ret = sg_convert_errno(get_scsi_pt_os_err(ptvp));
-    else if (-2 == ret) {
+    if (-1 == ret) {
+        if (get_scsi_pt_transport_err(ptvp))
+            ret = SG_LIB_TRANSPORT_ERROR;
+        else
+            ret = sg_convert_errno(get_scsi_pt_os_err(ptvp));
+    } else if (-2 == ret) {
         switch (sense_cat) {
         case SG_LIB_CAT_RECOVERED:
         case SG_LIB_CAT_NO_SENSE:
@@ -501,7 +519,7 @@ sg_ll_mode_select10_v2(int sg_fd, bool pf, bool rtd, bool sp, void * paramp,
     int res, ret, sense_cat;
     uint8_t modes_cdb[MODE_SELECT10_CMDLEN] =
         {MODE_SELECT10_CMD, 0, 0, 0, 0, 0, 0, 0, 0, 0};
-    uint8_t sense_b[SENSE_BUFF_LEN];
+    uint8_t sense_b[SENSE_BUFF_LEN] SG_C_CPP_ZERO_INIT;
     struct sg_pt_base * ptvp;
 
     modes_cdb[1] = (uint8_t)((pf ? 0x10 : 0x0) | (sp ? 0x1 : 0x0));
@@ -531,9 +549,12 @@ sg_ll_mode_select10_v2(int sg_fd, bool pf, bool rtd, bool sp, void * paramp,
     set_scsi_pt_data_out(ptvp, (uint8_t *)paramp, param_len);
     res = do_scsi_pt(ptvp, sg_fd, DEF_PT_TIMEOUT, verbose);
     ret = sg_cmds_process_resp(ptvp, cdb_s, res, noisy, verbose, &sense_cat);
-    if (-1 == ret)
-        ret = sg_convert_errno(get_scsi_pt_os_err(ptvp));
-    else if (-2 == ret) {
+    if (-1 == ret) {
+        if (get_scsi_pt_transport_err(ptvp))
+            ret = SG_LIB_TRANSPORT_ERROR;
+        else
+            ret = sg_convert_errno(get_scsi_pt_os_err(ptvp));
+    } else if (-2 == ret) {
         switch (sense_cat) {
         case SG_LIB_CAT_RECOVERED:
         case SG_LIB_CAT_NO_SENSE:
@@ -635,21 +656,24 @@ an_err:
     return -1;
 }
 
-/* Fetches current, changeable, default and/or saveable modes pages as
- * indicated by pcontrol_arr for given pg_code and sub_pg_code. If
- * mode6==false then use MODE SENSE (10) else use MODE SENSE (6). If
- * flexible set and mode data length seems wrong then try and
- * fix (compensating hack for bad device or driver). pcontrol_arr
- * should have 4 elements for output of current, changeable, default
- * and saved values respectively. Each element should be NULL or
- * at least mx_mpage_len bytes long.
- * Return of 0 -> overall success, various SG_LIB_CAT_* positive values or
- * -1 -> other errors.
+/* Fetches current, changeable, default and/or saveable mode pages (note:
+ * _only_ mode pages) as indicated by pcontrol_arr for given pg_code and
+ * sub_pg_code. If mode6 is true then use MODE SENSE (6) else use MODE SENSE
+ * (10). If flexible true and mode data length seems wrong then try and fix
+ * (compensating hack for bad device or driver). pcontrol_arr should have 4
+ * elements for output of current, changeable, default and saved values
+ * respectively. Each element should be NULL or a pointer to memory that is
+ * at least mx_mpage_len bytes long. The length of the fetched mode page
+ * (or pages) is written to the reported_lenp pointer, if it is non-NULL.
+ * That should be the length written to each pointer within pcontrol_arr .
  * If success_mask pointer is not NULL then first zeros it. Then set bits
  * 0, 1, 2 and/or 3 if the current, changeable, default and saved values
- * respectively have been fetched. If error on current page
- * then stops and returns that error; otherwise continues if an error is
- * detected but returns the first error encountered.  */
+ * respectively have been fetched. Returns 0 if overall success, otherwise
+ * a SG_LIB_CAT type error is returned or -1 for an uncategorized error.
+ * If error on current page then stops and returns that error; otherwise
+ * continues if an error is detected, returning the first error encountered.
+ * So if the saved page control is not supported, for example, then 7 is
+ * written to *smask and SG_LIB_CAT_ILLEGAL_REQ is returned. */
 int
 sg_get_mode_page_controls(int sg_fd, bool mode6, int pg_code, int sub_pg_code,
                           bool dbd, bool flexible, int mx_mpage_len,
@@ -660,7 +684,8 @@ sg_get_mode_page_controls(int sg_fd, bool mode6, int pg_code, int sub_pg_code,
     int k, n, res, offset, calc_len, xfer_len;
     int resid = 0;
     const int msense10_hlen = MODE10_RESP_HDR_LEN;
-    uint8_t buff[MODE_RESP_ARB_LEN];
+    uint8_t * buffp;
+    uint8_t * free_buffp;
     char ebuff[EBUFF_SZ];
     int first_err = 0;
 
@@ -670,27 +695,32 @@ sg_get_mode_page_controls(int sg_fd, bool mode6, int pg_code, int sub_pg_code,
         *reported_lenp = 0;
     if (mx_mpage_len < 4)
         return 0;
+    buffp = sg_memalign(MODE_RESP_ARB_LEN, 0, &free_buffp, false);
+    if (NULL == buffp)
+        return sg_convert_errno(ENOMEM);
+
     memset(ebuff, 0, sizeof(ebuff));
     /* first try to find length of current page response */
-    memset(buff, 0, msense10_hlen);
+    memset(buffp, 0, msense10_hlen);
     if (mode6)  /* want first 8 bytes just in case */
         res = sg_ll_mode_sense6(sg_fd, dbd, 0 /* pc */, pg_code,
-                                sub_pg_code, buff, msense10_hlen, true,
+                                sub_pg_code, buffp, msense10_hlen, true,
                                 verbose);
     else        /* MODE SENSE(10) obviously */
         res = sg_ll_mode_sense10_v2(sg_fd, false /* llbaa */, dbd,
-                                    0 /* pc */, pg_code, sub_pg_code, buff,
-                                    msense10_hlen, 0, &resid, true, verbose);
-    if (0 != res)
-        return res;
-    n = buff[0];
+                                    0 /* pc */, pg_code, sub_pg_code, buffp,
+                                    msense10_hlen, 0, NULL, true, verbose);
+    if (0 != res) {
+        first_err = res;
+        goto fini;
+    }
+    n = buffp[0];
     if (reported_lenp) {
-        int m;
+        int bd_len, non_mps;
+        int m = sg_msense_calc_length(buffp, msense10_hlen, mode6, &bd_len);
 
-        m = sg_msense_calc_length(buff, msense10_hlen, mode6, NULL) - resid;
-        if (m < 0)      /* Grrr, this should not happen */
-            m = 0;
-        *reported_lenp = m;
+        non_mps = bd_len + (mode6 ? 4 : 8);
+        *reported_lenp = (m > non_mps) ? m - non_mps : 0;
     }
     resp_mode6 = mode6;
     if (flexible) {
@@ -701,10 +731,10 @@ sg_get_mode_page_controls(int sg_fd, bool mode6, int pg_code, int sub_pg_code,
                       "response processing\n", n);
         }
         if ((! mode6) && (n > 5)) {
-            if ((n > 11) && (0 == (n % 2)) && (0 == buff[4]) &&
-                (0 == buff[5]) && (0 == buff[6])) {
-                buff[1] = n;
-                buff[0] = 0;
+            if ((n > 11) && (0 == (n % 2)) && (0 == buffp[4]) &&
+                (0 == buffp[5]) && (0 == buffp[6])) {
+                buffp[1] = n;
+                buffp[0] = 0;
                 if (verbose)
                     pr2ws(">>> msense(10) but resp[0]=%d and not msense(6) "
                           "response so fix length\n", n);
@@ -714,15 +744,16 @@ sg_get_mode_page_controls(int sg_fd, bool mode6, int pg_code, int sub_pg_code,
     }
     if (verbose && (resp_mode6 != mode6))
         pr2ws(">>> msense(%d) but resp[0]=%d so switch response "
-              "processing\n", (mode6 ? 6 : 10), buff[0]);
-    calc_len = sg_msense_calc_length(buff, msense10_hlen, resp_mode6, NULL);
+              "processing\n", (mode6 ? 6 : 10), buffp[0]);
+    calc_len = sg_msense_calc_length(buffp, msense10_hlen, resp_mode6, NULL);
     if (calc_len > MODE_RESP_ARB_LEN)
         calc_len = MODE_RESP_ARB_LEN;
-    offset = sg_mode_page_offset(buff, calc_len, resp_mode6, ebuff, EBUFF_SZ);
+    offset = sg_mode_page_offset(buffp, calc_len, resp_mode6, ebuff, EBUFF_SZ);
     if (offset < 0) {
         if (('\0' != ebuff[0]) && (verbose > 0))
             pr2ws("%s: %s\n", __func__, ebuff);
-        return SG_LIB_CAT_MALFORMED;
+        first_err = SG_LIB_CAT_MALFORMED;
+        goto fini;
     }
     xfer_len = calc_len - offset;
     if (xfer_len > mx_mpage_len)
@@ -735,12 +766,12 @@ sg_get_mode_page_controls(int sg_fd, bool mode6, int pg_code, int sub_pg_code,
         resid = 0;
         if (mode6)
             res = sg_ll_mode_sense6(sg_fd, dbd, k /* pc */,
-                                    pg_code, sub_pg_code, buff,
+                                    pg_code, sub_pg_code, buffp,
                                     calc_len, true, verbose);
         else
             res = sg_ll_mode_sense10_v2(sg_fd, false /* llbaa */, dbd,
                                         k /* pc */, pg_code, sub_pg_code,
-                                        buff, calc_len, 0, &resid, true,
+                                        buffp, calc_len, 0, &resid, true,
                                         verbose);
         if (res || resid) {
             if (0 == first_err) {
@@ -759,10 +790,13 @@ sg_get_mode_page_controls(int sg_fd, bool mode6, int pg_code, int sub_pg_code,
                 continue;
         }
         if (xfer_len > 0)
-            memcpy(pcontrol_arr[k], buff + offset, xfer_len);
+            memcpy(pcontrol_arr[k], buffp + offset, xfer_len);
         if (success_mask)
             *success_mask |= (1 << k);
     }
+fini:
+    if (free_buffp)
+        free(free_buffp);
     return first_err;
 }
 
@@ -797,7 +831,7 @@ sg_ll_log_sense_v2(int sg_fd, bool ppc, bool sp, int pc, int pg_code,
     int res, ret, sense_cat, resid;
     uint8_t logs_cdb[LOG_SENSE_CMDLEN] =
         {LOG_SENSE_CMD, 0, 0, 0, 0, 0, 0, 0, 0, 0};
-    uint8_t sense_b[SENSE_BUFF_LEN];
+    uint8_t sense_b[SENSE_BUFF_LEN] SG_C_CPP_ZERO_INIT;
     struct sg_pt_base * ptvp;
 
     if (mx_resp_len > 0xffff) {
@@ -829,9 +863,12 @@ sg_ll_log_sense_v2(int sg_fd, bool ppc, bool sp, int pc, int pg_code,
     resid = get_scsi_pt_resid(ptvp);
     if (residp)
         *residp = resid;
-    if (-1 == ret)
-        ret = sg_convert_errno(get_scsi_pt_os_err(ptvp));
-    else if (-2 == ret) {
+    if (-1 == ret) {
+        if (get_scsi_pt_transport_err(ptvp))
+            ret = SG_LIB_TRANSPORT_ERROR;
+        else
+            ret = sg_convert_errno(get_scsi_pt_os_err(ptvp));
+    } else if (-2 == ret) {
         switch (sense_cat) {
         case SG_LIB_CAT_RECOVERED:
         case SG_LIB_CAT_NO_SENSE:
@@ -878,7 +915,7 @@ sg_ll_log_select(int sg_fd, bool pcr, bool sp, int pc, int pg_code,
     int res, ret, sense_cat;
     uint8_t logs_cdb[LOG_SELECT_CMDLEN] =
         {LOG_SELECT_CMD, 0, 0, 0, 0, 0, 0, 0, 0, 0};
-    uint8_t sense_b[SENSE_BUFF_LEN];
+    uint8_t sense_b[SENSE_BUFF_LEN] SG_C_CPP_ZERO_INIT;
     struct sg_pt_base * ptvp;
 
     if (param_len > 0xffff) {
@@ -908,9 +945,12 @@ sg_ll_log_select(int sg_fd, bool pcr, bool sp, int pc, int pg_code,
     set_scsi_pt_data_out(ptvp, paramp, param_len);
     res = do_scsi_pt(ptvp, sg_fd, DEF_PT_TIMEOUT, verbose);
     ret = sg_cmds_process_resp(ptvp, cdb_s, res, noisy, verbose, &sense_cat);
-    if (-1 == ret)
-        ret = sg_convert_errno(get_scsi_pt_os_err(ptvp));
-    else if (-2 == ret) {
+    if (-1 == ret) {
+        if (get_scsi_pt_transport_err(ptvp))
+            ret = SG_LIB_TRANSPORT_ERROR;
+        else
+            ret = sg_convert_errno(get_scsi_pt_os_err(ptvp));
+    } else if (-2 == ret) {
         switch (sense_cat) {
         case SG_LIB_CAT_RECOVERED:
         case SG_LIB_CAT_NO_SENSE:
@@ -946,7 +986,7 @@ sg_ll_start_stop_unit_com(struct sg_pt_base * ptvp, int sg_fd, bool immed,
     bool local_cdb = true;
     int res, ret, sense_cat;
     uint8_t ssuBlk[START_STOP_CMDLEN] = {START_STOP_CMD, 0, 0, 0, 0, 0};
-    uint8_t sense_b[SENSE_BUFF_LEN];
+    uint8_t sense_b[SENSE_BUFF_LEN] SG_C_CPP_ZERO_INIT;
 
     if (immed)
         ssuBlk[1] = 0x1;
@@ -985,9 +1025,12 @@ sg_ll_start_stop_unit_com(struct sg_pt_base * ptvp, int sg_fd, bool immed,
     }
     res = do_scsi_pt(ptvp, -1, START_PT_TIMEOUT, verbose);
     ret = sg_cmds_process_resp(ptvp, cdb_s, res, noisy, verbose, &sense_cat);
-    if (-1 == ret)
-        ret = sg_convert_errno(get_scsi_pt_os_err(ptvp));
-    else if (-2 == ret) {
+    if (-1 == ret) {
+        if (get_scsi_pt_transport_err(ptvp))
+            ret = SG_LIB_TRANSPORT_ERROR;
+        else
+            ret = sg_convert_errno(get_scsi_pt_os_err(ptvp));
+    } else if (-2 == ret) {
         switch (sense_cat) {
         case SG_LIB_CAT_RECOVERED:
         case SG_LIB_CAT_NO_SENSE:
@@ -1043,7 +1086,7 @@ sg_ll_prevent_allow(int sg_fd, int prevent, bool noisy, int verbose)
     int res, ret, sense_cat;
     uint8_t p_cdb[PREVENT_ALLOW_CMDLEN] =
                 {PREVENT_ALLOW_CMD, 0, 0, 0, 0, 0};
-    uint8_t sense_b[SENSE_BUFF_LEN];
+    uint8_t sense_b[SENSE_BUFF_LEN] SG_C_CPP_ZERO_INIT;
     struct sg_pt_base * ptvp;
 
     if ((prevent < 0) || (prevent > 3)) {
@@ -1065,9 +1108,12 @@ sg_ll_prevent_allow(int sg_fd, int prevent, bool noisy, int verbose)
     set_scsi_pt_sense(ptvp, sense_b, sizeof(sense_b));
     res = do_scsi_pt(ptvp, sg_fd, DEF_PT_TIMEOUT, verbose);
     ret = sg_cmds_process_resp(ptvp, cdb_s, res, noisy, verbose, &sense_cat);
-    if (-1 == ret)
-        ret = sg_convert_errno(get_scsi_pt_os_err(ptvp));
-    else if (-2 == ret) {
+    if (-1 == ret) {
+        if (get_scsi_pt_transport_err(ptvp))
+            ret = SG_LIB_TRANSPORT_ERROR;
+        else
+            ret = sg_convert_errno(get_scsi_pt_os_err(ptvp));
+    } else if (-2 == ret) {
         switch (sense_cat) {
         case SG_LIB_CAT_RECOVERED:
         case SG_LIB_CAT_NO_SENSE:
