@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2005-2019 Douglas Gilbert.
+ * Copyright (c) 2005-2023 Douglas Gilbert.
  * All rights reserved.
  * Use of this source code is governed by a BSD-style
  * license that can be found in the BSD_LICENSE file.
@@ -7,7 +7,7 @@
  * SPDX-License-Identifier: BSD-2-Clause
  */
 
-/* sg_pt_linux version 1.47 20190612 */
+/* sg_pt_linux version 1.56 20230412 */
 
 
 #include <stdio.h>
@@ -31,7 +31,9 @@
 #include "config.h"
 #endif
 
+#ifdef HAVE_LINUX_MAJOR_H
 #include <linux/major.h>
+#endif
 
 #include "sg_pt.h"
 #include "sg_lib.h"
@@ -50,7 +52,7 @@
 #endif
 
 #ifndef BLOCK_EXT_MAJOR
-#define BLOCK_EXT_MAJOR 259
+#define BLOCK_EXT_MAJOR 259     /* used by NVMe block devices */
 #endif
 
 #define DEF_TIMEOUT 60000       /* 60,000 millisecs (60 seconds) */
@@ -71,8 +73,11 @@ static const char * linux_host_bytes[] = {
     "DID_NEXUS_FAILURE (reservation conflict)",
     "DID_ALLOC_FAILURE",
     "DID_MEDIUM_ERROR",
+    "DID_TRANSPORT_MARGINAL",   /*0x14 */
 };
 
+/* These where made obsolete around lk 5.12.0 . Only DRIVER_SENSE [0x8] is
+ * defined in scsi/sg.h for backward comaptibility */
 static const char * linux_driver_bytes[] = {
     "DRIVER_OK", "DRIVER_BUSY", "DRIVER_SOFT", "DRIVER_MEDIA",
     "DRIVER_ERROR", "DRIVER_INVALID", "DRIVER_TIMEOUT", "DRIVER_HARD",
@@ -111,6 +116,7 @@ static const char * linux_driver_suggests[] = {
 bool sg_bsg_nvme_char_major_checked = false;
 int sg_bsg_major = 0;
 volatile int sg_nvme_char_major = 0;
+volatile int sg_nvme_gen_char_major = 0;
 
 bool sg_checked_version_num = false;
 int sg_driver_version_num = 0;
@@ -125,13 +131,14 @@ long sg_lin_page_size = 4096;   /* default, overridden with correct value */
 void
 sg_find_bsg_nvme_char_major(int verbose)
 {
-    bool got_one = false;
+    int num_got = 0;
     int n;
-    const char * proc_devices = "/proc/devices";
     char * cp;
     FILE *fp;
     char a[128];
     char b[128];
+    static const int blen = sizeof(b);
+    static const char * proc_devices = "/proc/devices";
 
     sg_lin_page_size = sysconf(_SC_PAGESIZE);
     if (NULL == (fp = fopen(proc_devices, "r"))) {
@@ -139,23 +146,27 @@ sg_find_bsg_nvme_char_major(int verbose)
             pr2ws("fopen %s failed: %s\n", proc_devices, strerror(errno));
         return;
     }
-    while ((cp = fgets(b, sizeof(b), fp))) {
+    while ((cp = fgets(b, blen, fp))) {
         if ((1 == sscanf(b, "%126s", a)) &&
             (0 == memcmp(a, "Character", 9)))
             break;
     }
-    while (cp && (cp = fgets(b, sizeof(b), fp))) {
+    while (cp && (cp = fgets(b, blen, fp))) {
         if (2 == sscanf(b, "%d %126s", &n, a)) {
             if (0 == strcmp("bsg", a)) {
                 sg_bsg_major = n;
-                if (got_one)
+                if (++num_got >= 3)
                     break;
-                got_one = true;
-            } else if (0 == strcmp("nvme", a)) {
-                sg_nvme_char_major = n;
-                if (got_one)
-                    break;
-                got_one = true;
+            } else if (0 == memcmp("nvme", a, 4)) {
+                if (0 == strcmp("nvme-generic", a)) {
+                    sg_nvme_gen_char_major = n;
+                    if (++num_got >= 3)
+                        break;
+                } else if (0 == strcmp("nvme", a)) {
+                    sg_nvme_char_major = n;
+                    if (++num_got >= 3)
+                        break;
+                }
             }
         } else
             break;
@@ -166,8 +177,11 @@ sg_find_bsg_nvme_char_major(int verbose)
                 pr2ws("found sg_bsg_major=%d\n", sg_bsg_major);
             if (sg_nvme_char_major > 0)
                 pr2ws("found sg_nvme_char_major=%d\n", sg_nvme_char_major);
+            if (sg_nvme_gen_char_major > 0)
+                pr2ws("found sg_nvme_gen_char_major=%d\n",
+                      sg_nvme_gen_char_major);
         } else
-            pr2ws("found no bsg not nvme char device in %s\n", proc_devices);
+            pr2ws("found no bsg nor nvme char device in %s\n", proc_devices);
     }
     fclose(fp);
 }
@@ -182,8 +196,10 @@ check_file_type(int dev_fd, struct stat * dev_statp, bool * is_bsg_p,
                 int verbose)
 {
     bool is_nvme = false;
+    bool is_nvme_gen = false;
     bool is_sg = false;
     bool is_bsg = false;
+    bool is_char = false;
     bool is_block = false;
     int os_err = 0;
     int major_num;
@@ -199,12 +215,32 @@ check_file_type(int dev_fd, struct stat * dev_statp, bool * is_bsg_p,
         }
         major_num = (int)SG_DEV_MAJOR(dev_statp->st_rdev);
         if (S_ISCHR(dev_statp->st_mode)) {
+            is_char = true;
             if (SCSI_GENERIC_MAJOR == major_num)
                 is_sg = true;
-            else if (sg_bsg_major == major_num)
-                is_bsg = true;
-            else if (sg_nvme_char_major == major_num)
-                is_nvme = true;
+            else {
+                if (! sg_bsg_nvme_char_major_checked) {
+                    sg_bsg_nvme_char_major_checked = true;
+                    sg_find_bsg_nvme_char_major(verbose);
+                }
+                if (sg_bsg_major == major_num)
+                    is_bsg = true;
+                else if (sg_nvme_char_major == major_num)
+                    is_nvme = true;
+                else if (sg_nvme_gen_char_major == major_num) {
+                    is_nvme_gen = true;
+                    nsid = ioctl(dev_fd, NVME_IOCTL_ID, NULL);
+                    if (SG_NVME_BROADCAST_NSID == nsid) {
+                        /* means ioctl error */
+                        os_err = errno;
+                        if (verbose)
+                            pr2ws("%s: ioctl(NVME_IOCTL_ID) failed: %s "
+                                  "(errno=%d) on NVMe generic\n", __func__,
+                                  safe_strerror(os_err), os_err);
+                    } else
+                        os_err = 0;
+                }
+            }
         } else if (S_ISBLK(dev_statp->st_mode)) {
             is_block = true;
             if (BLOCK_EXT_MAJOR == major_num) {
@@ -232,6 +268,8 @@ skip_out:
             pr2ws("sg device\n");
         else if (is_bsg)
             pr2ws("bsg device\n");
+        else if (is_nvme_gen)
+            pr2ws("NVMe generic char device, nsid=%d\n", nsid);
         else if (is_nvme && (0 == nsid))
             pr2ws("NVMe char device\n");
         else if (is_nvme)
@@ -239,13 +277,15 @@ skip_out:
                   ((uint32_t)-1 == nsid) ? -1LL : (long long)nsid);
         else if (is_block)
             pr2ws("block device\n");
+        else if (is_char)
+            pr2ws("character device\n");
         else
             pr2ws("undetermined device, could be regular file\n");
     }
     if (is_bsg_p)
         *is_bsg_p = is_bsg;
     if (is_nvme_p)
-        *is_nvme_p = is_nvme;
+        *is_nvme_p = is_nvme | is_nvme_gen;
     if (nsid_p)
         *nsid_p = nsid;
     if (os_err_p)
@@ -259,8 +299,8 @@ skip_out:
  * device_name. Returns 1 if SCSI generic pass-though device, returns 2 if
  * secondary SCSI pass-through device (in Linux a bsg device); returns 3 is
  * char NVMe device (i.e. no NSID); returns 4 if block NVMe device (includes
- * NSID), or 0 if something else (e.g. ATA block device) or dev_fd < 0.
- * If error, returns negated errno (operating system) value. */
+ * NSID), or 0 if something else (e.g. SCSI or ATA block device) or
+ * dev_fd < 0. If error, returns negated errno (operating system) value. */
 int
 check_pt_file_handle(int dev_fd, const char * device_name, int verbose)
 {
@@ -268,10 +308,6 @@ check_pt_file_handle(int dev_fd, const char * device_name, int verbose)
         pr2ws("%s: dev_fd=%d, device_name: %s\n", __func__, dev_fd,
               device_name);
     /* Linux doesn't need device_name to determine which pass-through */
-    if (! sg_bsg_nvme_char_major_checked) {
-        sg_bsg_nvme_char_major_checked = true;
-        sg_find_bsg_nvme_char_major(verbose);
-    }
     if (dev_fd >= 0) {
         bool is_sg, is_bsg, is_nvme;
         int err;
@@ -336,10 +372,6 @@ scsi_pt_open_flags(const char * device_name, int flags, int verbose)
 {
     int fd;
 
-    if (! sg_bsg_nvme_char_major_checked) {
-        sg_bsg_nvme_char_major_checked = true;
-        sg_find_bsg_nvme_char_major(verbose);
-    }
     if (verbose > 1) {
         pr2ws("open %s with flags=0x%x\n", device_name, flags);
     }
@@ -385,12 +417,13 @@ static bool ev_dsense = false;
 struct sg_pt_base *
 construct_scsi_pt_obj_with_fd(int dev_fd, int verbose)
 {
-    int err;
     struct sg_pt_linux_scsi * ptp;
 
     ptp = (struct sg_pt_linux_scsi *)
           calloc(1, sizeof(struct sg_pt_linux_scsi));
     if (ptp) {
+        int err;
+
 #if (HAVE_NVME && (! IGNORE_NVME))
         sntl_init_dev_stat(&ptp->dev_stat);
         if (! checked_ev_dsense) {
@@ -435,8 +468,8 @@ destruct_scsi_pt_obj(struct sg_pt_base * vp)
             ptp->free_nvme_id_ctlp = NULL;
             ptp->nvme_id_ctlp = NULL;
         }
-        if (ptp)
-            free(ptp);
+        if (vp)
+            free(vp);
     }
 }
 
@@ -444,13 +477,14 @@ destruct_scsi_pt_obj(struct sg_pt_base * vp)
 void
 clear_scsi_pt_obj(struct sg_pt_base * vp)
 {
-    bool is_sg, is_bsg, is_nvme;
-    int fd;
-    uint32_t nvme_nsid;
-    struct sg_sntl_dev_state_t dev_stat;
     struct sg_pt_linux_scsi * ptp = &vp->impl;
 
     if (ptp) {
+        bool is_sg, is_bsg, is_nvme;
+        int fd;
+        uint32_t nvme_nsid;
+        struct sg_sntl_dev_state_t dev_stat;
+
         fd = ptp->dev_fd;
         is_sg = ptp->is_sg;
         is_bsg = ptp->is_bsg;
@@ -471,12 +505,31 @@ clear_scsi_pt_obj(struct sg_pt_base * vp)
         ptp->is_sg = is_sg;
         ptp->is_bsg = is_bsg;
         ptp->is_nvme = is_nvme;
-        ptp->nvme_direct = false;
+        ptp->nvme_our_sntl = false;
         ptp->nvme_nsid = nvme_nsid;
         ptp->dev_stat = dev_stat;
     }
 }
 
+void
+partial_clear_scsi_pt_obj(struct sg_pt_base * vp)
+{
+    if (vp) {
+        struct sg_pt_linux_scsi * ptp = &vp->impl;
+
+        ptp->in_err = 0;
+        ptp->os_err = 0;
+        ptp->io_hdr.device_status = 0;
+        ptp->io_hdr.transport_status = 0;
+        ptp->io_hdr.driver_status = 0;
+        ptp->io_hdr.din_xferp = 0;
+        ptp->io_hdr.din_xfer_len = 0;
+        ptp->io_hdr.dout_xferp = 0;
+        ptp->io_hdr.dout_xfer_len = 0;
+        ptp->nvme_result = 0;
+    }
+}
+
 #ifndef SG_SET_GET_EXTENDED
 
 /* If both sei_wr_mask and sei_rd_mask are 0, this ioctl does nothing */
@@ -517,10 +570,6 @@ set_pt_file_handle(struct sg_pt_base * vp, int dev_fd, int verbose)
     struct sg_pt_linux_scsi * ptp = &vp->impl;
     struct stat a_stat;
 
-    if (! sg_bsg_nvme_char_major_checked) {
-        sg_bsg_nvme_char_major_checked = true;
-        sg_find_bsg_nvme_char_major(verbose);
-    }
     ptp->dev_fd = dev_fd;
     if (dev_fd >= 0) {
         ptp->is_sg = check_file_type(dev_fd, &a_stat, &ptp->is_bsg,
@@ -528,6 +577,7 @@ set_pt_file_handle(struct sg_pt_base * vp, int dev_fd, int verbose)
                                      &ptp->os_err, verbose);
         if (ptp->is_sg && (! sg_checked_version_num)) {
             if (ioctl(dev_fd, SG_GET_VERSION_NUM, &ptp->sg_version) < 0) {
+                ptp->os_err = errno;
                 ptp->sg_version = 0;
                 if (verbose > 3)
                     pr2ws("%s: ioctl(SG_GET_VERSION_NUM) failed: errno: %d "
@@ -568,6 +618,7 @@ set_pt_file_handle(struct sg_pt_base * vp, int dev_fd, int verbose)
             seip->ctl_flags_wr_mask |= SG_CTL_FLAGM_TIME_IN_NS;
             seip->ctl_flags |= SG_CTL_FLAGM_TIME_IN_NS;
             if (ioctl(dev_fd, SG_SET_GET_EXTENDED, seip) < 0) {
+                ptp->os_err = errno;
                 if (verbose > 2)
                     pr2ws("%s: unable to override milli --> nanoseconds: "
                           "%s\n", __func__, safe_strerror(errno));
@@ -588,7 +639,7 @@ set_pt_file_handle(struct sg_pt_base * vp, int dev_fd, int verbose)
         ptp->is_sg = false;
         ptp->is_bsg = false;
         ptp->is_nvme = false;
-        ptp->nvme_direct = false;
+        ptp->nvme_our_sntl = false;
         ptp->nvme_nsid = 0;
         ptp->os_err = 0;
     }
@@ -619,21 +670,36 @@ set_scsi_pt_cdb(struct sg_pt_base * vp, const uint8_t * cdb,
 {
     struct sg_pt_linux_scsi * ptp = &vp->impl;
 
-    if (ptp->io_hdr.request)
-        ++ptp->in_err;
     ptp->io_hdr.request = (__u64)(sg_uintptr_t)cdb;
     ptp->io_hdr.request_len = cdb_len;
 }
 
+int
+get_scsi_pt_cdb_len(const struct sg_pt_base * vp)
+{
+    const struct sg_pt_linux_scsi * ptp = &vp->impl;
+
+    return ptp->io_hdr.request_len;
+}
+
+uint8_t *
+get_scsi_pt_cdb_buf(const struct sg_pt_base * vp)
+{
+    const struct sg_pt_linux_scsi * ptp = &vp->impl;
+
+    return (uint8_t *)(sg_uintptr_t)ptp->io_hdr.request;
+}
+
 void
 set_scsi_pt_sense(struct sg_pt_base * vp, uint8_t * sense,
                   int max_sense_len)
 {
     struct sg_pt_linux_scsi * ptp = &vp->impl;
 
-    if (ptp->io_hdr.response)
-        ++ptp->in_err;
-    memset(sense, 0, max_sense_len);
+    if (sense) {
+        if (max_sense_len > 0)
+            memset(sense, 0, max_sense_len);
+    }
     ptp->io_hdr.response = (__u64)(sg_uintptr_t)sense;
     ptp->io_hdr.max_response_len = max_sense_len;
 }
@@ -686,7 +752,7 @@ set_scsi_pt_packet_id(struct sg_pt_base * vp, int pack_id)
 {
     struct sg_pt_linux_scsi * ptp = &vp->impl;
 
-    ptp->io_hdr.request_extra = pack_id;        /* was place in spare_in */
+    ptp->io_hdr.request_extra = pack_id;        /* was placed in spare_in */
 }
 
 void
@@ -760,7 +826,7 @@ get_scsi_pt_resid(const struct sg_pt_base * vp)
 {
     const struct sg_pt_linux_scsi * ptp = &vp->impl;
 
-    if ((NULL == ptp) || (ptp->nvme_direct))
+    if ((NULL == ptp) || (ptp->is_nvme && ! ptp->nvme_our_sntl))
         return 0;
     else if ((ptp->io_hdr.din_xfer_len > 0) &&
              (ptp->io_hdr.dout_xfer_len > 0))
@@ -819,8 +885,8 @@ get_scsi_pt_status_response(const struct sg_pt_base * vp)
 
     if (NULL == ptp)
         return 0;
-    return (int)(ptp->nvme_direct ? ptp->nvme_status :
-                                    ptp->io_hdr.device_status);
+    return (int)((ptp->is_nvme && ! ptp->nvme_our_sntl) ?
+                         ptp->nvme_status : ptp->io_hdr.device_status);
 }
 
 uint32_t
@@ -830,8 +896,8 @@ get_pt_result(const struct sg_pt_base * vp)
 
     if (NULL == ptp)
         return 0;
-    return ptp->nvme_direct ? ptp->nvme_result :
-                              ptp->io_hdr.device_status;
+    return (ptp->is_nvme && ! ptp->nvme_our_sntl) ?
+                        ptp->nvme_result : ptp->io_hdr.device_status;
 }
 
 int
@@ -847,7 +913,7 @@ get_scsi_pt_sense_buf(const struct sg_pt_base * vp)
 {
     const struct sg_pt_linux_scsi * ptp = &vp->impl;
 
-    return (uint8_t *)ptp->io_hdr.response;
+    return (uint8_t *)(sg_uintptr_t)ptp->io_hdr.response;
 }
 
 int
@@ -917,16 +983,13 @@ get_scsi_pt_transport_err_str(const struct sg_pt_base * vp, int max_b_len,
         return b;
     }
     cp += n;
-    driv = ds & SG_LIB_DRIVER_MASK;
-    if (driv < (int)SG_ARRAY_SIZE(linux_driver_bytes))
-        driv_cp = linux_driver_bytes[driv];
-#if 0
-    sugg = (ds & SG_LIB_SUGGEST_MASK) >> 4;
-    if (sugg < SG_ARRAY_SIZE(linux_driver_suggests)
-        sugg_cp = linux_driver_suggests[sugg];
-#endif
-    n = snprintf(cp, m, "Driver_status=0x%02x [%s]\n", ds, driv_cp);
-    m -= n;
+    if (ds) {
+        driv = ds & SG_LIB_DRIVER_MASK;
+        if (driv < (int)SG_ARRAY_SIZE(linux_driver_bytes))
+            driv_cp = linux_driver_bytes[driv];
+        n = snprintf(cp, m, "Driver_status=0x%02x [%s]\n", ds, driv_cp);
+        m -= n;
+    }
     if (m < 1)
         b[max_b_len - 1] = '\0';
     return b;
@@ -1085,14 +1148,9 @@ do_scsi_pt_v4(struct sg_pt_linux_scsi * ptp, int fd, int time_secs,
 int
 do_scsi_pt(struct sg_pt_base * vp, int fd, int time_secs, int verbose)
 {
-    int err;
     struct sg_pt_linux_scsi * ptp = &vp->impl;
     bool have_checked_for_type = (ptp->dev_fd >= 0);
 
-    if (! sg_bsg_nvme_char_major_checked) {
-        sg_bsg_nvme_char_major_checked = true;
-        sg_find_bsg_nvme_char_major(verbose);
-    }
     if (ptp->in_err) {
         if (verbose)
             pr2ws("Replicated or unused set_scsi_pt... functions\n");
@@ -1113,12 +1171,16 @@ do_scsi_pt(struct sg_pt_base * vp, int fd, int time_secs, int verbose)
     } else
         fd = ptp->dev_fd;
     if (! have_checked_for_type) {
-        err = set_pt_file_handle(vp, ptp->dev_fd, verbose);
+        int err = set_pt_file_handle(vp, ptp->dev_fd, verbose);
+
         if (err)
             return -ptp->os_err;
     }
     if (ptp->os_err)
         return -ptp->os_err;
+    if (verbose > 5)
+        pr2ws("%s:  is_nvme=%d, is_sg=%d, is_bsg=%d\n", __func__,
+              (int)ptp->is_nvme, (int)ptp->is_sg, (int)ptp->is_bsg);
     if (ptp->is_nvme)
         return sg_do_nvme_pt(vp, -1, time_secs, verbose);
     else if (ptp->is_sg) {
