@@ -6,7 +6,7 @@
  *
  * Copyright (C) 2003  Grant Grundler    grundler at parisc-linux dot org
  * Copyright (C) 2003  James Bottomley       jejb at parisc-linux dot org
- * Copyright (C) 2005-2020  Douglas Gilbert   dgilbert at interlog dot com
+ * Copyright (C) 2005-2023  Douglas Gilbert   dgilbert at interlog dot com
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -15,7 +15,7 @@
  *
  * SPDX-License-Identifier: GPL-2.0-or-later
  *
- * See http://www.t10.org for relevant standards and drafts. The most recent
+ * See https://www.t10.org for relevant standards and drafts. The most recent
  * draft is SBC-4 revision 2.
  */
 
@@ -40,9 +40,11 @@
 #include "sg_pr2serr.h"
 #include "sg_pt.h"
 
-static const char * version_str = "1.62 20201223";
+static const char * version_str = "1.73 20230618";
 
 
+#define MY_NAME "sg_format"
+
 #define RW_ERROR_RECOVERY_PAGE 1  /* can give alternate with --mode=MP */
 
 #define SHORT_TIMEOUT           20   /* 20 seconds unless --wait given */
@@ -58,17 +60,6 @@ static const char * version_str = "1.62 20201223";
                                       true -> request sense */
 #define MAX_BUFF_SZ     252
 
-#if defined(MSC_VER) || defined(__MINGW32__)
-#define HAVE_MS_SLEEP
-#endif
-
-#ifdef HAVE_MS_SLEEP
-#include <windows.h>
-#define sleep_for(seconds)    Sleep( (seconds) * 1000)
-#else
-#define sleep_for(seconds)    sleep(seconds)
-#endif
-
 /* FORMAT UNIT (SBC) and FORMAT MEDIUM (SSC) share the same opcode */
 #define SG_FORMAT_MEDIUM_CMD 0x4
 #define SG_FORMAT_MEDIUM_CMDLEN 6
@@ -80,6 +71,8 @@ static const char * version_str = "1.62 20201223";
 #define SENSE_BUFF_LEN 64       /* Arbitrary, could be larger */
 
 struct opts_t {
+        bool cappid;            /* -a */
+        bool cappid_twice;      /* -aa */
         bool cmplst;            /* -C value */
         bool cmplst_given;
         bool dry_run;           /* -d */
@@ -120,7 +113,8 @@ struct opts_t {
 
 
 
-static struct option long_options[] = {
+static const struct option long_options[] = {
+        {"cappid", no_argument, 0, 'a'},
         {"count", required_argument, 0, 'c'},
         {"cmplst", required_argument, 0, 'C'},
         {"dcrt", no_argument, 0, 'D'},
@@ -149,6 +143,7 @@ static struct option long_options[] = {
         {"size", required_argument, 0, 's'},
         {"tape", required_argument, 0, 'T'},
         {"timeout", required_argument, 0, 'm'},
+        {"tmo", required_argument, 0, 'm'},
         {"verbose", no_argument, 0, 'v'},
         {"verify", no_argument, 0, 'y'},
         {"version", no_argument, 0, 'V'},
@@ -156,29 +151,35 @@ static struct option long_options[] = {
         {0, 0, 0, 0},
 };
 
+static bool has_cappid_vpd = false;
+static bool has_fpresets_vpd = false;
 static const char * fu_s = "Format unit";
 static const char * fm_s = "Format medium";
 static const char * fwp_s = "Format with preset";
+static const char * tawvv_s =
+                "try again with '-v' or '-vv' option for more information";
 
 
 static void
 usage()
 {
         printf("Usage:\n"
-               "  sg_format [--cmplst=0|1] [--count=COUNT] [--dcrt] "
-               "[--dry-run] [--early]\n"
-               "            [--ffmt=FFMT] [--fmtmaxlba] [--fmtpinfo=FPI] "
-               "[--format] [--help]\n"
-               "            [--ip-def] [--long] [--mode=MP] [--pfu=PFU] "
-               "[--pie=PIE]\n"
-               "            [--pinfo] [--poll=PT] [--preset=ID] [--quick] "
-               "[--resize]\n"
-               "            [--rto_req] [--security] [--six] [--size=LB_SZ] "
-               "[--tape=FM]\n"
-               "            [--timeout=SECS] [--verbose] [--verify] "
-               "[--version] [--wait]\n"
-               "            DEVICE\n"
+               "  sg_format [--cappid] [--cmplst=0|1] [--count=COUNT] "
+               "[--dcrt] [--dry-run]\n"
+               "            [--early] [--ffmt=FFMT] [--fmtmaxlba] "
+               "[--fmtpinfo=FPI]\n"
+               "            [--format] [--help] [--ip-def] [--long] "
+               "[--mode=MP] [--pfu=PFU]\n"
+               "            [--pie=PIE] [--pinfo] [--poll=PT] [--preset=ID] "
+               "[--quick]\n"
+               "            [--resize] [--rto_req] [--security] [--six] "
+               "[--size=LB_SZ]\n"
+               "            [--tape=FM] [--timeout=SECS] [--verbose] "
+               "[--verify] [--version]\n"
+               "            [--wait] DEVICE\n"
                "  where:\n"
+               "    --cappid|-a     set CAPPID bit in Mode Select if count "
+               "change\n"
                "    --cmplst=0|1\n"
                "      -C 0|1        sets CMPLST bit in format cdb "
                "(def: 1; if FFMT: 0)\n"
@@ -200,14 +201,14 @@ usage()
                "1: after\n"
                "                           format, unwritten data read "
                "without error\n"
+               "    --fmtmaxlba|-b    sets FMTMAXLBA field in FORMAT WITH "
+               "PRESET\n"
                "    --fmtpinfo=FPI|-f FPI    FMTPINFO field value "
                "(default: 0)\n"
                "    --format|-F     do FORMAT UNIT (default: report current "
                "count and size)\n"
                "                    use thrice for FORMAT UNIT command "
                "only\n"
-               "    --fmtmaxlba|-b    sets FMTMAXLBA field in FORMAT WITH "
-               "PRESET\n"
                "    --help|-h       prints out this usage message\n"
                "    --ip-def|-I     use default initialization pattern\n"
                "    --long|-l       allow for 64 bit lbas (default: assume "
@@ -278,7 +279,7 @@ sg_ll_format_medium(int sg_fd, bool verify, bool immed, int format,
         int ret, res, sense_cat;
         uint8_t fm_cdb[SG_FORMAT_MEDIUM_CMDLEN] =
                                   {SG_FORMAT_MEDIUM_CMD, 0, 0, 0, 0, 0};
-        uint8_t sense_b[SENSE_BUFF_LEN];
+        uint8_t sense_b[SENSE_BUFF_LEN] SG_C_CPP_ZERO_INIT;
         struct sg_pt_base * ptvp;
 
         if (verify)
@@ -308,9 +309,12 @@ sg_ll_format_medium(int sg_fd, bool verify, bool immed, int format,
         res = do_scsi_pt(ptvp, sg_fd, timeout, verbose);
         ret = sg_cmds_process_resp(ptvp, fm_s, res, noisy, verbose,
                                    &sense_cat);
-        if (-1 == ret)
+        if (-1 == ret) {
+            if (get_scsi_pt_transport_err(ptvp))
+                ret = SG_LIB_TRANSPORT_ERROR;
+            else
                 ret = sg_convert_errno(get_scsi_pt_os_err(ptvp));
-        else if (-2 == ret) {
+        } else if (-2 == ret) {
                 switch (sense_cat) {
                 case SG_LIB_CAT_RECOVERED:
                 case SG_LIB_CAT_NO_SENSE:
@@ -340,7 +344,7 @@ sg_ll_format_with_preset(int sg_fd, bool immed, bool fmtmaxlba,
         int ret, res, sense_cat;
         uint8_t fwp_cdb[SG_FORMAT_WITH_PRESET_CMDLEN] =
                      {SG_FORMAT_WITH_PRESET_CMD, 0, 0, 0, 0, 0, 0, 0, 0, 0};
-        uint8_t sense_b[SENSE_BUFF_LEN];
+        uint8_t sense_b[SENSE_BUFF_LEN] SG_C_CPP_ZERO_INIT;
         struct sg_pt_base * ptvp;
 
         if (immed)
@@ -367,9 +371,12 @@ sg_ll_format_with_preset(int sg_fd, bool immed, bool fmtmaxlba,
         res = do_scsi_pt(ptvp, sg_fd, timeout, verbose);
         ret = sg_cmds_process_resp(ptvp, fwp_s, res, noisy, verbose,
                                    &sense_cat);
-        if (-1 == ret)
+        if (-1 == ret) {
+            if (get_scsi_pt_transport_err(ptvp))
+                ret = SG_LIB_TRANSPORT_ERROR;
+            else
                 ret = sg_convert_errno(get_scsi_pt_os_err(ptvp));
-        else if (-2 == ret) {
+        } else if (-2 == ret) {
                 switch (sense_cat) {
                 case SG_LIB_CAT_RECOVERED:
                 case SG_LIB_CAT_NO_SENSE:
@@ -474,6 +481,15 @@ scsi_format_unit(int fd, const struct opts_t * op)
             free(free_param);
 
         if (res) {
+                if ((SG_LIB_CAT_INVALID_OP == res) && has_fpresets_vpd)
+                        pr2serr("FORMAT UNIT is not supported but maybe "
+                                "FORMAT WITH PRESET is, see '--preset='\n");
+                else if (SG_LIB_CAT_ILLEGAL_REQ == res)
+                        pr2serr("Problem with FORMAT UNIT cdb, %s\n",
+                                tawvv_s);
+                else if (SG_LIB_CAT_INVALID_PARAM == res)
+                        pr2serr("Problem with FORMAT UNIT parameter list, "
+                                "%s\n", tawvv_s);
                 sg_get_category_sense_str(res, sizeof(b), b, vb);
                 pr2serr("%s command: %s\n", fu_s, b);
                 return res;
@@ -502,7 +518,7 @@ scsi_format_unit(int fd, const struct opts_t * op)
                                     POLL_DURATION_SECS;
         if (! op->poll_type) {
                 for(first = true; ; first = false) {
-                        sleep_for(poll_wait_secs);
+                        sg_sleep_secs(poll_wait_secs);
                         progress = -1;
                         res = sg_ll_test_unit_ready_progress(fd, 0, &progress,
                                              true, (vb > 1) ? (vb - 1) : 0);
@@ -531,7 +547,7 @@ scsi_format_unit(int fd, const struct opts_t * op)
                         return sg_convert_errno(ENOMEM);
                 }
                 for(first = true; ; first = false) {
-                        sleep_for(poll_wait_secs);
+                        sg_sleep_secs(poll_wait_secs);
                         memset(reqSense, 0x0, MAX_BUFF_SZ);
                         res = sg_ll_request_sense(fd, false, reqSense,
                                                   MAX_BUFF_SZ, false,
@@ -623,7 +639,7 @@ scsi_format_medium(int fd, const struct opts_t * op)
         }
         if (! op->poll_type) {
                 for(first = true; ; first = false) {
-                        sleep_for(POLL_DURATION_SECS);
+                        sg_sleep_secs(POLL_DURATION_SECS);
                         progress = -1;
                         res = sg_ll_test_unit_ready_progress(fd, 0, &progress,
                                              true, (vb > 1) ? (vb - 1) : 0);
@@ -652,7 +668,7 @@ scsi_format_medium(int fd, const struct opts_t * op)
                         return sg_convert_errno(ENOMEM);
                 }
                 for(first = true; ; first = false) {
-                        sleep_for(POLL_DURATION_SECS);
+                        sg_sleep_secs(POLL_DURATION_SECS);
                         memset(reqSense, 0x0, MAX_BUFF_SZ);
                         res = sg_ll_request_sense(fd, false, reqSense,
                                                   MAX_BUFF_SZ, false,
@@ -742,7 +758,7 @@ scsi_format_with_preset(int fd, const struct opts_t * op)
         }
         if (! op->poll_type) {
                 for(first = true; ; first = false) {
-                        sleep_for(POLL_DURATION_SECS);
+                        sg_sleep_secs(POLL_DURATION_SECS);
                         progress = -1;
                         res = sg_ll_test_unit_ready_progress(fd, 0, &progress,
                                              true, (vb > 1) ? (vb - 1) : 0);
@@ -771,7 +787,7 @@ scsi_format_with_preset(int fd, const struct opts_t * op)
                         return sg_convert_errno(ENOMEM);
                 }
                 for(first = true; ; first = false) {
-                        sleep_for(POLL_DURATION_SECS);
+                        sg_sleep_secs(POLL_DURATION_SECS);
                         memset(reqSense, 0x0, MAX_BUFF_SZ);
                         res = sg_ll_request_sense(fd, false, reqSense,
                                                   MAX_BUFF_SZ, false,
@@ -809,7 +825,7 @@ scsi_format_with_preset(int fd, const struct opts_t * op)
         return 0;
 }
 
-#define VPD_DEVICE_ID 0x83
+// #define VPD_DEVICE_ID 0x83
 #define VPD_ASSOC_LU 0
 #define VPD_ASSOC_TPORT 1
 #define TPROTO_ISCSI 5
@@ -873,13 +889,17 @@ get_lu_name(const uint8_t * bp, int u_len, char * b, int b_len)
 #define VPD_SUPPORTED_VPDS 0x0
 #define VPD_UNIT_SERIAL_NUM 0x80
 #define VPD_DEVICE_ID 0x83
+#define VPD_FORMAT_PRESETS 0xb8
+#define VPD_CAP_PROD_ID 0xba            /* sbc5r04 */
 #define MAX_VPD_RESP_LEN 256
 
 static int
 print_dev_id(int fd, uint8_t * sinq_resp, int max_rlen,
              const struct opts_t * op)
 {
-        int k, n, verb, pdt, has_sn, has_di;
+        bool has_sn = false;
+        bool has_di = false;
+        int k, n, verb, pdt;
         int res = 0;
         uint8_t  * b;
         uint8_t  * free_b = NULL;
@@ -903,7 +923,7 @@ print_dev_id(int fd, uint8_t * sinq_resp, int max_rlen,
                 n = SAFE_STD_INQ_RESP_LEN;
         memcpy(sinq_resp, b, (n < max_rlen) ? n : max_rlen);
         if (n == SAFE_STD_INQ_RESP_LEN) {
-                pdt = b[0] & 0x1f;
+                pdt = b[0] & PDT_MASK;
                 printf("    %.8s  %.16s  %.4s   peripheral_type: %s [0x%x]\n",
                        (const char *)(b + 8), (const char *)(b + 16),
                        (const char *)(b + 32),
@@ -935,12 +955,16 @@ print_dev_id(int fd, uint8_t * sinq_resp, int max_rlen,
         n = sg_get_unaligned_be16(b + 2);
         if (n > (SAFE_STD_INQ_RESP_LEN - 4))
                 n = (SAFE_STD_INQ_RESP_LEN - 4);
-        for (k = 0, has_sn = 0, has_di = 0; k < n; ++k) {
-                if (VPD_UNIT_SERIAL_NUM == b[4 + k])
-                        ++has_sn;
-                else if (VPD_DEVICE_ID == b[4 + k]) {
-                        ++has_di;
-                        break;
+        for (k = 0; k < n; ++k) {
+                if (VPD_UNIT_SERIAL_NUM == b[4 + k])            /* 0x80 */
+                        has_sn = true;
+                else if (VPD_DEVICE_ID == b[4 + k])             /* 0x83 */
+                        has_di = true;
+                else if (VPD_FORMAT_PRESETS == b[4 + k])        /* 0xb8 */
+                        has_fpresets_vpd = true;
+                else if (VPD_CAP_PROD_ID == b[4 + k]) {         /* 0xba */
+                        has_cappid_vpd = true;
+                        break;          /* should be in ascending order */
                 }
         }
         if (has_sn) {
@@ -987,6 +1011,10 @@ print_dev_id(int fd, uint8_t * sinq_resp, int max_rlen,
                         printf("      LU name: %.*s\n", n, a);
         }
 out:
+        if (has_cappid_vpd)
+                pr2serr("Capacity/Product identification mapping VPD page "
+                        "present.\nIf changing size --cappid option may be "
+                        "needed, see 'sg_vpd -p cap'\n");
         if (free_b)
                 free(free_b);
         return res;
@@ -1273,12 +1301,17 @@ parse_cmd_line(struct opts_t * op, int argc, char **argv)
                 int c;
 
                 c = getopt_long(argc, argv,
-                                "bc:C:dDeE:f:FhIlm:M:pP:q:QrRs:St:T:vVwx:y6",
+                                "abc:C:dDeE:f:FhIlm:M:pP:q:QrRs:St:T:vVwx:y6",
                                 long_options, &option_index);
                 if (c == -1)
                         break;
 
                 switch (c) {
+                case 'a':
+                        if (op->cappid)
+                                op->cappid_twice = true;
+                        op->cappid = true;
+                        break;
                 case 'b':
                         op->fmtmaxlba = true;
                         break;
@@ -1549,6 +1582,8 @@ main(int argc, char **argv)
 
         op = &opts;
         memset(op, 0, sizeof(opts));
+        if (getenv("SG3_UTILS_INVOCATION"))
+                sg_rep_invocation(MY_NAME, version_str, argc, argv, NULL);
         ret = parse_cmd_line(op, argc, argv);
         if (ret)
                 return (SG_LIB_OK_FALSE == ret) ? 0 : ret;
@@ -1580,12 +1615,12 @@ main(int argc, char **argv)
                 } else
                         goto out;
         } else
-                pdt = 0x1f & inq_resp[0];
+                pdt = PDT_MASK & inq_resp[0];
         if (op->format) {
                 if ((PDT_DISK != pdt) && (PDT_OPTICAL != pdt) &&
-                    (PDT_RBC != pdt)) {
+                    (PDT_RBC != pdt) && (PDT_ZBC != pdt)) {
                         pr2serr("This format is only defined for disks "
-                                "(using SBC-2 or RBC) and MO media\n");
+                                "(using SBC-2+, ZBC or RBC) and MO media\n");
                         ret = SG_LIB_CAT_MALFORMED;
                         goto out;
                 }
@@ -1625,16 +1660,42 @@ again_sp_false:
                                 res = sg_ll_mode_select6(fd, true /* PF */,
                                                          sp, dbuff, calc_len,
                                                          true, vb);
-                        else
+                        else {
+                                if (op->cappid) {
+                                        uint8_t dsp_blk = 0x20;
+
+                                        if ((! has_cappid_vpd) &&
+                                            (! op->cappid_twice))
+                                                dsp_blk = 0x0;
+                                        if (calc_len > 3)
+                                                dbuff[3] |= dsp_blk;
+                                }
                                 res = sg_ll_mode_select10(fd, true /* PF */,
                                                           sp, dbuff, calc_len,
                                                           true, vb);
+                        }
                         if ((SG_LIB_CAT_ILLEGAL_REQ == res) && sp) {
                                 pr2serr("Try MODE SELECT again with SP=0 "
                                         "this time\n");
                                 sp = false;
                                 goto again_sp_false;
                         }
+                        if (SG_LIB_CAT_INVALID_PARAM == res) {
+                                pr2serr("mode select, bad parameter list: ");
+                                if (has_cappid_vpd && op->cappid)
+                                        pr2serr("check COUNT against "
+                                                "'sg_vpd -p cap'\n");
+                                else if (has_cappid_vpd)
+                                        pr2serr("may need '--cappid' "
+                                                "option\n");
+                                else if (op->cappid)
+                                        pr2serr("try removing '--cappid' "
+                                                "option\n");
+                                else
+                                        pr2serr("see manufacturer's "
+                                                "manual\n");
+
+                        }
                 }
                 ret = res;
                 if (res) {
@@ -1674,24 +1735,8 @@ again_sp_false:
 
         if (op->format) {
 format_only:
-                if (op->quick)
-                        goto skip_f_unit_reconsider;
-                printf("\nA FORMAT UNIT will commence in 15 seconds\n");
-                printf("    ALL data on %s will be DESTROYED\n",
-                       op->device_name);
-                printf("        Press control-C to abort\n");
-                sleep_for(5);
-                printf("\nA FORMAT UNIT will commence in 10 seconds\n");
-                printf("    ALL data on %s will be DESTROYED\n",
-                       op->device_name);
-                printf("        Press control-C to abort\n");
-                sleep_for(5);
-                printf("\nA FORMAT UNIT will commence in 5 seconds\n");
-                printf("    ALL data on %s will be DESTROYED\n",
-                       op->device_name);
-                printf("        Press control-C to abort\n");
-                sleep_for(5);
-skip_f_unit_reconsider:
+                if (! op->quick)
+                    sg_warn_and_wait("FORMAT UNIT", op->device_name, true);
                 res = scsi_format_unit(fd, op);
                 ret = res;
                 if (res) {
@@ -1706,24 +1751,8 @@ skip_f_unit_reconsider:
 format_med:
         if (! op->poll_type_given) /* SSC-5 specifies REQUEST SENSE polling */
                 op->poll_type = true;
-        if (op->quick)
-                goto skip_f_med_reconsider;
-        printf("\nA FORMAT MEDIUM will commence in 15 seconds\n");
-        printf("    ALL data on %s will be DESTROYED\n",
-               op->device_name);
-        printf("        Press control-C to abort\n");
-        sleep_for(5);
-        printf("\nA FORMAT MEDIUM will commence in 10 seconds\n");
-        printf("    ALL data on %s will be DESTROYED\n",
-               op->device_name);
-        printf("        Press control-C to abort\n");
-        sleep_for(5);
-        printf("\nA FORMAT MEDIUM will commence in 5 seconds\n");
-        printf("    ALL data on %s will be DESTROYED\n",
-               op->device_name);
-        printf("        Press control-C to abort\n");
-        sleep_for(5);
-skip_f_med_reconsider:
+        if (! op->quick)
+            sg_warn_and_wait("FORMAT MEDIUM", op->device_name, true);
         res = scsi_format_medium(fd, op);
         ret = res;
         if (res) {
@@ -1734,24 +1763,8 @@ skip_f_med_reconsider:
         goto out;
 
 format_with_pre:
-        if (op->quick)
-                goto skip_f_with_pre_reconsider;
-        printf("\nA FORMAT WITH PRESET will commence in 15 seconds\n");
-        printf("    ALL data on %s will be DESTROYED\n",
-               op->device_name);
-        printf("        Press control-C to abort\n");
-        sleep_for(5);
-        printf("\nA FORMAT WITH PRESET will commence in 10 seconds\n");
-        printf("    ALL data on %s will be DESTROYED\n",
-               op->device_name);
-        printf("        Press control-C to abort\n");
-        sleep_for(5);
-        printf("\nA FORMAT WITH PRESET will commence in 5 seconds\n");
-        printf("    ALL data on %s will be DESTROYED\n",
-               op->device_name);
-        printf("        Press control-C to abort\n");
-        sleep_for(5);
-skip_f_with_pre_reconsider:
+        if (! op->quick)
+            sg_warn_and_wait("FORMAT WITH PRESET", op->device_name, true);
         res = scsi_format_with_preset(fd, op);
         ret = res;
         if (res) {
@@ -1775,8 +1788,7 @@ out:
         }
         if (0 == vb) {
                 if (! sg_if_can2stderr("sg_format failed: ", ret))
-                        pr2serr("Some error occurred, try again with '-v' "
-                                "or '-vv' for more information\n");
+                        pr2serr("Some error occurred, %s\n", tawvv_s);
         }
         return (ret >= 0) ? ret : SG_LIB_CAT_OTHER;
 }
