@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2004-2018 Douglas Gilbert.
+ * Copyright (c) 2004-2023 Douglas Gilbert.
  * All rights reserved.
  * Use of this source code is governed by a BSD-style
  * license that can be found in the BSD_LICENSE file.
@@ -33,7 +33,7 @@
 
 */
 
-static const char * version_str = "0.49 20180626";    /* mmc6r02 */
+static const char * version_str = "0.51 20230618";    /* mmc6r02 */
 
 #define MX_ALLOC_LEN 8192
 #define NAME_BUFF_SZ 64
@@ -43,20 +43,20 @@ static const char * version_str = "0.49 20180626";    /* mmc6r02 */
 
 static uint8_t resp_buffer[MX_ALLOC_LEN];
 
-static struct option long_options[] = {
-        {"brief", no_argument, 0, 'b'},
-        {"current", no_argument, 0, 'c'},
-        {"help", no_argument, 0, 'h'},
-        {"hex", no_argument, 0, 'H'},
-        {"inner-hex", no_argument, 0, 'i'},
-        {"list", no_argument, 0, 'l'},
-        {"raw", no_argument, 0, 'R'},
-        {"readonly", no_argument, 0, 'q'},
-        {"rt", required_argument, 0, 'r'},
-        {"starting", required_argument, 0, 's'},
-        {"verbose", no_argument, 0, 'v'},
-        {"version", no_argument, 0, 'V'},
-        {0, 0, 0, 0},
+static const struct option long_options[] = {
+    {"brief", no_argument, 0, 'b'},
+    {"current", no_argument, 0, 'c'},
+    {"help", no_argument, 0, 'h'},
+    {"hex", no_argument, 0, 'H'},
+    {"inner-hex", no_argument, 0, 'i'},
+    {"list", no_argument, 0, 'l'},
+    {"raw", no_argument, 0, 'R'},
+    {"readonly", no_argument, 0, 'q'},
+    {"rt", required_argument, 0, 'r'},
+    {"starting", required_argument, 0, 's'},
+    {"verbose", no_argument, 0, 'v'},
+    {"version", no_argument, 0, 'V'},
+    {0, 0, 0, 0},
 };
 
 
@@ -100,44 +100,44 @@ struct val_desc_t {
         const char * desc;
 };
 
-static struct val_desc_t profile_desc_arr[] = {
-        {0x0, "No current profile"},
-        {0x1, "Non-removable disk (obs)"},
-        {0x2, "Removable disk"},
-        {0x3, "Magneto optical erasable"},
-        {0x4, "Optical write once"},
-        {0x5, "AS-MO"},
-        {0x8, "CD-ROM"},
-        {0x9, "CD-R"},
-        {0xa, "CD-RW"},
-        {0x10, "DVD-ROM"},
-        {0x11, "DVD-R sequential recording"},
-        {0x12, "DVD-RAM"},
-        {0x13, "DVD-RW restricted overwrite"},
-        {0x14, "DVD-RW sequential recording"},
-        {0x15, "DVD-R dual layer sequental recording"},
-        {0x16, "DVD-R dual layer jump recording"},
-        {0x17, "DVD-RW dual layer"},
-        {0x18, "DVD-Download disc recording"},
-        {0x1a, "DVD+RW"},
-        {0x1b, "DVD+R"},
-        {0x20, "DDCD-ROM"},
-        {0x21, "DDCD-R"},
-        {0x22, "DDCD-RW"},
-        {0x2a, "DVD+RW dual layer"},
-        {0x2b, "DVD+R dual layer"},
-        {0x40, "BD-ROM"},
-        {0x41, "BD-R SRM"},
-        {0x42, "BD-R RRM"},
-        {0x43, "BD-RE"},
-        {0x50, "HD DVD-ROM"},
-        {0x51, "HD DVD-R"},
-        {0x52, "HD DVD-RAM"},
-        {0x53, "HD DVD-RW"},
-        {0x58, "HD DVD-R dual layer"},
-        {0x5a, "HD DVD-RW dual layer"},
-        {0xffff, "Non-conforming profile"},
-        {-1, NULL},
+static const struct val_desc_t profile_desc_arr[] = {
+    {0x0, "No current profile"},
+    {0x1, "Non-removable disk (obs)"},
+    {0x2, "Removable disk"},
+    {0x3, "Magneto optical erasable"},
+    {0x4, "Optical write once"},
+    {0x5, "AS-MO"},
+    {0x8, "CD-ROM"},
+    {0x9, "CD-R"},
+    {0xa, "CD-RW"},
+    {0x10, "DVD-ROM"},
+    {0x11, "DVD-R sequential recording"},
+    {0x12, "DVD-RAM"},
+    {0x13, "DVD-RW restricted overwrite"},
+    {0x14, "DVD-RW sequential recording"},
+    {0x15, "DVD-R dual layer sequental recording"},
+    {0x16, "DVD-R dual layer jump recording"},
+    {0x17, "DVD-RW dual layer"},
+    {0x18, "DVD-Download disc recording"},
+    {0x1a, "DVD+RW"},
+    {0x1b, "DVD+R"},
+    {0x20, "DDCD-ROM"},
+    {0x21, "DDCD-R"},
+    {0x22, "DDCD-RW"},
+    {0x2a, "DVD+RW dual layer"},
+    {0x2b, "DVD+R dual layer"},
+    {0x40, "BD-ROM"},
+    {0x41, "BD-R SRM"},
+    {0x42, "BD-R RRM"},
+    {0x43, "BD-RE"},
+    {0x50, "HD DVD-ROM"},
+    {0x51, "HD DVD-R"},
+    {0x52, "HD DVD-RAM"},
+    {0x53, "HD DVD-RW"},
+    {0x58, "HD DVD-R dual layer"},
+    {0x5a, "HD DVD-RW dual layer"},
+    {0xffff, "Non-conforming profile"},
+    {-1, NULL},
 };
 
 static const char *
@@ -155,68 +155,68 @@ get_profile_str(int profile_num, char * buff)
     return buff;
 }
 
-static struct val_desc_t feature_desc_arr[] = {
-        {0x0, "Profile list"},
-        {0x1, "Core"},
-        {0x2, "Morphing"},
-        {0x3, "Removable media"},
-        {0x4, "Write Protect"},
-        {0x10, "Random readable"},
-        {0x1d, "Multi-read"},
-        {0x1e, "CD read"},
-        {0x1f, "DVD read"},
-        {0x20, "Random writable"},
-        {0x21, "Incremental streaming writable"},
-        {0x22, "Sector erasable"},
-        {0x23, "Formattable"},
-        {0x24, "Hardware defect management"},
-        {0x25, "Write once"},
-        {0x26, "Restricted overwrite"},
-        {0x27, "CD-RW CAV write"},
-        {0x28, "MRW"},          /* Mount Rainier reWritable */
-        {0x29, "Enhanced defect reporting"},
-        {0x2a, "DVD+RW"},
-        {0x2b, "DVD+R"},
-        {0x2c, "Rigid restricted overwrite"},
-        {0x2d, "CD track-at-once"},
-        {0x2e, "CD mastering (session at once)"},
-        {0x2f, "DVD-R/-RW write"},
-        {0x30, "Double density CD read"},
-        {0x31, "Double density CD-R write"},
-        {0x32, "Double density CD-RW write"},
-        {0x33, "Layer jump recording"},
-        {0x34, "LJ rigid restricted oberwrite"},
-        {0x35, "Stop long operation"},
-        {0x37, "CD-RW media write support"},
-        {0x38, "BD-R POW"},
-        {0x3a, "DVD+RW dual layer"},
-        {0x3b, "DVD+R dual layer"},
-        {0x40, "BD read"},
-        {0x41, "BD write"},
-        {0x42, "TSR (timely safe recording)"},
-        {0x50, "HD DVD read"},
-        {0x51, "HD DVD write"},
-        {0x52, "HD DVD-RW fragment recording"},
-        {0x80, "Hybrid disc"},
-        {0x100, "Power management"},
-        {0x101, "SMART"},
-        {0x102, "Embedded changer"},
-        {0x103, "CD audio external play"},
-        {0x104, "Microcode upgrade"},
-        {0x105, "Timeout"},
-        {0x106, "DVD CSS"},
-        {0x107, "Real time streaming"},
-        {0x108, "Drive serial number"},
-        {0x109, "Media serial number"},
-        {0x10a, "Disc control blocks"},
-        {0x10b, "DVD CPRM"},
-        {0x10c, "Firmware information"},
-        {0x10d, "AACS"},
-        {0x10e, "DVD CSS managed recording"},
-        {0x110, "VCPS"},
-        {0x113, "SecurDisc"},
-        {0x120, "BD CPS"},
-        {0x142, "OSSC"},
+static const struct val_desc_t feature_desc_arr[] = {
+    {0x0, "Profile list"},
+    {0x1, "Core"},
+    {0x2, "Morphing"},
+    {0x3, "Removable media"},
+    {0x4, "Write Protect"},
+    {0x10, "Random readable"},
+    {0x1d, "Multi-read"},
+    {0x1e, "CD read"},
+    {0x1f, "DVD read"},
+    {0x20, "Random writable"},
+    {0x21, "Incremental streaming writable"},
+    {0x22, "Sector erasable"},
+    {0x23, "Formattable"},
+    {0x24, "Hardware defect management"},
+    {0x25, "Write once"},
+    {0x26, "Restricted overwrite"},
+    {0x27, "CD-RW CAV write"},
+    {0x28, "MRW"},          /* Mount Rainier reWritable */
+    {0x29, "Enhanced defect reporting"},
+    {0x2a, "DVD+RW"},
+    {0x2b, "DVD+R"},
+    {0x2c, "Rigid restricted overwrite"},
+    {0x2d, "CD track-at-once"},
+    {0x2e, "CD mastering (session at once)"},
+    {0x2f, "DVD-R/-RW write"},
+    {0x30, "Double density CD read"},
+    {0x31, "Double density CD-R write"},
+    {0x32, "Double density CD-RW write"},
+    {0x33, "Layer jump recording"},
+    {0x34, "LJ rigid restricted oberwrite"},
+    {0x35, "Stop long operation"},
+    {0x37, "CD-RW media write support"},
+    {0x38, "BD-R POW"},
+    {0x3a, "DVD+RW dual layer"},
+    {0x3b, "DVD+R dual layer"},
+    {0x40, "BD read"},
+    {0x41, "BD write"},
+    {0x42, "TSR (timely safe recording)"},
+    {0x50, "HD DVD read"},
+    {0x51, "HD DVD write"},
+    {0x52, "HD DVD-RW fragment recording"},
+    {0x80, "Hybrid disc"},
+    {0x100, "Power management"},
+    {0x101, "SMART"},
+    {0x102, "Embedded changer"},
+    {0x103, "CD audio external play"},
+    {0x104, "Microcode upgrade"},
+    {0x105, "Timeout"},
+    {0x106, "DVD CSS"},
+    {0x107, "Real time streaming"},
+    {0x108, "Drive serial number"},
+    {0x109, "Media serial number"},
+    {0x10a, "Disc control blocks"},
+    {0x10b, "DVD CPRM"},
+    {0x10c, "Firmware information"},
+    {0x10d, "AACS"},
+    {0x10e, "DVD CSS managed recording"},
+    {0x110, "VCPS"},
+    {0x113, "SecurDisc"},
+    {0x120, "BD CPS"},
+    {0x142, "OSSC"},
 };
 
 static const char *
@@ -1045,7 +1045,7 @@ main(int argc, char * argv[])
     pr2serr("In DEBUG mode, ");
     if (verbose_given && version_given) {
         pr2serr("but override: '-vV' given, zero verbose and continue\n");
-        verbose_given = false;
+        /* verbose_given = false; */
         version_given = false;
         verbose = 0;
     } else if (! verbose_given) {
