@@ -1,5 +1,5 @@
 /* A utility program originally written for the Linux OS SCSI subsystem.
- *  Copyright (C) 2004-2019 D. Gilbert
+ *  Copyright (C) 2004-2023 D. Gilbert
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
  *  the Free Software Foundation; either version 2, or (at your option)
@@ -35,7 +35,8 @@
 #include "sg_unaligned.h"
 #include "sg_pr2serr.h"
 
-static const char * version_str = "0.67 20190913";
+static const char * version_str = "0.73 20230622";
+static const char * my_name = "sg_persist: ";
 
 
 #define PRIN_RKEY_SA     0x0
@@ -55,7 +56,6 @@ static const char * version_str = "0.67 20190913";
 #define MX_TIDS 32
 #define MX_TID_LEN 256
 
-#define ME "sg_persist"
 
 #define SG_PERSIST_IN_RDONLY "SG_PERSIST_IN_RDONLY"
 
@@ -84,7 +84,7 @@ struct opts_t {
 };
 
 
-static struct option long_options[] = {
+static const struct option long_options[] = {
     {"alloc-length", required_argument, 0, 'l'},
     {"alloc_length", required_argument, 0, 'l'},
     {"clear", no_argument, 0, 'C'},
@@ -141,7 +141,7 @@ static struct option long_options[] = {
     {0, 0, 0, 0}
 };
 
-static const char * prin_sa_strs[] = {
+static const char * const prin_sa_strs[] = {
     "Read keys",
     "Read reservation",
     "Report capabilities",
@@ -153,7 +153,7 @@ static const char * prin_sa_strs[] = {
 };
 static const int num_prin_sa_strs = SG_ARRAY_SIZE(prin_sa_strs);
 
-static const char * prout_sa_strs[] = {
+static const char * const prout_sa_strs[] = {
     "Register",
     "Reserve",
     "Release",
@@ -167,7 +167,7 @@ static const char * prout_sa_strs[] = {
 };
 static const int num_prout_sa_strs = SG_ARRAY_SIZE(prout_sa_strs);
 
-static const char * pr_type_strs[] = {
+static const char * const pr_type_strs[] = {
     "obsolete [0]",
     "Write Exclusive",
     "obsolete [2]",
@@ -181,6 +181,9 @@ static const char * pr_type_strs[] = {
     "obsolete [0xd]", "obsolete [0xe]", "obsolete [0xf]",
 };
 
+static const char * const prout_s = "PR out";
+static const char * const prin_s = "PR in";
+
 
 static void
 usage(int help)
@@ -300,13 +303,13 @@ prin_work(int sg_fd, const struct opts_t * op)
             snprintf(b, sizeof(b), "service action=0x%x", op->prin_sa);
 
         if (SG_LIB_CAT_INVALID_OP == res)
-            pr2serr("PR in (%s): command not supported\n", b);
+            pr2serr("%s (%s): command not supported\n", prin_s, b);
         else if (SG_LIB_CAT_ILLEGAL_REQ == res)
-            pr2serr("PR in (%s): bad field in cdb or parameter list (perhaps "
-                    "unsupported service action)\n", b);
+            pr2serr("%s (%s): bad field in cdb or parameter list (perhaps "
+                    "unsupported service action)\n", prin_s, b);
         else {
             sg_get_category_sense_str(res, sizeof(bb), bb, op->verbose);
-            pr2serr("PR in (%s): %s\n", b, bb);
+            pr2serr("%s (%s): %s\n", prin_s, b, bb);
         }
         goto fini;
     }
@@ -516,17 +519,19 @@ prout_work(int sg_fd, struct opts_t * op)
             snprintf(b, sizeof(b), "service action=0x%x", op->prout_sa);
         if (res) {
             if (SG_LIB_CAT_INVALID_OP == res)
-                pr2serr("PR out (%s): command not supported\n", b);
+                pr2serr("%s (%s): command not supported\n", prout_s, b);
             else if (SG_LIB_CAT_ILLEGAL_REQ == res)
-                pr2serr("PR out (%s): bad field in cdb or parameter list "
-                        "(perhaps unsupported service action)\n", b);
+                pr2serr("%s (%s): bad field in cdb (perhaps unsupported "
+                        "service action)\n", prout_s, b);
+            else if (SG_LIB_CAT_INVALID_PARAM == res)
+                pr2serr("%s (%s): bad field parameter list\n", prout_s, b);
             else {
                 sg_get_category_sense_str(res, sizeof(bb), bb, op->verbose);
-                pr2serr("PR out (%s): %s\n", b, bb);
+                pr2serr("%s (%s): %s\n", prout_s, b, bb);
             }
             goto fini;
         } else if (op->verbose)
-            pr2serr("PR out: command (%s) successful\n", b);
+            pr2serr("%s: command (%s) successful\n", prout_s, b);
     }
 fini:
     if (free_pr_buff)
@@ -541,6 +546,7 @@ prout_reg_move_work(int sg_fd, struct opts_t * op)
     int res = 0;
     uint8_t * pr_buff = NULL;
     uint8_t * free_pr_buff = NULL;
+    static const char * ram_s = "register and move";
 
     t_arr_len = compact_transportid_array(op);
     pr_buff = sg_memalign(op->alloc_len, 0 /* page aligned */, &free_pr_buff,
@@ -568,19 +574,21 @@ prout_reg_move_work(int sg_fd, struct opts_t * op)
                                        pr_buff, len, true, op->verbose);
     if (res) {
        if (SG_LIB_CAT_INVALID_OP == res)
-            pr2serr("PR out (register and move): command not supported\n");
+            pr2serr("%s (%s): command not supported\n", prout_s, ram_s);
         else if (SG_LIB_CAT_ILLEGAL_REQ == res)
-            pr2serr("PR out (register and move): bad field in cdb or "
-                    "parameter list (perhaps unsupported service action)\n");
+            pr2serr("%s (%s): bad field in cdb (perhaps unsupported "
+                    "service action)\n", prout_s, ram_s);
+        else if (SG_LIB_CAT_INVALID_PARAM == res)
+            pr2serr("%s (%s): bad field in parameter list\n", prout_s, ram_s);
         else {
             char bb[80];
 
             sg_get_category_sense_str(res, sizeof(bb), bb, op->verbose);
-            pr2serr("PR out (register and move): %s\n", bb);
+            pr2serr("%s (%s): %s\n", prout_s, ram_s, bb);
         }
         goto fini;
     } else if (op->verbose)
-        pr2serr("PR out: 'register and move' command successful\n");
+        pr2serr("%s (%s): command successful\n", prout_s, ram_s);
 fini:
     if (free_pr_buff)
         free(free_pr_buff);
@@ -779,7 +787,7 @@ decode_file_tids(const char * fnp, struct opts_t * op)
             continue;
         }
         if (carry_over[0]) {
-            if (isxdigit(line[0])) {
+            if (isxdigit((uint8_t)line[0])) {
                 carry_over[1] = line[0];
                 carry_over[2] = '\0';
                 if (1 == sscanf(carry_over, "%x", &h))
@@ -826,7 +834,7 @@ decode_file_tids(const char * fnp, struct opts_t * op)
                     pr2serr("%s: array length exceeded\n", __func__);
                     goto bad;
                 }
-                op->transportid_arr[off + k] = h;  /* keep code checker happy */
+                op->transportid_arr[off + k] = h;/* keep code checker happy */
                 lcp = strpbrk(lcp, " ,\t");
                 if (NULL == lcp)
                     break;
@@ -965,6 +973,8 @@ main(int argc, char * argv[])
     op->prout_sa = -1;
     op->inquiry = true;
     op->alloc_len = MX_ALLOC_LEN;
+   if (getenv("SG3_UTILS_INVOCATION"))
+        sg_rep_invocation(my_name, version_str, argc, argv, stderr);
 
     while (1) {
         int option_index = 0;
@@ -1235,24 +1245,28 @@ main(int argc, char * argv[])
         (0 == op->prout_type)) {
         pr2serr("warning>>> --prout-type probably needs to be given\n");
     }
-    if ((op->verbose > 2) && op->num_transportids) {
-        char b[1024];
-        uint8_t * bp;
+    if (op->verbose > 2) {
+        pr2serr("Number of PROUT parameters: %d\n", num_prout_param);
+        if (op->num_transportids) {
+            char b[1024];
+            uint8_t * bp;
 
-        pr2serr("number of tranport-ids decoded from command line (or "
-                "stdin): %d\n", op->num_transportids);
-        pr2serr("  Decode given transport-ids:\n");
-        for (k = 0; k < op->num_transportids; ++k) {
-            bp = op->transportid_arr + (MX_TID_LEN * k);
-            printf("%s", sg_decode_transportid_str("      ", bp, MX_TID_LEN,
-                                                   true, sizeof(b), b));
+            pr2serr("number of tranport-ids decoded from command line (or "
+                    "stdin): %d\n", op->num_transportids);
+            pr2serr("  Decode given transport-ids:\n");
+            for (k = 0; k < op->num_transportids; ++k) {
+                bp = op->transportid_arr + (MX_TID_LEN * k);
+                printf("%s",
+                       sg_decode_transportid_str("      ", bp, MX_TID_LEN,
+                                                 true, sizeof(b), b));
+            }
         }
     }
 
     if (op->inquiry) {
         if ((sg_fd = sg_cmds_open_device(device_name, true /* ro */,
                                          op->verbose)) < 0) {
-            pr2serr("%s: error opening file (ro): %s: %s\n", ME,
+            pr2serr("%serror opening file (ro): %s: %s\n", my_name,
                     device_name, safe_strerror(-sg_fd));
             ret = sg_convert_errno(-sg_fd);
             flagged = true;
@@ -1269,7 +1283,7 @@ main(int argc, char * argv[])
             else
                 printf("  Peripheral device type: 0x%x\n", peri_type);
         } else {
-            printf("%s: SCSI INQUIRY failed on %s", ME, device_name);
+            printf("%sSCSI INQUIRY failed on %s", my_name, device_name);
             if (ret < 0) {
                 ret = -ret;
                 printf(": %s\n", safe_strerror(ret));
@@ -1279,7 +1293,9 @@ main(int argc, char * argv[])
             flagged = true;
             goto fini;
         }
-        sg_cmds_close_device(sg_fd);
+        res = sg_cmds_close_device(sg_fd);
+        if (res < 0)
+            pr2serr("%ssg_cmds_close_device() failed res=%d\n", my_name, res);
     }
 
     if (! op->readwrite_force) {
@@ -1291,7 +1307,7 @@ main(int argc, char * argv[])
         op->readonly = false;      /* '-yy' force open(RW) */
     sg_fd = sg_cmds_open_device(device_name, op->readonly, op->verbose);
     if (sg_fd < 0) {
-        pr2serr("%s: error opening file %s (r%s): %s\n", ME, device_name,
+        pr2serr("%serror opening file %s (r%s): %s\n", my_name, device_name,
                 (op->readonly ? "o" : "w"), safe_strerror(-sg_fd));
         ret = sg_convert_errno(-sg_fd);
         flagged = true;
