@@ -1,7 +1,7 @@
 /*
  * A utility program originally written for the Linux OS SCSI subsystem.
  *
- * Copyright (C) 2000-2021 Ingo van Lil <inguin@gmx.de>
+ * Copyright (C) 2000-2023 Ingo van Lil <inguin@gmx.de>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -39,7 +39,9 @@
 #include "sg_pr2serr.h"
 #include "sg_unaligned.h"
 
-#define SG_RAW_VERSION "0.4.34 (2021-01-03)"
+#define SG_RAW_VERSION "0.4.42 (2023-06-22)"
+
+static const char * my_name = "sg_raw: ";
 
 #define DEFAULT_TIMEOUT 20
 #define MIN_SCSI_CDBSZ 6
@@ -51,7 +53,7 @@
 #define NVME_DATA_LEN_DATA_IN  0xfffffffe
 #define NVME_DATA_LEN_DATA_OUT 0xfffffffd
 
-static struct option long_options[] = {
+static const struct option long_options[] = {
     { "binary",  no_argument,       NULL, 'b' },
     { "cmdfile", required_argument, NULL, 'c' },
     { "cmdset",  required_argument, NULL, 'C' },
@@ -68,6 +70,7 @@ static struct option long_options[] = {
     { "scan",    required_argument, NULL, 'Q' },
     { "send",    required_argument, NULL, 's' },
     { "timeout", required_argument, NULL, 't' },
+    { "tmo",     required_argument, NULL, 't' },
     { "verbose", no_argument,       NULL, 'v' },
     { "version", no_argument,       NULL, 'V' },
     { 0, 0, 0, 0 }
@@ -80,7 +83,7 @@ struct opts_t {
     bool do_dataout;
     bool do_enumerate;
     bool no_sense;
-    bool nvm;           /* the NVMe command set containing its READ+WRITE */
+    bool do_nvm;     /* the NVMe command set: NVM containing its READ+WRITE */
     bool do_help;
     bool verbose_given;
     bool version_given;
@@ -111,7 +114,7 @@ pr_version()
             "This is free software.  You may redistribute copies of it "
             "under the terms of\n"
             "the GNU General Public License "
-            "<http://www.gnu.org/licenses/gpl.html>.\n"
+            "<https://www.gnu.org/licenses/gpl.html>.\n"
             "There is NO WARRANTY, to the extent permitted by law.\n");
 }
 
@@ -133,17 +136,18 @@ usage()
             "DEVICE but\n"
             "                         ignores it\n"
             "  --help|-h              Show this message and exit\n"
-            "  --infile=IFILE|-i IFILE    Read data to send from IFILE "
-            "(default:\n"
-            "                             stdin)\n"
+            "  --infile=IFILE|-i IFILE    Read binary data to send (i.e. "
+            "data-out)\n"
+            "                             from IFILE (default: stdin)\n"
             "  --nosense|-n           Don't display sense information\n"
             "  --nvm|-N               command is for NVM command set (e.g. "
             "Read);\n"
             "                         default, if NVMe fd, Admin command "
             "set\n"
-            "  --outfile=OFILE|-o OFILE    Write binary data to OFILE (def: "
-            "hexdump\n"
-            "                              to stdout)\n"
+            "  --outfile=OFILE|-o OFILE    Write binary data from device "
+            "(i.e. data-in)\n"
+            "                              to OFILE (def: hexdump to "
+            "stdout)\n"
             "  --raw|-w               interpret CF (command file) as "
             "binary (def:\n"
             "                         interpret as ASCII hex)\n"
@@ -164,9 +168,9 @@ usage()
             "  --verbose|-v           Increase verbosity\n"
             "  --version|-V           Show version information and exit\n"
             "\n"
-            "Between 6 and 260 command bytes (two hex digits each) can be "
-            "specified\nand will be sent to DEVICE. Lengths RLEN, SLEN and "
-            "KLEN are decimal by\ndefault. Bidirectional commands "
+            "Between 6 and 260 command bytes (one or two hex digits each) "
+            "can be\nspecified and will be sent to DEVICE. Lengths RLEN, "
+            "SLEN and KLEN are\ndecimal by default. Bidirectional commands "
             "accepted.\n\nSimple example: Perform INQUIRY on /dev/sg0:\n"
             "  sg_raw -r 1k /dev/sg0 12 00 00 00 60 00\n");
 }
@@ -225,7 +229,7 @@ parse_cmd_line(struct opts_t * op, int argc, char *argv[])
             op->no_sense = true;
             break;
         case 'N':
-            op->nvm = true;
+            op->do_nvm = true;
             break;
         case 'o':
             if (op->datain_file) {
@@ -299,13 +303,6 @@ parse_cmd_line(struct opts_t * op, int argc, char *argv[])
         }
     }
 
-    if (optind >= argc) {
-        pr2serr("No device specified\n\n");
-        return SG_LIB_SYNTAX_ERROR;
-    }
-    op->device_name = argv[optind];
-    ++optind;
-
     if (op->version_given
 #ifdef DEBUG
         && ! op->verbose_given
@@ -313,16 +310,24 @@ parse_cmd_line(struct opts_t * op, int argc, char *argv[])
        )
         return 0;
 
+    if (optind >= argc) {
+        pr2serr("No device specified\n");
+        return SG_LIB_SYNTAX_ERROR;
+    }
+    op->device_name = argv[optind];
+    ++optind;
+
     while (optind < argc) {
         char *opt = argv[optind++];
         char *endptr;
         int cmd = strtol(opt, &endptr, 16);
+
         if (*opt == '\0' || *endptr != '\0' || cmd < 0x00 || cmd > 0xff) {
             pr2serr("Invalid command byte '%s'\n", opt);
             return SG_LIB_SYNTAX_ERROR;
         }
 
-        if (op->cdb_length > MAX_SCSI_CDBSZ) {
+        if (op->cdb_length >= MAX_SCSI_CDBSZ) {
             pr2serr("CDB too long (max. %d bytes)\n", MAX_SCSI_CDBSZ);
             return SG_LIB_SYNTAX_ERROR;
         }
@@ -379,7 +384,7 @@ parse_cmd_line(struct opts_t * op, int argc, char *argv[])
             printf("Attempt to decode cdb name: %s\n", b);
         } else
             printf(">>> Seems to be NVMe %s command\n",
-                   sg_get_nvme_opcode_name(op->cdb[0], true /* admin */,
+                   sg_get_nvme_opcode_name(op->cdb[0], ! op->do_nvm,
                                            sizeof(b), b));
     }
     return 0;
@@ -541,7 +546,7 @@ main(int argc, char *argv[])
     uint16_t sct_sc;
     uint32_t result;
     struct sg_pt_base *ptvp = NULL;
-    uint8_t sense_buffer[32];
+    uint8_t sense_buffer[32] SG_C_CPP_ZERO_INIT;
     uint8_t * dinp = NULL;
     uint8_t * doutp = NULL;
     uint8_t * free_buf_out = NULL;
@@ -554,6 +559,8 @@ main(int argc, char *argv[])
     op = &opts;
     memset(op, 0, sizeof(opts));
     op->timeout = DEFAULT_TIMEOUT;
+    if (getenv("SG3_UTILS_INVOCATION"))
+        sg_rep_invocation(my_name, SG_RAW_VERSION, argc, argv, stderr);
     ret = parse_cmd_line(op, argc, argv);
 #ifdef DEBUG
     pr2serr("In DEBUG mode, ");
@@ -577,6 +584,7 @@ main(int argc, char *argv[])
     }
 
     if (ret != 0) {
+        pr2serr("\n");  /* blank line before outputting usage */
         usage();
         goto done;
     } else if (op->do_help) {
@@ -584,6 +592,8 @@ main(int argc, char *argv[])
         goto done;
     } else if (op->do_enumerate)
         goto done;
+    if (0 == op->timeout)       /* timeout of 0 seconds is not a good idea */
+        op->timeout = DEFAULT_TIMEOUT;
 
     sg_fd = scsi_pt_open_device(op->device_name, op->readonly,
                                 op->verbose);
@@ -679,7 +689,7 @@ and_again:
             hex2stderr(op->cdb, op->cdb_length, -1);
             if (op->verbose > 1)
                 pr2serr("  Command name: %s\n",
-                        sg_get_nvme_opcode_name(op->cdb[0], true /* admin */,
+                        sg_get_nvme_opcode_name(op->cdb[0], ! op->do_nvm,
                                                 b_len, b));
         }
     }
@@ -689,7 +699,7 @@ and_again:
                 (int)sizeof(sense_buffer));
     set_scsi_pt_sense(ptvp, sense_buffer, sizeof(sense_buffer));
 
-    if (op->nvm)
+    if (op->do_nvm)
         ret = do_nvm_pt(ptvp, 0, op->timeout, op->verbose);
     else
         ret = do_scsi_pt(ptvp, -1, op->timeout, op->verbose);
@@ -732,7 +742,7 @@ and_again:
         k = -ret;
         pr2serr("do_scsi_pt: %s\n", safe_strerror(k));
         err = get_scsi_pt_os_err(ptvp);
-        if (err != k)
+        if ((err != 0) && (err != k))
             pr2serr("    ... or perhaps: %s\n", safe_strerror(err));
         ret = sg_convert_errno(err);
         goto done;
