@@ -1,5 +1,5 @@
 /* A utility program originally written for the Linux OS SCSI subsystem.
- *  Copyright (C) 1999-2019 D. Gilbert
+ *  Copyright (C) 1999-2022 D. Gilbert
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
  *  the Free Software Foundation; either version 2, or (at your option)
@@ -57,22 +57,22 @@
 #endif
 
 
-static const char * version_str = "5.07 20191226";
+static const char * version_str = "5.10 20230622";
 
-static struct option long_options[] = {
-        {"buffer", required_argument, 0, 'b'},
-        {"dio", no_argument, 0, 'd'},
-        {"echo", no_argument, 0, 'e'},
-        {"help", no_argument, 0, 'h'},
-        {"mmap", no_argument, 0, 'm'},
-        {"new", no_argument, 0, 'N'},
-        {"old", no_argument, 0, 'O'},
-        {"quick", no_argument, 0, 'q'},
-        {"size", required_argument, 0, 's'},
-        {"time", no_argument, 0, 't'},
-        {"verbose", no_argument, 0, 'v'},
-        {"version", no_argument, 0, 'V'},
-        {0, 0, 0, 0},
+static const struct option long_options[] = {
+    {"buffer", required_argument, 0, 'b'},
+    {"dio", no_argument, 0, 'd'},
+    {"echo", no_argument, 0, 'e'},
+    {"help", no_argument, 0, 'h'},
+    {"mmap", no_argument, 0, 'm'},
+    {"new", no_argument, 0, 'N'},
+    {"old", no_argument, 0, 'O'},
+    {"quick", no_argument, 0, 'q'},
+    {"size", required_argument, 0, 's'},
+    {"time", no_argument, 0, 't'},
+    {"verbose", no_argument, 0, 'v'},
+    {"version", no_argument, 0, 'V'},
+    {0, 0, 0, 0},
 };
 
 struct opts_t {
@@ -368,8 +368,8 @@ main(int argc, char * argv[])
     struct opts_t * op;
     uint8_t * rbBuff = NULL;
     void * rawp = NULL;
-    uint8_t sense_buffer[32];
-    uint8_t rb_cdb [RB_CMD_LEN];
+    uint8_t sense_buffer[32] SG_C_CPP_ZERO_INIT;
+    uint8_t rb_cdb [RB_CMD_LEN] SG_C_CPP_ZERO_INIT;
     struct sg_io_hdr io_hdr;
     struct timeval start_tm, end_tm;
     struct opts_t opts;
@@ -436,7 +436,6 @@ main(int argc, char * argv[])
     }
     rbBuff = (uint8_t *)rawp;
 
-    memset(rb_cdb, 0, RB_CMD_LEN);
     rb_cdb[0] = RB_OPCODE;
     rb_cdb[1] = op->do_echo ? RB_MODE_ECHO_DESC : RB_MODE_DESC;
     rb_cdb[8] = RB_DESC_LEN;
@@ -632,6 +631,8 @@ main(int argc, char * argv[])
 
 #ifdef DEBUG
         if (clear) {
+            int j;
+
             for (j = 0; j < buf_size; ++j) {
                 if (rbBuff[j] != 0) {
                     clear = false;
