@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2004-2021 Douglas Gilbert.
+ * Copyright (c) 2004-2023 Douglas Gilbert.
  * All rights reserved.
  * Use of this source code is governed by a BSD-style
  * license that can be found in the BSD_LICENSE file.
@@ -34,7 +34,9 @@
  * This program issues the SCSI command REQUEST SENSE to the given SCSI device.
  */
 
-static const char * version_str = "1.36 20210329";
+static const char * version_str = "1.45 20230622";
+
+static const char * my_name = "sg_requests: ";  /* REQUEST Sense command */
 
 #define MAX_REQS_RESP_LEN 255
 #define DEF_REQS_RESP_LEN 252
@@ -45,37 +47,27 @@ static const char * version_str = "1.36 20210329";
 #define REQUEST_SENSE_CMD 0x3
 #define REQUEST_SENSE_CMDLEN 6
 
-/* Not all environments support the Unix sleep() */
-#if defined(MSC_VER) || defined(__MINGW32__)
-#define HAVE_MS_SLEEP
-#endif
-#ifdef HAVE_MS_SLEEP
-#include <windows.h>
-#define sleep_for(seconds)    Sleep( (seconds) * 1000)
-#else
-#define sleep_for(seconds)    sleep(seconds)
-#endif
 
-#define ME "sg_requests: "
-
-
-static struct option long_options[] = {
-        {"desc", no_argument, 0, 'd'},
-        {"error", no_argument, 0, 'e'},
-        {"help", no_argument, 0, 'h'},
-        {"hex", no_argument, 0, 'H'},
-        {"maxlen", required_argument, 0, 'm'},
-        {"num", required_argument, 0, 'n'},
-        {"number", required_argument, 0, 'n'},
-        {"progress", no_argument, 0, 'p'},
-        {"raw", no_argument, 0, 'r'},
-        {"status", no_argument, 0, 's'},
-        {"time", no_argument, 0, 't'},
-        {"verbose", no_argument, 0, 'v'},
-        {"version", no_argument, 0, 'V'},
-        {0, 0, 0, 0},
+static const struct option long_options[] = {
+    {"desc", no_argument, 0, 'd'},
+    {"error", no_argument, 0, 'e'},
+    {"help", no_argument, 0, 'h'},
+    {"hex", no_argument, 0, 'H'},
+    {"maxlen", required_argument, 0, 'm'},
+    {"num", required_argument, 0, 'n'},
+    {"number", required_argument, 0, 'n'},
+    {"progress", no_argument, 0, 'p'},
+    {"raw", no_argument, 0, 'r'},
+    {"status", no_argument, 0, 's'},
+    {"time", no_argument, 0, 't'},
+    {"timeout", required_argument, 0, 'T'},
+    {"tmo", required_argument, 0, 'T'},
+    {"verbose", no_argument, 0, 'v'},
+    {"version", no_argument, 0, 'V'},
+    {0, 0, 0, 0},
 };
 
+
 static void
 usage()
 {
@@ -83,8 +75,9 @@ usage()
             "[--maxlen=LEN]\n"
             "                   [--num=NUM] [--number=NUM] [--progress] "
             "[--raw]\n"
-            "                   [--status] [--time] [--verbose] "
-            "[--version] DEVICE\n"
+            "                   [--status] [--time] [--timeout=SE] "
+            "[--verbose]\n"
+            "                   [--version] DEVICE\n"
             "  where:\n"
             "    --desc|-d         set flag for descriptor sense "
             "format\n"
@@ -126,7 +119,8 @@ dStrRaw(const uint8_t * str, int len)
 int
 main(int argc, char * argv[])
 {
-    int c, n, k, progress, rs, sense_cat, act_din_len;
+    int c, n, k, progress, rs, sense_cat;
+    int act_din_len = 0;
     int do_error = 0;
     int err = 0;
     int num_errs = 0;
@@ -145,6 +139,7 @@ main(int argc, char * argv[])
     int num_rs = 1;
     int do_hex = 0;
     int maxlen = 0;
+    int tmo = 0;
     int verbose = 0;
     const char * device_name = NULL;
     int ret = 0;
@@ -152,16 +147,18 @@ main(int argc, char * argv[])
     char b[256];
     uint8_t rs_cdb[REQUEST_SENSE_CMDLEN] =
         {REQUEST_SENSE_CMD, 0, 0, 0, 0, 0};
-    uint8_t sense_b[SENSE_BUFF_LEN];
+    uint8_t sense_b[SENSE_BUFF_LEN] SG_C_CPP_ZERO_INIT;
 #ifndef SG_LIB_MINGW
     bool do_time = false;
     struct timeval start_tm, end_tm;
 #endif
 
+    if (getenv("SG3_UTILS_INVOCATION"))
+        sg_rep_invocation(my_name, version_str, argc, argv, stderr);
     while (1) {
         int option_index = 0;
 
-        c = getopt_long(argc, argv, "dehHm:n:prstvV", long_options,
+        c = getopt_long(argc, argv, "dehHm:n:prstT:vV", long_options,
                         &option_index);
         if (c == -1)
             break;
@@ -189,8 +186,8 @@ main(int argc, char * argv[])
             }
             break;
         case 'n':
-           num_rs = sg_get_num(optarg);
-           if (num_rs < 1) {
+            num_rs = sg_get_num(optarg);
+            if (num_rs < 1) {
                 pr2serr("bad argument to '--num'\n");
                 return SG_LIB_SYNTAX_ERROR;
             }
@@ -209,6 +206,13 @@ main(int argc, char * argv[])
             do_time = true;
 #endif
             break;
+        case 'T':
+            tmo = sg_get_num(optarg);
+            if (tmo < 0) {
+                pr2serr("bad argument to '--timeout='\n");
+                return SG_LIB_SYNTAX_ERROR;
+            }
+            break;
         case 'v':
             verbose_given = true;
             ++verbose;
@@ -238,7 +242,7 @@ main(int argc, char * argv[])
     pr2serr("In DEBUG mode, ");
     if (verbose_given && version_given) {
         pr2serr("but override: '-vV' given, zero verbose and continue\n");
-        verbose_given = false;
+        /* verbose_given = false; */
         version_given = false;
         verbose = 0;
     } else if (! verbose_given) {
@@ -251,7 +255,7 @@ main(int argc, char * argv[])
         pr2serr("Not in DEBUG mode, so '-vV' has no special action\n");
 #endif
     if (version_given) {
-        pr2serr(ME "version: %s\n", version_str);
+        pr2serr("%sversion: %s\n", my_name, version_str);
         return 0;
     }
 
@@ -262,6 +266,8 @@ main(int argc, char * argv[])
         usage();
         return SG_LIB_SYNTAX_ERROR;
     }
+    if (0 == tmo)
+        tmo = DEF_PT_TIMEOUT;
     if (do_raw) {
         if (sg_set_binary_mode(STDOUT_FILENO) < 0) {
             perror("sg_set_binary_mode");
@@ -288,7 +294,7 @@ main(int argc, char * argv[])
     sg_fd = sg_cmds_open_device(device_name, true /* ro */, verbose);
     if (sg_fd < 0) {
         if (not_raw_hex && verbose)
-            pr2serr(ME "open error: %s: %s\n", device_name,
+            pr2serr("%sopen error: %s: %s\n", my_name, device_name,
                     safe_strerror(-sg_fd));
         ret = sg_convert_errno(-sg_fd);
         goto finish;
@@ -309,7 +315,7 @@ main(int argc, char * argv[])
         for (k = 0; k < num_rs; ++k) {
             act_din_len = 0;
             if (k > 0)
-                sleep_for(30);
+                sg_sleep_secs(30);
             set_scsi_pt_cdb(ptvp, rs_cdb, sizeof(rs_cdb));
             set_scsi_pt_sense(ptvp, sense_b, sizeof(sense_b));
             memset(rsBuff, 0x0, sizeof(rsBuff));
@@ -320,18 +326,21 @@ main(int argc, char * argv[])
                 n = 0;
             } else {
                 if (verbose && (0 == k)) {
-                    char b[128];
+                    char bb[128];
 
                     pr2serr("    cdb: %s\n",
                             sg_get_command_str(rs_cdb, REQUEST_SENSE_CMDLEN,
-                                               true, sizeof(b), b));
+                                               true, sizeof(bb), bb));
                 }
-                rs = do_scsi_pt(ptvp, -1, DEF_PT_TIMEOUT, verbose);
+                rs = do_scsi_pt(ptvp, -1, tmo, verbose);
                 n = sg_cmds_process_resp(ptvp, "Request sense", rs, (0 == k),
                                          verbose, &sense_cat);
             }
             if (-1 == n) {
-                ret = sg_convert_errno(get_scsi_pt_os_err(ptvp));
+                if (get_scsi_pt_transport_err(ptvp))
+                    ret = SG_LIB_TRANSPORT_ERROR;
+                else
+                    ret = sg_convert_errno(get_scsi_pt_os_err(ptvp));
                 goto finish;
             } else if (-2 == n) {
                 switch (sense_cat) {
@@ -413,18 +422,21 @@ main(int argc, char * argv[])
             n = 0;
         } else {
             if (verbose && (0 == k)) {
-                char b[128];
+                char bb[128];
 
                 pr2serr("    cdb: %s\n",
                         sg_get_command_str(rs_cdb, REQUEST_SENSE_CMDLEN,
-                                           true, sizeof(b), b));
+                                           true, sizeof(bb), bb));
             }
-            rs = do_scsi_pt(ptvp, -1, DEF_PT_TIMEOUT, verbose);
+            rs = do_scsi_pt(ptvp, -1, tmo, verbose);
             n = sg_cmds_process_resp(ptvp, "Request sense", rs, (0 == k),
                                      verbose, &sense_cat);
         }
         if (-1 == n) {
-            ret = sg_convert_errno(get_scsi_pt_os_err(ptvp));
+            if (get_scsi_pt_transport_err(ptvp))
+                ret = SG_LIB_TRANSPORT_ERROR;
+            else
+                ret = sg_convert_errno(get_scsi_pt_os_err(ptvp));
             goto finish;
         } else if (-2 == n) {
             switch (sense_cat) {
@@ -468,11 +480,11 @@ main(int argc, char * argv[])
                     most_recent_skey = ssh.sense_key;
                 }
                 if (not_raw_hex && ((1 == num_rs) || verbose)) {
-                    char b[144];
+                    char bb[144];
 
                     sg_get_sense_str(NULL, rsBuff, act_din_len,
-                                     false, sizeof(b), b);
-                    pr2serr("data-in decoded as sense:\n%s\n", b);
+                                     false, sizeof(bb), bb);
+                    pr2serr("data-in decoded as sense:\n%s\n", bb);
                 }
             }
         }
