@@ -1,8 +1,10 @@
 /*
- * Copyright (c) 2004-2018 Christophe Varoqui and Douglas Gilbert.
+ * Copyright (c) 2004-2023 Christophe Varoqui and Douglas Gilbert.
  * All rights reserved.
  * Use of this source code is governed by a BSD-style
  * license that can be found in the BSD_LICENSE file.
+ *
+ * SPDX-License-Identifier: BSD-2-Clause
  */
 
 #include <unistd.h>
@@ -31,7 +33,7 @@
  * to the given SCSI device.
  */
 
-static const char * version_str = "1.27 20180628";
+static const char * version_str = "1.29 20230622";
 
 #define REPORT_TGT_GRP_BUFF_LEN 1024
 
@@ -47,16 +49,16 @@ static const char * version_str = "1.27 20180628";
 #define STATUS_CODE_CHANGED_BY_SET 0x1
 #define STATUS_CODE_CHANGED_BY_IMPLICIT 0x2
 
-static struct option long_options[] = {
-        {"decode", no_argument, 0, 'd'},
-        {"extended", no_argument, 0, 'e'},
-        {"help", no_argument, 0, 'h'},
-        {"hex", no_argument, 0, 'H'},
-        {"raw", no_argument, 0, 'r'},
-        {"readonly", no_argument, 0, 'R'},
-        {"verbose", no_argument, 0, 'v'},
-        {"version", no_argument, 0, 'V'},
-        {0, 0, 0, 0},
+static const struct option long_options[] = {
+    {"decode", no_argument, 0, 'd'},
+    {"extended", no_argument, 0, 'e'},
+    {"help", no_argument, 0, 'h'},
+    {"hex", no_argument, 0, 'H'},
+    {"raw", no_argument, 0, 'r'},
+    {"readonly", no_argument, 0, 'R'},
+    {"verbose", no_argument, 0, 'v'},
+    {"version", no_argument, 0, 'V'},
+    {0, 0, 0, 0},
 };
 
 
@@ -151,11 +153,11 @@ main(int argc, char * argv[])
     bool extended = false;
     bool verbose_given = false;
     bool version_given = false;
-    int k, j, off, res, c, report_len, tgt_port_count;
+    int k, j, off, res, c, report_len, buff_len, tgt_port_count;
     int sg_fd = -1;
     int ret = 0;
     int verbose = 0;
-    uint8_t reportTgtGrpBuff[REPORT_TGT_GRP_BUFF_LEN];
+    uint8_t * reportTgtGrpBuff = NULL;
     uint8_t * bp;
     const char * device_name = NULL;
 
@@ -216,7 +218,7 @@ main(int argc, char * argv[])
     pr2serr("In DEBUG mode, ");
     if (verbose_given && version_given) {
         pr2serr("but override: '-vV' given, zero verbose and continue\n");
-        verbose_given = false;
+        /* verbose_given = false; */
         version_given = false;
         verbose = 0;
     } else if (! verbose_given) {
@@ -253,20 +255,26 @@ main(int argc, char * argv[])
         goto err_out;
     }
 
-    memset(reportTgtGrpBuff, 0x0, sizeof(reportTgtGrpBuff));
-    /* trunc = 0; */
+    buff_len = REPORT_TGT_GRP_BUFF_LEN;
+
+retry:
+    reportTgtGrpBuff = (uint8_t *)malloc(buff_len);
+    if (NULL == reportTgtGrpBuff) {
+        pr2serr("    Out of memory (ram)\n");
+        goto err_out;
+    }
+    memset(reportTgtGrpBuff, 0x0, buff_len);
 
     res = sg_ll_report_tgt_prt_grp2(sg_fd, reportTgtGrpBuff,
-                                    sizeof(reportTgtGrpBuff),
+                                    buff_len,
                                     extended, true, verbose);
     ret = res;
     if (0 == res) {
         report_len = sg_get_unaligned_be32(reportTgtGrpBuff + 0) + 4;
-        if (report_len > (int)sizeof(reportTgtGrpBuff)) {
-            /* trunc = 1; */
-            pr2serr("  <<report too long for internal buffer, output "
-                    "truncated\n");
-            report_len = (int)sizeof(reportTgtGrpBuff);
+        if (report_len > buff_len) {
+            free(reportTgtGrpBuff);
+            buff_len = report_len;
+            goto retry;
         }
         if (raw) {
             dStrRaw(reportTgtGrpBuff, report_len);
@@ -352,6 +360,8 @@ err_out:
                 ret = sg_convert_errno(-res);
         }
     }
+    if (reportTgtGrpBuff)
+        free(reportTgtGrpBuff);
     if (0 == verbose) {
         if (! sg_if_can2stderr("sg_rtpg failed: ", ret))
             pr2serr("Some error occurred, try again with '-v' "
