@@ -1,8 +1,10 @@
 /*
- * Copyright (c) 2004-2018 Hannes Reinecke and Douglas Gilbert.
+ * Copyright (c) 2004-2023 Hannes Reinecke and Douglas Gilbert.
  * All rights reserved.
  * Use of this source code is governed by a BSD-style
  * license that can be found in the BSD_LICENSE file.
+ *
+ * SPDX-License-Identifier: BSD-2-Clause
  */
 
 #include <unistd.h>
@@ -32,7 +34,7 @@
  *  to the 'SCSI Accessed Fault-Tolerant Enclosures' (SAF-TE) spec.
  */
 
-static const char * version_str = "0.33 20180628";
+static const char * version_str = "0.34 20230519";
 
 
 #define SENSE_BUFF_LEN 64       /* Arbitrary, could be larger */
@@ -494,7 +496,7 @@ usage()
             "Queries a SAF-TE processor device\n");
 }
 
-static struct option long_options[] = {
+static const struct option long_options[] = {
     {"config", 0, 0, 'c'},
     {"devstatus", 0, 0, 'd'},
     {"encstatus", 0, 0, 's'},
@@ -604,7 +606,7 @@ main(int argc, char * argv[])
     pr2serr("In DEBUG mode, ");
     if (verbose_given && version_given) {
         pr2serr("but override: '-vV' given, zero verbose and continue\n");
-        verbose_given = false;
+        /* verbose_given = false; */
         version_given = false;
         verbose = 0;
     } else if (! verbose_given) {
