@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2011-2020 Douglas Gilbert.
+ * Copyright (c) 2011-2023 Douglas Gilbert.
  * All rights reserved.
  * Use of this source code is governed by a BSD-style
  * license that can be found in the BSD_LICENSE file.
@@ -33,19 +33,7 @@
 #include "sg_unaligned.h"
 #include "sg_pr2serr.h"
 
-static const char * version_str = "1.15 20201223";
-
-/* Not all environments support the Unix sleep() */
-#if defined(MSC_VER) || defined(__MINGW32__)
-#define HAVE_MS_SLEEP
-#endif
-#ifdef HAVE_MS_SLEEP
-#include <windows.h>
-#define sleep_for(seconds)    Sleep( (seconds) * 1000)
-#else
-#define sleep_for(seconds)    sleep(seconds)
-#endif
-
+static const char * version_str = "1.22 20230622";
 
 #define ME "sg_sanitize: "
 
@@ -66,7 +54,7 @@ static const char * version_str = "1.15 20201223";
 #define POLL_DURATION_SECS 60
 
 
-static struct option long_options[] = {
+static const struct option long_options[] = {
     {"ause", no_argument, 0, 'A'},
     {"block", no_argument, 0, 'B'},
     {"count", required_argument, 0, 'c'},
@@ -84,6 +72,7 @@ static struct option long_options[] = {
     {"quick", no_argument, 0, 'Q'},
     {"test", required_argument, 0, 'T'},
     {"timeout", required_argument, 0, 't'},
+    {"tmo", required_argument, 0, 't'},
     {"verbose", no_argument, 0, 'v'},
     {"version", no_argument, 0, 'V'},
     {"wait", no_argument, 0, 'w'},
@@ -138,7 +127,7 @@ usage()
           "field\n"
           "                         (def: clear 'desc' field)\n"
           "    --dry-run|-D         to preparation but bypass SANITIZE "
-          "commnd\n"
+          "command\n"
           "    --early|-e           exit once sanitize started (IMMED set "
           "in cdb)\n"
           "                         user can monitor progress with REQUEST "
@@ -185,7 +174,7 @@ do_sanitize(int sg_fd, const struct opts_t * op, const void * param_lstp,
     bool immed;
     int ret, res, sense_cat, timeout;
     uint8_t san_cdb[SANITIZE_OP_LEN];
-    uint8_t sense_b[SENSE_BUFF_LEN];
+    uint8_t sense_b[SENSE_BUFF_LEN] SG_C_CPP_ZERO_INIT;
     struct sg_pt_base * ptvp;
 
     if (op->early || op->wait)
@@ -245,9 +234,12 @@ do_sanitize(int sg_fd, const struct opts_t * op, const void * param_lstp,
     res = do_scsi_pt(ptvp, sg_fd, timeout, op->verbose);
     ret = sg_cmds_process_resp(ptvp, "Sanitize", res, true /*noisy */,
                                op->verbose, &sense_cat);
-    if (-1 == ret)
-        ret = sg_convert_errno(get_scsi_pt_os_err(ptvp));
-    else if (-2 == ret) {
+    if (-1 == ret) {
+        if (get_scsi_pt_transport_err(ptvp))
+            ret = SG_LIB_TRANSPORT_ERROR;
+        else
+            ret = sg_convert_errno(get_scsi_pt_os_err(ptvp));
+    } else if (-2 == ret) {
         switch (sense_cat) {
         case SG_LIB_CAT_RECOVERED:
         case SG_LIB_CAT_NO_SENSE:
@@ -267,6 +259,11 @@ do_sanitize(int sg_fd, const struct opts_t * op, const void * param_lstp,
             }
             ret = sense_cat;
             break;
+        case SG_LIB_CAT_INVALID_PARAM:
+            pr2serr("Invalid field in parameter list, add '-vv' to find "
+                    "invalid field location\n");
+            ret = sense_cat;
+            break;
         default:
             ret = sense_cat;
             break;
@@ -365,7 +362,7 @@ print_dev_id(int fd, uint8_t * sinq_resp, int max_rlen, int verbose)
         n = SAFE_STD_INQ_RESP_LEN;
     memcpy(sinq_resp, b, (n < max_rlen) ? n : max_rlen);
     if (n == SAFE_STD_INQ_RESP_LEN) {
-        pdt = b[0] & 0x1f;
+        pdt = b[0] & PDT_MASK;
         printf("    %.8s  %.16s  %.4s   peripheral_type: %s [0x%x]\n",
                (const char *)(b + 8), (const char *)(b + 16),
                (const char *)(b + 32),
@@ -717,20 +714,8 @@ main(int argc, char * argv[])
         sg_put_unaligned_be16((uint16_t)op->ipl, wBuff + 2);
     }
 
-    if ((! op->quick) && (! op->fail)) {
-        printf("\nA SANITIZE will commence in 15 seconds\n");
-        printf("    ALL data on %s will be DESTROYED\n", device_name);
-        printf("        Press control-C to abort\n");
-        sleep_for(5);
-        printf("\nA SANITIZE will commence in 10 seconds\n");
-        printf("    ALL data on %s will be DESTROYED\n", device_name);
-        printf("        Press control-C to abort\n");
-        sleep_for(5);
-        printf("\nA SANITIZE will commence in 5 seconds\n");
-        printf("    ALL data on %s will be DESTROYED\n", device_name);
-        printf("        Press control-C to abort\n");
-        sleep_for(5);
-    }
+    if ((! op->quick) && (! op->fail))
+        sg_warn_and_wait("SANITIZE", device_name, true);
 
     ret = do_sanitize(sg_fd, op, wBuff, param_lst_len);
     if (ret) {
@@ -744,7 +729,7 @@ main(int argc, char * argv[])
                 pr2serr("Due to --dry-run option, leave poll loop\n");
                 break;
             }
-            sleep_for(POLL_DURATION_SECS);
+            sg_sleep_secs(POLL_DURATION_SECS);
             memset(rsBuff, 0x0, sizeof(rsBuff));
             res = sg_ll_request_sense(sg_fd, op->desc, rsBuff, sizeof(rsBuff),
                                       1, vb);
