@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2006-2018 Douglas Gilbert.
+ * Copyright (c) 2006-2023 Douglas Gilbert.
  * All rights reserved.
  * Use of this source code is governed by a BSD-style
  * license that can be found in the BSD_LICENSE file.
@@ -30,7 +30,7 @@
 
 /* This program uses a ATA PASS-THROUGH SCSI command to package an
  * ATA IDENTIFY (PACKAGE) DEVICE command. It is based on the SCSI to
- * ATA Translation (SAT) drafts and standards. See http://www.t10.org
+ * ATA Translation (SAT) drafts and standards. See https://www.t10.org
  * for drafts. SAT is a standard: SAT ANSI INCITS 431-2007 (draft prior
  * to that is sat-r09.pdf). SAT-2 is also a standard: SAT-2 ANSI INCITS
  * 465-2010 and the draft prior to that is sat2r09.pdf . The SAT-3 is
@@ -54,22 +54,22 @@
 
 #define EBUFF_SZ 256
 
-static const char * version_str = "1.17 20180515";
+static const char * version_str = "1.22 20230622";
 
-static struct option long_options[] = {
-        {"ck-cond", no_argument, 0, 'c'},
-        {"ck_cond", no_argument, 0, 'c'},
-        {"extend", no_argument, 0, 'e'},
-        {"help", no_argument, 0, 'h'},
-        {"hex", no_argument, 0, 'H'},
-        {"len", required_argument, 0, 'l'},
-        {"ident", no_argument, 0, 'i'},
-        {"packet", no_argument, 0, 'p'},
-        {"raw", no_argument, 0, 'r'},
-        {"readonly", no_argument, 0, 'R'},
-        {"verbose", no_argument, 0, 'v'},
-        {"version", no_argument, 0, 'V'},
-        {0, 0, 0, 0},
+static const struct option long_options[] = {
+    {"ck-cond", no_argument, 0, 'c'},
+    {"ck_cond", no_argument, 0, 'c'},
+    {"extend", no_argument, 0, 'e'},
+    {"help", no_argument, 0, 'h'},
+    {"hex", no_argument, 0, 'H'},
+    {"len", required_argument, 0, 'l'},
+    {"ident", no_argument, 0, 'i'},
+    {"packet", no_argument, 0, 'p'},
+    {"raw", no_argument, 0, 'r'},
+    {"readonly", no_argument, 0, 'R'},
+    {"verbose", no_argument, 0, 'v'},
+    {"version", no_argument, 0, 'V'},
+    {0, 0, 0, 0},
 };
 
 
@@ -119,8 +119,8 @@ do_identify_dev(int sg_fd, bool do_packet, int cdb_len, bool ck_cond,
                 bool extend, bool do_ident, int do_hex, bool do_raw,
                 int verbose)
 {
-    bool t_type = false;/* false -> 512 byte blocks,
-                           true -> device's LB size */
+    const bool t_type = false;/* false -> 512 byte blocks,
+                                 true -> device's LB size */
     bool t_dir = true;  /* false -> to device, true -> from device */
     bool byte_block = true; /* false -> bytes, true -> 512 byte blocks (if
                                t_type=false) */
@@ -136,21 +136,18 @@ do_identify_dev(int sg_fd, bool do_packet, int cdb_len, bool ck_cond,
     uint64_t ull;
     struct sg_scsi_sense_hdr ssh;
     uint8_t inBuff[ID_RESPONSE_LEN];
-    uint8_t sense_buffer[64];
-    uint8_t ata_return_desc[16];
+    uint8_t sense_buffer[64] SG_C_CPP_ZERO_INIT;
+    uint8_t ata_return_desc[16] SG_C_CPP_ZERO_INIT;
     uint8_t apt_cdb[SAT_ATA_PASS_THROUGH16_LEN] =
                 {SAT_ATA_PASS_THROUGH16, 0, 0, 0, 0, 0, 0, 0,
                  0, 0, 0, 0, 0, 0, 0, 0};
     uint8_t apt12_cdb[SAT_ATA_PASS_THROUGH12_LEN] =
                 {SAT_ATA_PASS_THROUGH12, 0, 0, 0, 0, 0, 0, 0,
                  0, 0, 0, 0};
-    uint8_t apt32_cdb[SAT_ATA_PASS_THROUGH32_LEN];
+    uint8_t apt32_cdb[SAT_ATA_PASS_THROUGH32_LEN] SG_C_CPP_ZERO_INIT;
     const unsigned short * usp;
 
     sb_sz = sizeof(sense_buffer);
-    memset(sense_buffer, 0, sb_sz);
-    memset(apt32_cdb, 0, sizeof(apt32_cdb));
-    memset(ata_return_desc, 0, sizeof(ata_return_desc));
     ok = false;
     switch (cdb_len) {
     case SAT_ATA_PASS_THROUGH32_LEN:    /* SAT-4 revision 5 or later */
@@ -224,8 +221,12 @@ do_identify_dev(int sg_fd, bool do_packet, int cdb_len, bool ck_cond,
     }
     if (0 == res) {
         ok = true;
-        if (verbose > 2)
+        if (verbose > 2) {
             pr2serr("command completed with SCSI GOOD status\n");
+            if (verbose > 2)
+                pr2serr("    requested_length=%d, resid=%d\n",
+                        ID_RESPONSE_LEN, resid);
+        }
     } else if ((res > 0) && (res & SAM_STAT_CHECK_CONDITION)) {
         if (verbose > 1) {
             pr2serr("ATA pass-through:\n");
@@ -485,7 +486,7 @@ main(int argc, char * argv[])
     pr2serr("In DEBUG mode, ");
     if (verbose_given && version_given) {
         pr2serr("but override: '-vV' given, zero verbose and continue\n");
-        verbose_given = false;
+        /* verbose_given = false; */
         version_given = false;
         verbose = 0;
     } else if (! verbose_given) {
