@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2006-2018 Douglas Gilbert.
+ * Copyright (c) 2006-2023 Douglas Gilbert.
  * All rights reserved.
  * Use of this source code is governed by a BSD-style
  * license that can be found in the BSD_LICENSE file.
@@ -28,11 +28,11 @@
 #include "sg_cmds_extra.h"
 #include "sg_pr2serr.h"
 
-static const char * version_str = "1.13 20180628";
+static const char * version_str = "1.17 20230622";
 
 /* This program uses a ATA PASS-THROUGH SCSI command. This usage is
  * defined in the SCSI to ATA Translation (SAT) drafts and standards.
- * See http://www.t10.org for drafts. SAT is a standard: SAT ANSI INCITS
+ * See https://www.t10.org for drafts. SAT is a standard: SAT ANSI INCITS
  * 431-2007 (draft prior to that is sat-r09.pdf). SAT-2 is also a
  * standard: SAT-2 ANSI INCITS 465-2010 and the draft prior to that is
  * sat2r09.pdf . The SAT-3 project has started and the most recent draft
@@ -63,19 +63,19 @@ static const char * version_str = "1.13 20180628";
 
 #define EBUFF_SZ 256
 
-static struct option long_options[] = {
-        {"ck_cond", no_argument, 0, 'c'},
-        {"ck-cond", no_argument, 0, 'c'},
-        {"extend", no_argument, 0, 'e'},
-        {"hex", no_argument, 0, 'H'},
-        {"ignore", no_argument, 0, 'i'},
-        {"len", no_argument, 0, 'l'},
-        {"raw", no_argument, 0, 'r'},
-        {"reset", no_argument, 0, 'R'},
-        {"help", no_argument, 0, 'h'},
-        {"verbose", no_argument, 0, 'v'},
-        {"version", no_argument, 0, 'V'},
-        {0, 0, 0, 0},
+static const struct option long_options[] = {
+    {"ck_cond", no_argument, 0, 'c'},
+    {"ck-cond", no_argument, 0, 'c'},
+    {"extend", no_argument, 0, 'e'},
+    {"hex", no_argument, 0, 'H'},
+    {"ignore", no_argument, 0, 'i'},
+    {"len", no_argument, 0, 'l'},
+    {"raw", no_argument, 0, 'r'},
+    {"reset", no_argument, 0, 'R'},
+    {"help", no_argument, 0, 'h'},
+    {"verbose", no_argument, 0, 'v'},
+    {"version", no_argument, 0, 'V'},
+    {0, 0, 0, 0},
 };
 
 struct phy_event_t {
@@ -83,7 +83,8 @@ struct phy_event_t {
     const char * desc;
 };
 
-static struct phy_event_t phy_event_arr[] = {   /* SATA 2.5 section 13.7.2 */
+/* SATA 2.5 section 13.7.2 */
+static const struct phy_event_t phy_event_arr[] = {
     {0x1, "Command failed and ICRC error bit set in Error register"}, /* M */
     {0x2, "R_ERR(p) response for data FIS"},
     {0x3, "R_ERR(p) response for device-to-host data FIS"},
@@ -106,6 +107,7 @@ static struct phy_event_t phy_event_arr[] = {   /* SATA 2.5 section 13.7.2 */
     {0x0, NULL},        /* end marker */        /* M(andatory) */
 };
 
+
 static void
 usage()
 {
@@ -154,15 +156,17 @@ dStrRaw(const uint8_t * str, int len)
 }
 
 /* ATA READ LOG EXT command [2Fh, PIO data-in] */
-/* N.B. "log_addr" is the log page number, "page_in_log" is usually false */
+/* N.B. "log_addr" is the log page number, "page_in_log" is usually 0 */
 static int
-do_read_log_ext(int sg_fd, int log_addr, bool page_in_log, int feature,
+do_read_log_ext(int sg_fd, int log_addr, int page_in_log, int feature,
                 int blk_count, void * resp, int mx_resp_len, int cdb_len,
                 bool ck_cond, bool extend, int do_hex, bool do_raw,
                 int verbose)
 {
     /* Following for ATA READ/WRITE MULTIPLE (EXT) cmds, normally 0 */
+#if 0
     bool t_type = false;/* false -> 512 byte LBs, true -> device's LB size */
+#endif
     bool t_dir = true;  /* false -> to device, 1 -> from device */
     bool byte_block = true; /* false -> bytes, true -> 512 byte blocks (if
                                t_type=false) */
@@ -175,8 +179,8 @@ do_read_log_ext(int sg_fd, int log_addr, bool page_in_log, int feature,
     int resid = 0;
     int sb_sz;
     struct sg_scsi_sense_hdr ssh;
-    uint8_t sense_buffer[64];
-    uint8_t ata_return_desc[16];
+    uint8_t sense_buffer[64] SG_C_CPP_ZERO_INIT;
+    uint8_t ata_return_desc[16] SG_C_CPP_ZERO_INIT;
     uint8_t apt_cdb[SAT_ATA_PASS_THROUGH16_LEN] =
                 {SAT_ATA_PASS_THROUGH16, 0, 0, 0, 0, 0, 0, 0,
                  0, 0, 0, 0, 0, 0, 0, 0};
@@ -185,8 +189,6 @@ do_read_log_ext(int sg_fd, int log_addr, bool page_in_log, int feature,
                  0, 0, 0, 0};
 
     sb_sz = sizeof(sense_buffer);
-    memset(sense_buffer, 0, sb_sz);
-    memset(ata_return_desc, 0, sizeof(ata_return_desc));
     ok = false;
     if (SAT_ATA_PASS_THROUGH16_LEN == cdb_len) {
         /* Prepare ATA PASS-THROUGH COMMAND (16) command */
@@ -205,8 +207,10 @@ do_read_log_ext(int sg_fd, int log_addr, bool page_in_log, int feature,
         apt_cdb[2] = t_length;
         if (ck_cond)
             apt_cdb[2] |= 0x20;
+#if 0
         if (t_type)
             apt_cdb[2] |= 0x10;
+#endif
         if (t_dir)
             apt_cdb[2] |= 0x8;
         if (byte_block)
@@ -226,8 +230,10 @@ do_read_log_ext(int sg_fd, int log_addr, bool page_in_log, int feature,
         apt12_cdb[2] = t_length;
         if (ck_cond)
             apt12_cdb[2] |= 0x20;
+#if 0
         if (t_type)
             apt12_cdb[2] |= 0x10;
+#endif
         if (t_dir)
             apt12_cdb[2] |= 0x8;
         if (byte_block)
@@ -451,7 +457,7 @@ int main(int argc, char * argv[])
     pr2serr("In DEBUG mode, ");
     if (verbose_given && version_given) {
         pr2serr("but override: '-vV' given, zero verbose and continue\n");
-        verbose_given = false;
+        /* verbose_given = false; */
         version_given = false;
         verbose = 0;
     } else if (! verbose_given) {
@@ -487,7 +493,7 @@ int main(int argc, char * argv[])
         return sg_convert_errno(err);
     }
     ret = do_read_log_ext(sg_fd, SATA_PHY_EVENT_LPAGE,
-                          false /* page_in_log */,
+                          0 /* page_in_log */,
                           (reset ? 1 : 0) /* feature */,
                           1 /* blk_count */, inBuff,
                           READ_LOG_EXT_RESPONSE_LEN, cdb_len, ck_cond,
