@@ -1,5 +1,5 @@
 /* A utility program originally written for the Linux OS SCSI subsystem.
- *  Copyright (C) 1999 - 2018 D. Gilbert
+ *  Copyright (C) 1999 - 2023 D. Gilbert
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
  *  the Free Software Foundation; either version 2, or (at your option)
@@ -53,7 +53,7 @@
 #include "sg_pr2serr.h"
 
 
-static const char * version_str = "4.17 20180219";
+static const char * version_str = "4.19 20230623";
 
 #define ME "sg_scan: "
 
@@ -67,7 +67,7 @@ static const char * version_str = "4.17 20180219";
 #define FNAME_SZ 64
 #define PRESENT_ARRAY_SIZE 8192
 
-static const char * sysfs_sg_dir = "/sys/class/scsi_generic";
+static const char * const sysfs_sg_dir = "/sys/class/scsi_generic";
 static int * gen_index_arr;
 
 typedef struct my_scsi_idlun {
@@ -203,6 +203,7 @@ int main(int argc, char * argv[])
         printf(ME "Out of memory\n");
         return SG_LIB_CAT_OTHER;
     }
+    strcpy(fname, "<null>");
 
     for (k = 1, j = 0; k < argc; ++k) {
         cp = argv[k];
@@ -270,7 +271,7 @@ int main(int argc, char * argv[])
 
     flags = O_NONBLOCK | (writeable ? O_RDWR : O_RDONLY);
 
-    for (k = 0, res = 0, j = 0, sg_fd = -1;
+    for (k = 0, res = 0, j = 0;
          (k < max_file_args)  && (has_file_args || (num_errors < MAX_ERRORS));
          ++k, res = ((sg_fd >= 0) ? close(sg_fd) : 0)) {
         if (res < 0) {
@@ -384,7 +385,7 @@ int main(int argc, char * argv[])
                     ++num_errors;
             }
         }
-    }
+    }           /* end of large for loop <<<<<<<<<<<<<<<<< */
     if ((num_errors >= MAX_ERRORS) && (num_silent < num_errors) &&
         (! has_file_args)) {
         printf("Stopping because there are too many error\n");
@@ -398,10 +399,9 @@ int sg3_inq(int sg_fd, uint8_t * inqBuff, bool do_extra)
 {
     bool ok;
     int err, sg_io;
-    uint8_t sense_buffer[32];
-    struct sg_io_hdr io_hdr;
+    uint8_t sense_buffer[32] SG_C_CPP_ZERO_INIT;
+    struct sg_io_hdr io_hdr SG_C_CPP_ZERO_INIT;
 
-    memset(&io_hdr, 0, sizeof(struct sg_io_hdr));
     memset(inqBuff, 0, INQ_REPLY_LEN);
     inqBuff[0] = 0x7f;
     io_hdr.interface_id = 'S';
@@ -451,7 +451,7 @@ int sg3_inq(int sg_fd, uint8_t * inqBuff, bool do_extra)
         printf("    %.8s  %.16s  %.4s ", p + 8, p + 16, p + 32);
         printf("[rmb=%d cmdq=%d pqual=%d pdev=0x%x] ",
                !!(p[1] & 0x80), !!(p[7] & 2), (p[0] & 0xe0) >> 5,
-               (p[0] & 0x1f));
+               (p[0] & PDT_MASK));
         if (do_extra && sg_io)
             printf("dur=%ums\n", io_hdr.duration);
         else
