@@ -1,5 +1,5 @@
 /*
-* Copyright (c) 2004-2018 Hannes Reinecke, Christophe Varoqui, Douglas Gilbert
+ * Copyright (c) 2004-2023 Hannes Reinecke, Christophe Varoqui, Douglas Gilbert
  * All rights reserved.
  * Use of this source code is governed by a BSD-style
  * license that can be found in the BSD_LICENSE file.
@@ -34,7 +34,7 @@
  * to the given SCSI device.
  */
 
-static const char * version_str = "1.19 20180628";
+static const char * version_str = "1.23 20230623";
 
 #define TGT_GRP_BUFF_LEN 1024
 #define MX_ALLOC_LEN (0xc000 + 0x80)
@@ -58,12 +58,12 @@ static const uint8_t state_sup_mask[] = {
 #else
 
 static const uint8_t state_sup_mask[] = {
-        [TPGS_STATE_OPTIMIZED]     = 0x01,
-        [TPGS_STATE_NONOPTIMIZED]  = 0x02,
-        [TPGS_STATE_STANDBY]       = 0x04,
-        [TPGS_STATE_UNAVAILABLE]   = 0x08,
-        [TPGS_STATE_OFFLINE]       = 0x40,
-        [TPGS_STATE_TRANSITIONING] = 0x80,
+    [TPGS_STATE_OPTIMIZED]     = 0x01,
+    [TPGS_STATE_NONOPTIMIZED]  = 0x02,
+    [TPGS_STATE_STANDBY]       = 0x04,
+    [TPGS_STATE_UNAVAILABLE]   = 0x08,
+    [TPGS_STATE_OFFLINE]       = 0x40,
+    [TPGS_STATE_TRANSITIONING] = 0x80,
 };
 
 #endif  /* C or C++ ? */
@@ -74,25 +74,25 @@ static const uint8_t state_sup_mask[] = {
 #define MAX_PORT_LIST_ARR_LEN 16
 
 struct tgtgrp {
-        int id;
-        int current;
-        int valid;
+    int id;
+    int current;
+    int valid;
 };
 
-static struct option long_options[] = {
-        {"active", no_argument, 0, 'a'},
-        {"help", no_argument, 0, 'h'},
-        {"hex", no_argument, 0, 'H'},
-        {"offline", no_argument, 0, 'l'},
-        {"optimized", no_argument, 0, 'o'},
-        {"raw", no_argument, 0, 'r'},
-        {"standby", no_argument, 0, 's'},
-        {"state", required_argument, 0, 'S'},
-        {"tp", required_argument, 0, 't'},
-        {"unavailable", no_argument, 0, 'u'},
-        {"verbose", no_argument, 0, 'v'},
-        {"version", no_argument, 0, 'V'},
-        {0, 0, 0, 0},
+static const struct option long_options[] = {
+    {"active", no_argument, 0, 'a'},
+    {"help", no_argument, 0, 'h'},
+    {"hex", no_argument, 0, 'H'},
+    {"offline", no_argument, 0, 'l'},
+    {"optimized", no_argument, 0, 'o'},
+    {"raw", no_argument, 0, 'r'},
+    {"standby", no_argument, 0, 's'},
+    {"state", required_argument, 0, 'S'},
+    {"tp", required_argument, 0, 't'},
+    {"unavailable", no_argument, 0, 'u'},
+    {"verbose", no_argument, 0, 'v'},
+    {"version", no_argument, 0, 'V'},
+    {0, 0, 0, 0},
 };
 
 static void
@@ -142,15 +142,14 @@ dStrRaw(const uint8_t * str, int len)
 static int
 decode_target_port(uint8_t * buff, int len, int *d_id, int *d_tpg)
 {
-    int c_set, assoc, desig_type, i_len;
-    int off, u;
+    int c_set, assoc, desig_type, i_len, off;
     const uint8_t * bp;
     const uint8_t * ip;
 
     *d_id = -1;
     *d_tpg = -1;
     off = -1;
-    while ((u = sg_vpd_dev_id_iter(buff, len, &off, -1, -1, -1)) == 0) {
+    while (sg_vpd_dev_id_iter(buff, len, &off, -1, -1, -1) == 0) {
         bp = buff + off;
         i_len = bp[3];
         if ((off + i_len + 4) > len) {
@@ -285,7 +284,7 @@ encode_tpgs_states(uint8_t *buff, struct tgtgrp *tgtState, int numgrp)
 }
 
 /* Read numbers (up to 32 bits in size) from command line (comma separated
- * list). Assumed decimal unless prefixed by '0x', '0X' or contains traling
+ * list). Assumed decimal unless prefixed by '0x', '0X' or contains trailing
  * 'h' or 'H' (which indicate hex). Returns 0 if ok, else error code. */
 static int
 build_port_arr(const char * inp, int * port_arr, int * port_arr_len,
@@ -357,13 +356,13 @@ build_state_arr(const char * inp, int * state_arr, int * state_arr_len,
     }
     for (k = 0; k < max_arr_len; ++k) {
         try_num = true;
-        if (isalpha(*lcp)) {
+        if (isalpha((uint8_t)*lcp)) {
             try_num = false;
-            switch (toupper(*lcp)) {
+            switch (toupper((uint8_t)*lcp)) {
             case 'A':
-                if ('N' == toupper(*(lcp + 1)))
+                if ('N' == toupper((uint8_t)*(lcp + 1)))
                     state_arr[k] = 1;
-                else if ('O' == toupper(*(lcp + 1)))
+                else if ('O' == toupper((uint8_t)*(lcp + 1)))
                     state_arr[k] = 0;
                 else
                     try_num = true;
@@ -509,7 +508,7 @@ main(int argc, char * argv[])
     pr2serr("In DEBUG mode, ");
     if (verbose_given && version_given) {
         pr2serr("but override: '-vV' given, zero verbose and continue\n");
-        verbose_given = false;
+        /* verbose_given = false; */
         version_given = false;
         verbose = 0;
     } else if (! verbose_given) {
