@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2018-2020 Douglas Gilbert.
+ * Copyright (c) 2018-2023 Douglas Gilbert.
  * All rights reserved.
  * Use of this source code is governed by a BSD-style
  * license that can be found in the BSD_LICENSE file.
@@ -35,7 +35,7 @@
  * to the given SCSI device. Based on sbc4r15.pdf .
  */
 
-static const char * version_str = "1.09 20200724";
+static const char * version_str = "1.15 20230623";
 
 #define STREAM_CONTROL_SA 0x14
 #define GET_STREAM_STATUS_SA 0x16
@@ -47,19 +47,19 @@ static const char * version_str = "1.09 20200724";
 #define DEF_PT_TIMEOUT  60      /* 60 seconds */
 
 
-static struct option long_options[] = {
-        {"brief", no_argument, 0, 'b'},
-        {"close", no_argument, 0, 'c'},
-        {"ctl", required_argument, 0, 'C'},
-        {"get", no_argument, 0, 'g'},
-        {"help", no_argument, 0, 'h'},
-        {"id", required_argument, 0, 'i'},
-        {"maxlen", required_argument, 0, 'm'},
-        {"open", no_argument, 0, 'o'},
-        {"readonly", no_argument, 0, 'r'},
-        {"verbose", no_argument, 0, 'v'},
-        {"version", no_argument, 0, 'V'},
-        {0, 0, 0, 0},
+static const struct option long_options[] = {
+    {"brief", no_argument, 0, 'b'},
+    {"close", no_argument, 0, 'c'},
+    {"ctl", required_argument, 0, 'C'},
+    {"get", no_argument, 0, 'g'},
+    {"help", no_argument, 0, 'h'},
+    {"id", required_argument, 0, 'i'},
+    {"maxlen", required_argument, 0, 'm'},
+    {"open", no_argument, 0, 'o'},
+    {"readonly", no_argument, 0, 'r'},
+    {"verbose", no_argument, 0, 'v'},
+    {"version", no_argument, 0, 'V'},
+    {0, 0, 0, 0},
 };
 
 
@@ -116,7 +116,7 @@ sg_ll_get_stream_status(int sg_fd, uint16_t s_str_id, uint8_t * resp,
     int k, ret, res, sense_cat;
     uint8_t gssCdb[16] = {SG_SERVICE_ACTION_IN_16,
            GET_STREAM_STATUS_SA, 0, 0,  0, 0, 0, 0,  0, 0, 0, 0, 0, 0, 0, 0};
-    uint8_t sense_b[SENSE_BUFF_LEN];
+    uint8_t sense_b[SENSE_BUFF_LEN] SG_C_CPP_ZERO_INIT;
     struct sg_pt_base * ptvp;
     static const char * const cmd_name = "Get stream status";
 
@@ -142,9 +142,12 @@ sg_ll_get_stream_status(int sg_fd, uint16_t s_str_id, uint8_t * resp,
     res = do_scsi_pt(ptvp, -1, DEF_PT_TIMEOUT, verbose);
     ret = sg_cmds_process_resp(ptvp, cmd_name, res, noisy, verbose,
                                &sense_cat);
-    if (-1 == ret)
-        ret = sg_convert_errno(get_scsi_pt_os_err(ptvp));
-    else if (-2 == ret) {
+    if (-1 == ret) {
+        if (get_scsi_pt_transport_err(ptvp))
+            ret = SG_LIB_TRANSPORT_ERROR;
+        else
+            ret = sg_convert_errno(get_scsi_pt_os_err(ptvp));
+    } else if (-2 == ret) {
         switch (sense_cat) {
         case SG_LIB_CAT_RECOVERED:
         case SG_LIB_CAT_NO_SENSE:
@@ -181,7 +184,7 @@ sg_ll_stream_control(int sg_fd, uint32_t str_ctl, uint16_t str_id,
     int k, ret, res, sense_cat;
     uint8_t scCdb[16] = {SG_SERVICE_ACTION_IN_16,
            STREAM_CONTROL_SA, 0, 0,  0, 0, 0, 0,  0, 0, 0, 0, 0, 0, 0, 0};
-    uint8_t sense_b[SENSE_BUFF_LEN];
+    uint8_t sense_b[SENSE_BUFF_LEN] SG_C_CPP_ZERO_INIT;
     struct sg_pt_base * ptvp;
     static const char * const cmd_name = "Stream control";
 
@@ -209,9 +212,12 @@ sg_ll_stream_control(int sg_fd, uint32_t str_ctl, uint16_t str_id,
     res = do_scsi_pt(ptvp, -1, DEF_PT_TIMEOUT, verbose);
     ret = sg_cmds_process_resp(ptvp, cmd_name, res, noisy, verbose,
                                &sense_cat);
-    if (-1 == ret)
-        ret = sg_convert_errno(get_scsi_pt_os_err(ptvp));
-    else if (-2 == ret) {
+    if (-1 == ret) {
+        if (get_scsi_pt_transport_err(ptvp))
+            ret = SG_LIB_TRANSPORT_ERROR;
+        else
+            ret = sg_convert_errno(get_scsi_pt_os_err(ptvp));
+    } else if (-2 == ret) {
         switch (sense_cat) {
         case SG_LIB_CAT_RECOVERED:
         case SG_LIB_CAT_NO_SENSE:
@@ -346,7 +352,7 @@ main(int argc, char * argv[])
     pr2serr("In DEBUG mode, ");
     if (verbose_given && version_given) {
         pr2serr("but override: '-vV' given, zero verbose and continue\n");
-        verbose_given = false;
+        /* verbose_given = false; */
         version_given = false;
         verbose = 0;
     } else if (! verbose_given) {
@@ -427,7 +433,7 @@ main(int argc, char * argv[])
             }
             goto fini;
         }
-        if ((maxlen - resid) < 4) {
+        if ((maxlen - resid) < 8) {
             pr2serr("Response too short (%d bytes) assigned stream id\n",
                         k);
             printf("-1\n");
@@ -435,7 +441,7 @@ main(int argc, char * argv[])
             goto fini;
         } else
             maxlen -= resid;
-        param_dl = sg_get_unaligned_be32(arr + 0) + 4;
+        param_dl = sg_get_unaligned_be32(arr + 0) + 8;
         if (param_dl > (uint32_t)maxlen) {
             pr2serr("Response truncated, need to set --maxlen=%u\n",
                     param_dl);
@@ -451,7 +457,7 @@ main(int argc, char * argv[])
             printf("Number of open streams: %u\n", num_streams);
         }
         maxlen = ((uint32_t)maxlen < param_dl) ? maxlen : (int)param_dl;
-        for (k = 8; k < (maxlen - 4); k += 8) {
+        for (k = 8; k < maxlen; k += 8) {
             stream_id = sg_get_unaligned_be16(arr + k + 2);
             if (do_brief)
                 printf("%u\n", stream_id);
