@@ -1,7 +1,7 @@
 /*
- * (c) 2000 Kurt Garloff <garloff at suse dot de>
+ * (c) 2000 Kurt Garloff
  * heavily based on Douglas Gilbert's sg_rbuf program.
- * (c) 1999-2019 Douglas Gilbert
+ * (c) 1999-2023 Douglas Gilbert
  *
  * Program to test the SCSI host adapter by issuing
  * write and read operations on a device's buffer
@@ -47,7 +47,7 @@
 #include "sg_pr2serr.h"
 
 
-static const char * version_str = "1.20 20191220";
+static const char * version_str = "1.23 20230623";
 
 #define BPI (signed)(sizeof(int))
 
@@ -77,7 +77,7 @@ static int addwrite  = 0;
 static int addread   = 0;
 static int verbose   = 0;
 
-static struct option long_options[] = {
+static const struct option long_options[] = {
         {"help", no_argument, 0, 'h'},
         {"quick", no_argument, 0, 'q'},
         {"addrd", required_argument, 0, 'r'},
@@ -200,22 +200,23 @@ void do_fill_buffer (int *buf, int len)
 {
         int sum;
         int i; int rln = len;
-        srand (time (0));
+
+        srand(time(0));
     retry:
         if (len >= BPI)
-                base = 0x12345678 + rand ();
+                base = 0x12345678 + rand();     /* don't need strong crypto */
         else
-                base = 0x12345678 + (char) rand ();
+                base = 0x12345678 + (char)rand();
         sum = base;
         for (i = 0; i < len/BPI - 1; i++)
         {
                 /* we rely on rand() giving full range of int */
-                buf[i] = rand ();
+                buf[i] = rand();
                 sum += buf[i];
         }
         while (rln%BPI)
         {
-                ((char*)buf)[--rln] = rand ();
+                ((char*)buf)[--rln] = rand();
                 sum += ((char*)buf)[rln];
         }
         if (len >= BPI) buf[len/BPI - 1] = 0x12345678 - sum;
@@ -502,7 +503,7 @@ int main (int argc, char * argv[])
         if (verbose_given && version_given) {
                 pr2serr("but override: '-vV' given, zero verbose and "
                         "continue\n");
-                verbose_given = false;
+                /* verbose_given = false; */
                 version_given = false;
                 verbose = 0;
         } else if (! verbose_given) {
