@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2015-2019 Douglas Gilbert.
+ * Copyright (c) 2015-2023 Douglas Gilbert.
  * All rights reserved.
  * Use of this source code is governed by a BSD-style
  * license that can be found in the BSD_LICENSE file.
@@ -37,7 +37,9 @@
  * to the given SCSI device. Based on spc5r07.pdf .
  */
 
-static const char * version_str = "1.14 20191220";
+static const char * version_str = "1.19 20230623";
+
+static const char * my_name = "sg_timestamp: ";
 
 #define REP_TIMESTAMP_CMDLEN 12
 #define SET_TIMESTAMP_CMDLEN 12
@@ -54,21 +56,21 @@ uint8_t d_buff[256];
  *                     0, 0}; */
 
 
-static struct option long_options[] = {
-        {"elapsed", no_argument, 0, 'e'},
-        {"help", no_argument, 0, 'h'},
-        {"hex", no_argument, 0, 'H'},
-        {"milliseconds", required_argument, 0, 'm'},
-        {"no_timestamp", no_argument, 0, 'N'},
-        {"no-timestamp", no_argument, 0, 'N'},
-        {"origin", no_argument, 0, 'o'},
-        {"raw", no_argument, 0, 'r'},
-        {"readonly", no_argument, 0, 'R'},
-        {"seconds", required_argument, 0, 's'},
-        {"srep", no_argument, 0, 'S'},
-        {"verbose", no_argument, 0, 'v'},
-        {"version", no_argument, 0, 'V'},
-        {0, 0, 0, 0},
+static const struct option long_options[] = {
+    {"elapsed", no_argument, 0, 'e'},
+    {"help", no_argument, 0, 'h'},
+    {"hex", no_argument, 0, 'H'},
+    {"milliseconds", required_argument, 0, 'm'},
+    {"no_timestamp", no_argument, 0, 'N'},
+    {"no-timestamp", no_argument, 0, 'N'},
+    {"origin", no_argument, 0, 'o'},
+    {"raw", no_argument, 0, 'r'},
+    {"readonly", no_argument, 0, 'R'},
+    {"seconds", required_argument, 0, 's'},
+    {"srep", no_argument, 0, 'S'},
+    {"verbose", no_argument, 0, 'v'},
+    {"version", no_argument, 0, 'V'},
+    {0, 0, 0, 0},
 };
 
 /* Indexed by 'timestamp origin' field value */
@@ -149,7 +151,7 @@ usage(int num)
            );
     return;
 page2:
-    pr2serr("sg_timestamp examples:\n"
+    pr2serr("sg_timestamp examples:\n\n"
             "It is possible that the target device containing a SCSI "
             "Logical Unit (LU)\nhas a battery (or supercapacitor) to "
             "keep its RTC (real time clock)\nticking during a power "
@@ -197,7 +199,7 @@ sg_ll_rep_timestamp(int sg_fd, void * resp, int mx_resp_len, int * residp,
     int k, ret, res, sense_cat;
     uint8_t rt_cdb[REP_TIMESTAMP_CMDLEN] =
           {SG_MAINTENANCE_IN, REP_TIMESTAMP_SA, 0, 0,  0, 0, 0, 0, 0, 0, 0, 0};
-    uint8_t sense_b[SENSE_BUFF_LEN];
+    uint8_t sense_b[SENSE_BUFF_LEN] SG_C_CPP_ZERO_INIT;
     struct sg_pt_base * ptvp;
 
     sg_put_unaligned_be32((uint32_t)mx_resp_len, rt_cdb + 6);
@@ -220,9 +222,12 @@ sg_ll_rep_timestamp(int sg_fd, void * resp, int mx_resp_len, int * residp,
     res = do_scsi_pt(ptvp, sg_fd, DEF_PT_TIMEOUT, verbose);
     ret = sg_cmds_process_resp(ptvp, "report timestamp", res, noisy, verbose,
                                &sense_cat);
-    if (-1 == ret)
-        ret = sg_convert_errno(get_scsi_pt_os_err(ptvp));
-    else if (-2 == ret) {
+    if (-1 == ret) {
+        if (get_scsi_pt_transport_err(ptvp))
+            ret = SG_LIB_TRANSPORT_ERROR;
+        else
+            ret = sg_convert_errno(get_scsi_pt_os_err(ptvp));
+    } else if (-2 == ret) {
         switch (sense_cat) {
         case SG_LIB_CAT_RECOVERED:
         case SG_LIB_CAT_NO_SENSE:
@@ -246,7 +251,6 @@ sg_ll_rep_timestamp(int sg_fd, void * resp, int mx_resp_len, int * residp,
     return ret;
 }
 
-
 /* Invokes the SET TIMESTAMP command.  Return of 0 -> success, various
  * SG_LIB_CAT_* positive values or -1 -> other errors */
 static int
@@ -257,7 +261,7 @@ sg_ll_set_timestamp(int sg_fd, void * paramp, int param_len, bool noisy,
     uint8_t st_cdb[SET_TIMESTAMP_CMDLEN] =
           {SG_MAINTENANCE_OUT, SET_TIMESTAMP_SA, 0, 0,  0, 0, 0, 0,
            0, 0, 0, 0};
-    uint8_t sense_b[SENSE_BUFF_LEN];
+    uint8_t sense_b[SENSE_BUFF_LEN] SG_C_CPP_ZERO_INIT;
     struct sg_pt_base * ptvp;
 
     sg_put_unaligned_be32(param_len, st_cdb + 6);
@@ -284,9 +288,12 @@ sg_ll_set_timestamp(int sg_fd, void * paramp, int param_len, bool noisy,
     res = do_scsi_pt(ptvp, sg_fd, DEF_PT_TIMEOUT, verbose);
     ret = sg_cmds_process_resp(ptvp, "set timestamp", res, noisy, verbose,
                                &sense_cat);
-    if (-1 == ret)
-        ret = sg_convert_errno(get_scsi_pt_os_err(ptvp));
-    else if (-2 == ret) {
+    if (-1 == ret) {
+        if (get_scsi_pt_transport_err(ptvp))
+            ret = SG_LIB_TRANSPORT_ERROR;
+        else
+            ret = sg_convert_errno(get_scsi_pt_os_err(ptvp));
+    } else if (-2 == ret) {
         switch (sense_cat) {
         case SG_LIB_CAT_RECOVERED:
         case SG_LIB_CAT_NO_SENSE:
@@ -337,6 +344,8 @@ main(int argc, char * argv[])
     const char * device_name = NULL;
     const char * cmd_name;
 
+    if (getenv("SG3_UTILS_INVOCATION"))
+        sg_rep_invocation(my_name, version_str, argc, argv, stderr);
     while (1) {
         int option_index = 0;
 
@@ -424,7 +433,7 @@ main(int argc, char * argv[])
     pr2serr("In DEBUG mode, ");
     if (verbose_given && version_given) {
         pr2serr("but override: '-vV' given, zero verbose and continue\n");
-        verbose_given = false;
+        /* verbose_given = false; */
         version_given = false;
         verbose = 0;
     } else if (! verbose_given) {
