@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2021 D. Gilbert
+ * Copyright (C) 2000-2023 D. Gilbert
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2, or (at your option)
@@ -45,25 +45,32 @@
 #include "sg_pr2serr.h"
 
 
-static const char * version_str = "3.48 20210102";
+static const char * version_str = "3.56 20230623";
+
+static const char * my_name = "sg_turs: ";
+
+static const char * tur_s = "Test unit ready";
 
 #define DEF_PT_TIMEOUT  60       /* 60 seconds */
 
 
-static struct option long_options[] = {
-        {"delay", required_argument, 0, 'd'},
-        {"help", no_argument, 0, 'h'},
-        {"low", no_argument, 0, 'l'},
-        {"new", no_argument, 0, 'N'},
-        {"number", required_argument, 0, 'n'},
-        {"num", required_argument, 0, 'n'}, /* added in v3.32 (sg3_utils
-                                * v1.43) for sg_requests compatibility */
-        {"old", no_argument, 0, 'O'},
-        {"progress", no_argument, 0, 'p'},
-        {"time", no_argument, 0, 't'},
-        {"verbose", no_argument, 0, 'v'},
-        {"version", no_argument, 0, 'V'},
-        {0, 0, 0, 0},
+static const struct option long_options[] = {
+    {"ascq", required_argument, 0, 'a'},
+    {"delay", required_argument, 0, 'd'},
+    {"help", no_argument, 0, 'h'},
+    {"low", no_argument, 0, 'l'},   /* use sg_pt, minimize open()s */
+    {"new", no_argument, 0, 'N'},
+    {"number", required_argument, 0, 'n'},
+    {"num", required_argument, 0, 'n'}, /* added in v3.32 (sg3_utils
+                            * v1.43) for sg_requests compatibility */
+    {"old", no_argument, 0, 'O'},
+    {"progress", no_argument, 0, 'p'},
+    {"time", no_argument, 0, 't'},
+    {"timeout", required_argument, 0, 'T'},
+    {"tmo", required_argument, 0, 'T'},
+    {"verbose", no_argument, 0, 'v'},
+    {"version", no_argument, 0, 'V'},
+    {0, 0, 0, 0},
 };
 
 struct opts_t {
@@ -74,9 +81,12 @@ struct opts_t {
     bool opts_new;
     bool verbose_given;
     bool version_given;
+    int asc;
+    int ascq;
     int delay;
     int do_help;
     int do_number;
+    int tmo;
     int verbose;
     const char * device_name;
 };
@@ -91,11 +101,16 @@ struct loop_res_t {
 static void
 usage()
 {
-    printf("Usage: sg_turs [--delay=MS] [--help] [--low] [--number=NUM] "
-           "[--num=NUM]\n"
-           "               [--progress] [--time] [--verbose] [--version] "
+    printf("Usage: sg_turs [--ascq=ASC[,ASQ]] [--delay=MS] [--help] "
+           "[--low]\n"
+           "               [--number=NUM] [--num=NUM] [--progress] "
+           "[--time]\n"
+           "               [--timeout=SE] [--verbose] [--version] "
            "DEVICE\n"
            "  where:\n"
+           "    --ascq=ASC[,ASQ] |    check sense from TUR for match on "
+           "ASC[,ASQ]\n"
+           "        -a ASC[,ASQ]      exit status 36 if sense code match\n"
            "    --delay=MS|-d MS    delay MS miiliseconds before sending "
            "each tur\n"
            "    --help|-h        print usage message then exit\n"
@@ -111,6 +126,10 @@ usage()
            "--delay=MS given\n"
            "    --time|-t        outputs total duration and commands per "
            "second\n"
+           "    --timeout SE |-T SE    command timeout on each "
+           "test_unit_ready command\n"
+           "                           (def: 0 which is mapped to 60 "
+           "seconds)\n"
            "    --verbose|-v     increase verbosity\n"
            "    --version|-V     print version string then exit\n\n"
            "Performs a SCSI TEST UNIT READY command (or many of them).\n"
@@ -150,16 +169,39 @@ static int
 new_parse_cmd_line(struct opts_t * op, int argc, char * argv[])
 {
     int c, n;
+    const char * ccp;
 
     while (1) {
         int option_index = 0;
 
-        c = getopt_long(argc, argv, "d:hln:NOptvV", long_options,
+        c = getopt_long(argc, argv, "a:d:hln:NOptT:vV", long_options,
                         &option_index);
         if (c == -1)
             break;
 
         switch (c) {
+        case 'a':
+            ccp = strchr(optarg, ',');
+            n = sg_get_num_nomult(optarg);
+            if ((n < 0) || (n > 255)) {
+                pr2serr("bad argument to '--ascq=\?\?', expect 0 to 255\n");
+                return SG_LIB_SYNTAX_ERROR;
+            }
+            op->asc = n;
+            if (ccp) {
+                if (0 == memcmp("-1", ccp + 1, 2)) {
+                    op->ascq = -1;
+                    break;
+                }
+                n = sg_get_num_nomult(ccp + 1);
+                if ((n < 0) || (n > 255)) {
+                    pr2serr("bad argument to '--ascq=0x%x,\?\?', expect 0 "
+                            "to 255\n", op->asc);
+                    return SG_LIB_SYNTAX_ERROR;
+                }
+                op->ascq = n;
+            }
+            break;
         case 'd':
             n = sg_get_num(optarg);
             if (n < 0) {
@@ -197,6 +239,15 @@ new_parse_cmd_line(struct opts_t * op, int argc, char * argv[])
         case 't':
             op->do_time = true;
             break;
+        case 'T':
+            n = sg_get_num(optarg);
+            if (n < 0) {
+                pr2serr("bad argument to '--timwout='\n");
+                usage();
+                return SG_LIB_SYNTAX_ERROR;
+            }
+            op->tmo = n;
+            break;
         case 'v':
             op->verbose_given = true;
             ++op->verbose;
@@ -330,6 +381,20 @@ parse_cmd_line(struct opts_t * op, int argc, char * argv[])
     return res;
 }
 
+#if defined(SG_LIB_MINGW)
+
+#include <windows.h>
+
+static void
+wait_millisecs(int millisecs)
+{
+    /* MinGW requires pthreads library for nanosleep, use Sleep() instead */
+    Sleep(millisecs);
+}
+
+
+#elif defined(HAVE_CLOCK_GETTIME) && defined(CLOCK_MONOTONIC)
+
 static void
 wait_millisecs(int millisecs)
 {
@@ -341,19 +406,90 @@ wait_millisecs(int millisecs)
                 wait_period = rem;
 }
 
+#else
+
+static void
+wait_millisecs(int millisecs)
+{
+    int res;
+    struct timeval wait_period;
+
+    wait_period.tv_sec = millisecs / 1000;
+    wait_period.tv_usec = (millisecs % 1000) * 1000;
+    res = select(0, NULL, NULL, NULL, &wait_period);
+    if (res < 0)
+        pr2serr("%s: unexpected select() errno=%d\n", __func__, errno);
+}
+#endif
+
+/* Invokes a SCSI TEST UNIT READY command.
+ * N.B. To access the sense buffer outside this routine then one be
+ * provided by the caller.
+ * 'pack_id' is just for diagnostics, safe to set to 0.
+ * Looks for progress indicator if 'progress' non-NULL;
+ * if found writes value [0..65535] else write -1.
+ * Returns 0 when successful, various SG_LIB_CAT_* positive values or
+ * -1 -> other errors */
+static int
+ll_test_unit_ready(struct sg_pt_base * ptvp, int pack_id, int tmo,
+                   int * progress, bool noisy, int verbose)
+{
+    int res, ret, sense_cat;
+
+    if (verbose) {
+        char b[128];
+
+        pr2serr("    %s cdb: %s\n", tur_s,
+                sg_get_command_str(get_scsi_pt_cdb_buf(ptvp),
+                                   get_scsi_pt_cdb_len(ptvp),
+                                   false, sizeof(b), b));
+    }
+    if (NULL == ptvp)
+        return SCSI_PT_DO_BAD_PARAMS;
+
+    set_scsi_pt_packet_id(ptvp, pack_id);
+    res = do_scsi_pt(ptvp, -1, tmo, verbose);
+    ret = sg_cmds_process_resp(ptvp, tur_s, res, noisy, verbose, &sense_cat);
+    if (-1 == ret) {
+        if (get_scsi_pt_transport_err(ptvp))
+            ret = SG_LIB_TRANSPORT_ERROR;
+        else
+            ret = sg_convert_errno(get_scsi_pt_os_err(ptvp));
+    } else if (-2 == ret) {
+        if (progress) {
+            int slen = get_scsi_pt_sense_len(ptvp);
+
+            if (! sg_get_sense_progress_fld(get_scsi_pt_sense_buf(ptvp),
+                                            slen, progress))
+                *progress = -1;
+        }
+        switch (sense_cat) {
+        case SG_LIB_CAT_RECOVERED:
+        case SG_LIB_CAT_NO_SENSE:
+            ret = 0;
+            break;
+        default:
+            ret = sense_cat;
+            break;
+        }
+    } else
+        ret = 0;
+    return ret;
+}
+
 /* Returns true if prints estimate of duration to ready */
 bool
-check_for_lu_becoming(struct sg_pt_base * ptvp)
+check_for_lu_becoming(struct sg_pt_base * ptvp,
+                      struct sg_scsi_sense_hdr * sshp)
 {
     int s_len = get_scsi_pt_sense_len(ptvp);
     uint64_t info;
     uint8_t * sense_b = get_scsi_pt_sense_buf(ptvp);
-    struct sg_scsi_sense_hdr ssh;
 
     /* Check for "LU is in process of becoming ready" with a non-zero INFO
      * field that isn't too big. As per 20-061r2 it means the following: */
-    if (sg_scsi_normalize_sense(sense_b, s_len, &ssh) && (ssh.asc == 0x4) &&
-        (ssh.ascq == 0x1) && sg_get_sense_info_fld(sense_b, s_len, &info) &&
+    if (sg_scsi_normalize_sense(sense_b, s_len, sshp) && (sshp->asc == 0x4) &&
+        (sshp->ascq == 0x1) && sg_get_sense_info_fld(sense_b, s_len, &info) &&
         (info > 0x0) && (info < 0x1000000)) {
         printf("device not ready, estimated to be ready in %" PRIu64
                " milliseconds\n", info);
@@ -371,7 +507,7 @@ loop_turs(struct sg_pt_base * ptvp, struct loop_res_t * resp,
     int packet_id = 0;
     int vb = op->verbose;
     char b[80];
-    uint8_t sense_b[32];
+    uint8_t sense_b[64] SG_C_CPP_ZERO_INIT;
 
     if (op->do_low) {
         int rs, n, sense_cat;
@@ -385,13 +521,18 @@ loop_turs(struct sg_pt_base * ptvp, struct loop_res_t * resp,
             set_scsi_pt_cdb(ptvp, cdb, sizeof(cdb));
             set_scsi_pt_sense(ptvp, sense_b, sizeof(sense_b));
             set_scsi_pt_packet_id(ptvp, ++packet_id);
-            rs = do_scsi_pt(ptvp, -1, DEF_PT_TIMEOUT, vb);
-            n = sg_cmds_process_resp(ptvp, "Test unit ready", rs, (0 == k),
+            rs = do_scsi_pt(ptvp, -1, op->tmo, vb);
+            n = sg_cmds_process_resp(ptvp, tur_s, rs, (0 == k),
                                      vb, &sense_cat);
             if (-1 == n) {
-                resp->ret = sg_convert_errno(get_scsi_pt_os_err(ptvp));
+                if (get_scsi_pt_transport_err(ptvp))
+                    resp->ret = SG_LIB_TRANSPORT_ERROR;
+                else
+                    resp->ret = sg_convert_errno(get_scsi_pt_os_err(ptvp));
                 return k;
             } else if (-2 == n) {
+                struct sg_scsi_sense_hdr ssh SG_C_CPP_ZERO_INIT;
+
                 switch (sense_cat) {
                 case SG_LIB_CAT_RECOVERED:
                 case SG_LIB_CAT_NO_SENSE:
@@ -399,9 +540,16 @@ loop_turs(struct sg_pt_base * ptvp, struct loop_res_t * resp,
                 case SG_LIB_CAT_NOT_READY:
                     ++resp->num_errs;
                     if ((1 == op->do_number) || (op->delay > 0)) {
-                        if (! check_for_lu_becoming(ptvp))
-                            printf("device not ready\n");
-                        resp->ret = sense_cat;
+                        if (! check_for_lu_becoming(ptvp, &ssh)) {
+                            if ((op->asc > 0) && (op->asc == ssh.asc) &&
+                                ((op->ascq < 0) || (op->ascq == ssh.ascq)))
+                                resp->ret = SG_LIB_OK_FALSE;
+                            else {
+                                printf("device not ready\n");
+                                resp->ret = sense_cat;
+                            }
+                        } else
+                            resp->ret = sense_cat;
                         resp->reported = true;
                     }
                     break;
@@ -412,6 +560,20 @@ loop_turs(struct sg_pt_base * ptvp, struct loop_res_t * resp,
                         resp->reported = true;
                     }
                     break;
+                case SG_LIB_CAT_STANDBY:
+                    ++resp->num_errs;
+                    if (vb) {
+                        pr2serr("Ignoring standby device (sense key)\n");
+                        resp->reported = true;
+                    }
+                    break;
+                case SG_LIB_CAT_UNAVAILABLE:
+                    ++resp->num_errs;
+                    if (vb) {
+                        pr2serr("Ignoring unavailable device (sense key)\n");
+                        resp->reported = true;
+                    }
+                    break;
                 default:
                     ++resp->num_errs;
                     if (1 == op->do_number) {
@@ -433,15 +595,24 @@ loop_turs(struct sg_pt_base * ptvp, struct loop_res_t * resp,
                 wait_millisecs(op->delay);
             set_scsi_pt_sense(ptvp, sense_b, sizeof(sense_b));
             /* Might get Unit Attention on first invocation */
-            res = sg_ll_test_unit_ready_pt(ptvp, k, (0 == k), vb);
+            res = ll_test_unit_ready(ptvp, k, op->tmo, NULL, (0 == k), vb);
             if (res) {
                 ++resp->num_errs;
                 resp->ret = res;
                 if ((1 == op->do_number) || (op->delay > 0)) {
                     if (SG_LIB_CAT_NOT_READY == res) {
-                        if (! check_for_lu_becoming(ptvp))
-                            printf("device not ready\n");
-			continue;
+                        struct sg_scsi_sense_hdr ssh SG_C_CPP_ZERO_INIT;
+
+                        if (! check_for_lu_becoming(ptvp, &ssh)) {
+                            if ((op->asc > 0) && (op->asc == ssh.asc) &&
+                                ((op->ascq < 0) || (op->ascq == ssh.ascq))) {
+                                resp->ret = SG_LIB_OK_FALSE;
+                                resp->reported = true;
+                                break;
+                            } else
+                                printf("device not ready\n");
+                        }
+                        continue;
                     } else {
                         sg_get_category_sense_str(res, sizeof(b), b, vb);
                         printf("%s\n", b);
@@ -478,8 +649,12 @@ main(int argc, char * argv[])
 
 
     memset(op, 0, sizeof(opts));
+    op->asc = -1;
+    op->ascq = -1;
     memset(resp, 0, sizeof(loop_res));
     op->do_number = 1;
+    if (getenv("SG3_UTILS_INVOCATION"))
+        sg_rep_invocation(my_name, version_str, argc, argv, stderr);
     res = parse_cmd_line(op, argc, argv);
     if (res)
         return res;
@@ -515,6 +690,8 @@ main(int argc, char * argv[])
         usage_for(op);
         return SG_LIB_SYNTAX_ERROR;
     }
+    if (0 == op->tmo)
+        op->tmo = DEF_PT_TIMEOUT;
 
     if ((sg_fd = sg_cmds_open_device(op->device_name, true /* ro */,
                                      op->verbose)) < 0) {
@@ -538,8 +715,8 @@ main(int argc, char * argv[])
                     wait_millisecs(op->delay);
             }
             progress = -1;
-            res = sg_ll_test_unit_ready_progress_pt(ptvp, k, &progress,
-                             (1 == op->do_number), op->verbose);
+            res = ll_test_unit_ready(ptvp, k, op->tmo, &progress,
+                                     (1 == op->do_number), op->verbose);
             if (progress < 0) {
                 ret = res;
                 break;
