@@ -1,8 +1,10 @@
 /*
- * Copyright (c) 2009-2018 Douglas Gilbert.
+ * Copyright (c) 2009-2023 Douglas Gilbert.
  * All rights reserved.
  * Use of this source code is governed by a BSD-style
  * license that can be found in the BSD_LICENSE file.
+ *
+ * SPDX-License-Identifier: BSD-2-Clause
  */
 
 #include <unistd.h>
@@ -27,16 +29,6 @@
 #include "sg_unaligned.h"
 #include "sg_pr2serr.h"
 
-#if defined(MSC_VER) || defined(__MINGW32__)
-#define HAVE_MS_SLEEP
-#endif
-
-#ifdef HAVE_MS_SLEEP
-#include <windows.h>
-#define sleep_for(seconds)    Sleep( (seconds) * 1000)
-#else
-#define sleep_for(seconds)    sleep(seconds)
-#endif
 
 /* A utility program originally written for the Linux OS SCSI subsystem.
  *
@@ -44,7 +36,8 @@
  * logical blocks. Note that DATA MAY BE LOST.
  */
 
-static const char * version_str = "1.17 20180628";
+static const char * version_str = "1.23 20230623";
+static const char * my_name = "sg_unmap: ";
 
 
 #define DEF_TIMEOUT_SECS 60
@@ -57,21 +50,21 @@ static const char * version_str = "1.17 20180628";
 #endif
 
 
-static struct option long_options[] = {
-        {"all", required_argument, 0, 'A'},
-        {"anchor", no_argument, 0, 'a'},
-        {"dry-run", no_argument, 0, 'd'},
-        {"dry_run", no_argument, 0, 'd'},
-        {"force", no_argument, 0, 'f'},
-        {"grpnum", required_argument, 0, 'g'},
-        {"help", no_argument, 0, 'h'},
-        {"in", required_argument, 0, 'I'},
-        {"lba", required_argument, 0, 'l'},
-        {"num", required_argument, 0, 'n'},
-        {"timeout", required_argument, 0, 't'},
-        {"verbose", no_argument, 0, 'v'},
-        {"version", no_argument, 0, 'V'},
-        {0, 0, 0, 0},
+static const struct option long_options[] = {
+    {"all", required_argument, 0, 'A'},
+    {"anchor", no_argument, 0, 'a'},
+    {"dry-run", no_argument, 0, 'd'},
+    {"dry_run", no_argument, 0, 'd'},
+    {"force", no_argument, 0, 'f'},
+    {"grpnum", required_argument, 0, 'g'},
+    {"help", no_argument, 0, 'h'},
+    {"in", required_argument, 0, 'I'},
+    {"lba", required_argument, 0, 'l'},
+    {"num", required_argument, 0, 'n'},
+    {"timeout", required_argument, 0, 't'},
+    {"verbose", no_argument, 0, 'v'},
+    {"version", no_argument, 0, 'V'},
+    {0, 0, 0, 0},
 };
 
 
@@ -261,7 +254,7 @@ build_joint_arr(const char * file_name, uint64_t * lba_arr, uint32_t * num_arr,
     int64_t ll;
     char line[1024];
     char * lcp;
-    FILE * fp;
+    FILE * fp = NULL;
 
     have_stdin = ((1 == strlen(file_name)) && ('-' == file_name[0]));
     if (have_stdin)
@@ -344,12 +337,12 @@ build_joint_arr(const char * file_name, uint64_t * lba_arr, uint32_t * num_arr,
         goto bad_exit;
     }
     *arr_len = off >> 1;
-    if (fp && (stdin != fp))
+    if (fp && (! have_stdin))
         fclose(fp);
     return 0;
 
 bad_exit:
-    if (fp && (stdin != fp))
+    if (fp && (! have_stdin))
         fclose(fp);
     return 1;
 }
@@ -387,7 +380,10 @@ main(int argc, char * argv[])
     uint64_t addr_arr[MAX_NUM_ADDR];
     uint32_t num_arr[MAX_NUM_ADDR];
     uint8_t param_arr[8 + (MAX_NUM_ADDR * 16)];
+    static const char * tryvv_s = ", try '-vv' for more information";
 
+    if (getenv("SG3_UTILS_INVOCATION"))
+        sg_rep_invocation(my_name, version_str, argc, argv, stderr);
     while (1) {
         int option_index = 0;
 
@@ -498,7 +494,7 @@ main(int argc, char * argv[])
     pr2serr("In DEBUG mode, ");
     if (verbose_given && version_given) {
         pr2serr("but override: '-vV' given, zero verbose and continue\n");
-        verbose_given = false;
+        /* verbose_given = false; */
         version_given = false;
         vb = 0;
     } else if (! verbose_given) {
@@ -674,25 +670,14 @@ main(int argc, char * argv[])
 
             printf("%s is:  %.8s  %.16s  %.4s\n", device_name,
                    inq_resp.vendor, inq_resp.product, inq_resp.revision);
-            sleep_for(3);
+            sg_sleep_secs(3);
             if (to_end_of_device)
-                snprintf(b, sizeof(b), "LBA 0x%" PRIx64 " to end of %s "
-                         "(0x%" PRIx64 ")", all_start, device_name, all_last);
+                snprintf(b, sizeof(b), "%s from LBA 0x%" PRIx64 " to end "
+                         "(0x%" PRIx64 ")", device_name, all_start, all_last);
             else
-                snprintf(b, sizeof(b), "LBA 0x%" PRIx64 " to 0x%" PRIx64
-                         " on %s", all_start, all_last, device_name);
-            printf("\nAn UNMAP (a.k.a. trim) will commence in 15 seconds\n");
-            printf("    ALL data from %s will be LOST\n", b);
-            printf("        Press control-C to abort\n");
-            sleep_for(5);
-            printf("\nAn UNMAP will commence in 10 seconds\n");
-            printf("    ALL data from %s will be LOST\n", b);
-            printf("        Press control-C to abort\n");
-            sleep_for(5);
-            printf("\nAn UNMAP (a.k.a. trim) will commence in 5 seconds\n");
-            printf("    ALL data from %s will be LOST\n", b);
-            printf("        Press control-C to abort\n");
-            sleep_for(7);
+                snprintf(b, sizeof(b), "%s from LBA 0x%" PRIx64 " to 0x%"
+                         PRIx64, device_name, all_start, all_last);
+            sg_warn_and_wait("UNMAP (a.k.a. trim)", b, false);
         }
         if (dry_run) {
             pr2serr("Doing dry-run, would have unmapped from LBA 0x%" PRIx64
@@ -755,19 +740,19 @@ retry:
         if (! do_force) {
             printf("%s is:  %.8s  %.16s  %.4s\n", device_name,
                    inq_resp.vendor, inq_resp.product, inq_resp.revision);
-            sleep_for(3);
+            sg_sleep_secs(3);
             printf("\nAn UNMAP (a.k.a. trim) will commence in 15 seconds\n");
             printf("    Some data will be LOST\n");
             printf("        Press control-C to abort\n");
-            sleep_for(5);
+            sg_sleep_secs(5);
             printf("\nAn UNMAP will commence in 10 seconds\n");
             printf("    Some data will be LOST\n");
             printf("        Press control-C to abort\n");
-            sleep_for(5);
+            sg_sleep_secs(5);
             printf("\nAn UNMAP (a.k.a. trim) will commence in 5 seconds\n");
             printf("    Some data will be LOST\n");
             printf("        Press control-C to abort\n");
-            sleep_for(7);
+            sg_sleep_secs(7);
         }
         res = sg_ll_unmap_v2(sg_fd, anchor, grpnum, timeout, param_arr,
                              param_len, true, vb);
@@ -787,7 +772,10 @@ retry:
             pr2serr("UNMAP not supported\n");
             break;
         case SG_LIB_CAT_ILLEGAL_REQ:
-            pr2serr("bad field in UNMAP cdb\n");
+            pr2serr("bad field in UNMAP cdb%s\n", tryvv_s);
+            break;
+        case SG_LIB_CAT_INVALID_PARAM:
+            pr2serr("bad field in UNMAP parameter list%s\n", tryvv_s);
             break;
         default:
             err_printed = false;
