@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2006-2018 Luben Tuikov and Douglas Gilbert.
+ * Copyright (c) 2006-2023 Luben Tuikov and Douglas Gilbert.
  * All rights reserved.
  * Use of this source code is governed by a BSD-style
  * license that can be found in the BSD_LICENSE file.
@@ -40,9 +40,10 @@
  * This utility issues the SCSI WRITE BUFFER command to the given device.
  */
 
-static const char * version_str = "1.29 20181112";    /* spc5r19 */
+static const char * version_str = "1.33 20230623";    /* spc6r07 */
+
+static const char * my_name = "sg_write_buffer: ";    /* spc6r07 */
 
-#define ME "sg_write_buffer: "
 #define DEF_XFER_LEN (8 * 1024 * 1024)
 #define EBUFF_SZ 256
 
@@ -51,25 +52,25 @@ static const char * version_str = "1.29 20181112";    /* spc5r19 */
 #define SENSE_BUFF_LEN 64       /* Arbitrary, could be larger */
 #define DEF_PT_TIMEOUT 300      /* 300 seconds, 5 minutes */
 
-static struct option long_options[] = {
-        {"bpw", required_argument, 0, 'b'},
-        {"dry-run", no_argument, 0, 'd'},
-        {"dry_run", no_argument, 0, 'd'},
-        {"help", no_argument, 0, 'h'},
-        {"id", required_argument, 0, 'i'},
-        {"in", required_argument, 0, 'I'},
-        {"length", required_argument, 0, 'l'},
-        {"mode", required_argument, 0, 'm'},
-        {"offset", required_argument, 0, 'o'},
-        {"read-stdin", no_argument, 0, 'r'},
-        {"read_stdin", no_argument, 0, 'r'},
-        {"raw", no_argument, 0, 'r'},
-        {"skip", required_argument, 0, 's'},
-        {"specific", required_argument, 0, 'S'},
-        {"timeout", required_argument, 0, 't' },
-        {"verbose", no_argument, 0, 'v'},
-        {"version", no_argument, 0, 'V'},
-        {0, 0, 0, 0},
+static const struct option long_options[] = {
+    {"bpw", required_argument, 0, 'b'},
+    {"dry-run", no_argument, 0, 'd'},
+    {"dry_run", no_argument, 0, 'd'},
+    {"help", no_argument, 0, 'h'},
+    {"id", required_argument, 0, 'i'},
+    {"in", required_argument, 0, 'I'},
+    {"length", required_argument, 0, 'l'},
+    {"mode", required_argument, 0, 'm'},
+    {"offset", required_argument, 0, 'o'},
+    {"read-stdin", no_argument, 0, 'r'},
+    {"read_stdin", no_argument, 0, 'r'},
+    {"raw", no_argument, 0, 'r'},
+    {"skip", required_argument, 0, 's'},
+    {"specific", required_argument, 0, 'S'},
+    {"timeout", required_argument, 0, 't' },
+    {"verbose", no_argument, 0, 'v'},
+    {"version", no_argument, 0, 'V'},
+    {0, 0, 0, 0},
 };
 
 
@@ -144,32 +145,32 @@ struct mode_s {
         const char *comment;
 };
 
-static struct mode_s mode_arr[] = {
-        {"hd",         MODE_HEADER_DATA, "combined header and data "
-                "(obsolete)"},
-        {"vendor",     MODE_VENDOR,    "vendor specific"},
-        {"data",       MODE_DATA,      "data"},
-        {"dmc",        MODE_DNLD_MC,   "download microcode and activate"},
-        {"dmc_save",   MODE_DNLD_MC_SAVE, "download microcode, save and "
-                "activate"},
-        {"dmc_offs",   MODE_DNLD_MC_OFFS, "download microcode with offsets "
-                "and activate"},
-        {"dmc_offs_save", MODE_DNLD_MC_OFFS_SAVE, "download microcode with "
-                "offsets, save and\n\t\t\t\tactivate"},
-        {"echo",       MODE_ECHO_BUFFER, "write data to echo buffer"},
-        {"dmc_offs_ev_defer", MODE_DNLD_MC_EV_OFFS_DEFER, "download "
-                "microcode with offsets, select\n\t\t\t\tactivation event, "
-                "save and defer activation"},
-        {"dmc_offs_defer", MODE_DNLD_MC_OFFS_DEFER, "download microcode "
-                "with offsets, save and\n\t\t\t\tdefer activation"},
-        {"activate_mc", MODE_ACTIVATE_MC, "activate deferred microcode"},
-        {"en_ex",      MODE_EN_EX_ECHO, "enable expander communications "
-                "protocol and\n\t\t\t\techo buffer (obsolete)"},
-        {"dis_ex",     MODE_DIS_EX, "disable expander communications "
-                "protocol\n\t\t\t\t(obsolete)"},
-        {"deh",        MODE_DNLD_ERR_HISTORY, "download application client "
-                "error history "},
-        {NULL, 0, NULL},
+static const struct mode_s mode_arr[] = {
+    {"hd",         MODE_HEADER_DATA, "combined header and data "
+            "(obsolete)"},
+    {"vendor",     MODE_VENDOR,    "vendor specific"},
+    {"data",       MODE_DATA,      "data"},
+    {"dmc",        MODE_DNLD_MC,   "download microcode and activate"},
+    {"dmc_save",   MODE_DNLD_MC_SAVE, "download microcode, save and "
+            "activate"},
+    {"dmc_offs",   MODE_DNLD_MC_OFFS, "download microcode with offsets "
+            "and activate"},
+    {"dmc_offs_save", MODE_DNLD_MC_OFFS_SAVE, "download microcode with "
+            "offsets, save and\n\t\t\t\tactivate"},
+    {"echo",       MODE_ECHO_BUFFER, "write data to echo buffer"},
+    {"dmc_offs_ev_defer", MODE_DNLD_MC_EV_OFFS_DEFER, "download "
+            "microcode with offsets, select\n\t\t\t\tactivation event, "
+            "save and defer activation"},
+    {"dmc_offs_defer", MODE_DNLD_MC_OFFS_DEFER, "download microcode "
+            "with offsets, save and\n\t\t\t\tdefer activation"},
+    {"activate_mc", MODE_ACTIVATE_MC, "activate deferred microcode"},
+    {"en_ex",      MODE_EN_EX_ECHO, "enable expander communications "
+            "protocol and\n\t\t\t\techo buffer (obsolete)"},
+    {"dis_ex",     MODE_DIS_EX, "disable expander communications "
+            "protocol\n\t\t\t\t(obsolete)"},
+    {"deh",        MODE_DNLD_ERR_HISTORY, "download application client "
+            "error history "},
+    {NULL, 0, NULL},
 };
 
 static void
@@ -220,6 +221,8 @@ main(int argc, char * argv[])
     const struct mode_s * mp;
     char ebuff[EBUFF_SZ];
 
+    if (getenv("SG3_UTILS_INVOCATION"))
+        sg_rep_invocation(my_name, version_str, argc, argv, stderr);
     while (1) {
         int option_index = 0;
 
@@ -268,7 +271,7 @@ main(int argc, char * argv[])
              wb_len_given = true;
              break;
         case 'm':
-            if (isdigit(*optarg)) {
+            if (isdigit((uint8_t)*optarg)) {
                 wb_mode = sg_get_num(optarg);
                 if ((wb_mode < 0) || (wb_mode > 31)) {
                     pr2serr("argument to '--mode' should be in the range 0 "
@@ -359,7 +362,7 @@ main(int argc, char * argv[])
     pr2serr("In DEBUG mode, ");
     if (verbose_given && version_given) {
         pr2serr("but override: '-vV' given, zero verbose and continue\n");
-        verbose_given = false;
+        /* verbose_given = false; */
         version_given = false;
         verbose = 0;
     } else if (! verbose_given) {
@@ -399,7 +402,7 @@ main(int argc, char * argv[])
     sg_fd = sg_cmds_open_device(device_name, false /* rw */, verbose);
     if (sg_fd < 0) {
         if (verbose)
-            pr2serr(ME "open error: %s: %s\n", device_name,
+            pr2serr("%sopen error: %s: %s\n", my_name, device_name,
                     safe_strerror(-sg_fd));
         ret = sg_convert_errno(-sg_fd);
         goto err_out;
@@ -409,7 +412,7 @@ main(int argc, char * argv[])
             wb_len = DEF_XFER_LEN;
         dop = sg_memalign(wb_len, 0, &free_dop, false);
         if (NULL == dop) {
-            pr2serr(ME "out of memory\n");
+            pr2serr("%sout of memory\n", my_name);
             ret = sg_convert_errno(ENOMEM);
             goto err_out;
         }
@@ -426,8 +429,8 @@ main(int argc, char * argv[])
             } else {
                 if ((infd = open(file_name, O_RDONLY)) < 0) {
                     ret = sg_convert_errno(errno);
-                    snprintf(ebuff, EBUFF_SZ,
-                             ME "could not open %s for reading", file_name);
+                    snprintf(ebuff, EBUFF_SZ, "%scould not open %s for "
+                             "reading", my_name, file_name);
                     perror(ebuff);
                     goto err_out;
                 } else if (sg_set_binary_mode(infd) < 0)
@@ -435,8 +438,9 @@ main(int argc, char * argv[])
                 if (wb_skip > 0) {
                     if (lseek(infd, wb_skip, SEEK_SET) < 0) {
                         ret = sg_convert_errno(errno);
-                        snprintf(ebuff,  EBUFF_SZ, ME "couldn't skip to "
-                                 "required position on %s", file_name);
+                        snprintf(ebuff,  EBUFF_SZ, "%scouldn't skip to "
+                                 "required position on %s", my_name,
+                                 file_name);
                         perror(ebuff);
                         close(infd);
                         goto err_out;
@@ -445,13 +449,14 @@ main(int argc, char * argv[])
             }
             if (infd == STDIN_FILENO) {
                 if (NULL == (read_buf = (uint8_t *)malloc(DEF_XFER_LEN))) {
-                    pr2serr(ME "out of memory\n");
+                    pr2serr("%sout of memory\n", my_name);
                     ret = SG_LIB_SYNTAX_ERROR;
                     goto err_out;
                 }
                 res = read(infd, read_buf, DEF_XFER_LEN);
                 if (res < 0) {
-                    snprintf(ebuff, EBUFF_SZ, ME "couldn't read from STDIN");
+                    snprintf(ebuff, EBUFF_SZ, "%scouldn't read from STDIN",
+                             my_name);
                     perror(ebuff);
                     ret = SG_LIB_FILE_ERROR;
                     goto err_out;
@@ -483,8 +488,8 @@ main(int argc, char * argv[])
                 res = read(infd, dop, wb_len);
                 if (res < 0) {
                     ret = sg_convert_errno(errno);
-                    snprintf(ebuff, EBUFF_SZ, ME "couldn't read from %s",
-                             file_name);
+                    snprintf(ebuff, EBUFF_SZ, "%scouldn't read from %s",
+                             my_name, file_name);
                     perror(ebuff);
                     if (! got_stdin)
                         close(infd);
