@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2009-2020 Douglas Gilbert.
+ * Copyright (c) 2009-2023 Douglas Gilbert.
  * All rights reserved.
  * Use of this source code is governed by a BSD-style
  * license that can be found in the BSD_LICENSE file.
@@ -33,7 +33,7 @@
 #include "sg_unaligned.h"
 #include "sg_pr2serr.h"
 
-static const char * version_str = "1.31 20200430";
+static const char * version_str = "1.35 20230623";
 
 
 #define ME "sg_write_same: "
@@ -59,7 +59,7 @@ static const char * version_str = "1.31 20200430";
 #define UINT32_MAX ((uint32_t)-1)
 #endif
 
-static struct option long_options[] = {
+static const struct option long_options[] = {
     {"10", no_argument, 0, 'R'},
     {"16", no_argument, 0, 'S'},
     {"32", no_argument, 0, 'T'},
@@ -169,8 +169,8 @@ do_write_same(int sg_fd, const struct opts_t * op, const void * dataoutp,
 {
     int ret, res, sense_cat, cdb_len;
     uint64_t llba;
-    uint8_t ws_cdb[WRITE_SAME32_LEN];
-    uint8_t sense_b[SENSE_BUFF_LEN];
+    uint8_t ws_cdb[WRITE_SAME32_LEN] SG_C_CPP_ZERO_INIT;
+    uint8_t sense_b[SENSE_BUFF_LEN] SG_C_CPP_ZERO_INIT;
     struct sg_pt_base * ptvp;
 
     cdb_len = op->pref_cdb_size;
@@ -194,7 +194,6 @@ do_write_same(int sg_fd, const struct opts_t * op, const void * dataoutp,
     }
     if (act_cdb_lenp)
         *act_cdb_lenp = cdb_len;
-    memset(ws_cdb, 0, sizeof(ws_cdb));
     switch (cdb_len) {
     case WRITE_SAME10_LEN:
         ws_cdb[0] = WRITE_SAME10_OP;
@@ -210,7 +209,7 @@ do_write_same(int sg_fd, const struct opts_t * op, const void * dataoutp,
         if (op->lbdata)
             ws_cdb[1] |= 0x2;
         sg_put_unaligned_be32((uint32_t)op->lba, ws_cdb + 2);
-        ws_cdb[6] = (op->grpnum & 0x1f);
+        ws_cdb[6] = (op->grpnum & GRPNUM_MASK);
         sg_put_unaligned_be16((uint16_t)op->numblocks, ws_cdb + 7);
         break;
     case WRITE_SAME16_LEN:
@@ -228,11 +227,11 @@ do_write_same(int sg_fd, const struct opts_t * op, const void * dataoutp,
             ws_cdb[1] |= 0x1;
         sg_put_unaligned_be64(op->lba, ws_cdb + 2);
         sg_put_unaligned_be32((uint32_t)op->numblocks, ws_cdb + 10);
-        ws_cdb[14] = (op->grpnum & 0x1f);
+        ws_cdb[14] = (op->grpnum & GRPNUM_MASK);
         break;
     case WRITE_SAME32_LEN:
         ws_cdb[0] = VARIABLE_LEN_OP;
-        ws_cdb[6] = (op->grpnum & 0x1f);
+        ws_cdb[6] = (op->grpnum & GRPNUM_MASK);
         ws_cdb[7] = WRITE_SAME32_ADD;
         sg_put_unaligned_be16((uint16_t)WRITE_SAME32_SA, ws_cdb + 8);
         ws_cdb[10] = ((op->wrprotect & 0x7) << 5);
@@ -276,9 +275,12 @@ do_write_same(int sg_fd, const struct opts_t * op, const void * dataoutp,
     res = do_scsi_pt(ptvp, sg_fd, op->timeout, op->verbose);
     ret = sg_cmds_process_resp(ptvp, "Write same", res, true /*noisy */,
                                op->verbose, &sense_cat);
-    if (-1 == ret)
-        get_scsi_pt_os_err(ptvp);
-    else if (-2 == ret) {
+    if (-1 == ret) {
+        if (get_scsi_pt_transport_err(ptvp))
+            ret = SG_LIB_TRANSPORT_ERROR;
+        else
+            ret = sg_convert_errno(get_scsi_pt_os_err(ptvp));
+    } else if (-2 == ret) {
         switch (sense_cat) {
         case SG_LIB_CAT_RECOVERED:
         case SG_LIB_CAT_NO_SENSE:
