@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2014-2020 Douglas Gilbert
+ * Copyright (c) 2014-2023 Douglas Gilbert
  * All rights reserved.
  * Use of this source code is governed by a BSD-style
  * license that can be found in the BSD_LICENSE file.
@@ -40,7 +40,7 @@
 #include "sg_unaligned.h"
 #include "sg_pr2serr.h"
 
-static const char * version_str = "1.18 20200430";
+static const char * version_str = "1.23 20230623";
 
 
 #define ME "sg_write_verify: "
@@ -58,7 +58,7 @@ static const char * version_str = "1.18 20200430";
 #define DEF_TIMEOUT_SECS 60
 
 
-static struct option long_options[] = {
+static const struct option long_options[] = {
     {"16", no_argument, 0, 'S'},
     {"bytchk", required_argument, 0, 'b'},
     {"dpo", no_argument, 0, 'd'},
@@ -129,7 +129,7 @@ run_scsi_transaction(int sg_fd, const uint8_t *cdbp, int cdb_len,
 {
     int res, sense_cat, ret;
     struct sg_pt_base * ptvp;
-    uint8_t sense_b[SENSE_BUFF_LEN];
+    uint8_t sense_b[SENSE_BUFF_LEN] SG_C_CPP_ZERO_INIT;
     char b[32];
 
     snprintf(b, sizeof(b), "Write and verify(%d)", cdb_len);
@@ -153,9 +153,12 @@ run_scsi_transaction(int sg_fd, const uint8_t *cdbp, int cdb_len,
     set_scsi_pt_data_out(ptvp, dop, do_len);
     res = do_scsi_pt(ptvp, sg_fd, timeout, verbose);
     ret = sg_cmds_process_resp(ptvp, b, res, noisy, verbose, &sense_cat);
-    if (-1 == ret)
-        ret = sg_convert_errno(get_scsi_pt_os_err(ptvp));
-    else if (-2 == ret) {
+    if (-1 == ret) {
+        if (get_scsi_pt_transport_err(ptvp))
+            ret = SG_LIB_TRANSPORT_ERROR;
+        else
+            ret = sg_convert_errno(get_scsi_pt_os_err(ptvp));
+    } else if (-2 == ret) {
         switch (sense_cat) {
         case SG_LIB_CAT_RECOVERED:
         case SG_LIB_CAT_NO_SENSE:
@@ -212,7 +215,7 @@ sg_ll_write_verify10(int sg_fd, int wrprotect, bool dpo, int bytchk,
        wv_cdb[1] |= ((bytchk & 0x3) << 1);
 
     sg_put_unaligned_be32((uint32_t)lba, wv_cdb + 2);
-    wv_cdb[6] = group & 0x1f;
+    wv_cdb[6] = group & GRPNUM_MASK;
     sg_put_unaligned_be16((uint16_t)num_lb, wv_cdb + 7);
     ret = run_scsi_transaction(sg_fd, wv_cdb, sizeof(wv_cdb), dop, do_len,
                                timeout, noisy, verbose);
@@ -240,7 +243,7 @@ sg_ll_write_verify16(int sg_fd, int wrprotect, bool dpo, int bytchk,
 
     sg_put_unaligned_be64(llba, wv_cdb + 2);
     sg_put_unaligned_be32((uint32_t)num_lb, wv_cdb + 10);
-    wv_cdb[14] = group & 0x1f;
+    wv_cdb[14] = group & GRPNUM_MASK;
     ret = run_scsi_transaction(sg_fd, wv_cdb, sizeof(wv_cdb), dop, do_len,
                                timeout, noisy, verbose);
     return ret;
@@ -422,7 +425,7 @@ main(int argc, char * argv[])
     pr2serr("In DEBUG mode, ");
     if (verbose_given && version_given) {
         pr2serr("but override: '-vV' given, zero verbose and continue\n");
-        verbose_given = false;
+        /* verbose_given = false; */
         version_given = false;
         verbose = 0;
     } else if (! verbose_given) {
