@@ -1,7 +1,7 @@
 /* A utility program for copying files. Specialised for "files" that
  * represent devices that understand the SCSI command set.
  *
- * Copyright (C) 1999 - 2020 D. Gilbert and P. Allworth
+ * Copyright (C) 1999 - 2023 D. Gilbert and P. Allworth
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2, or (at your option)
@@ -27,7 +27,7 @@
    then only the read side will be mmap-ed, while the write side will
    use normal IO.
 
-   This version is designed for the linux kernel 2.4, 2.6, 3, 4 and 5 series.
+   This version is designed for the Linux kernel 2.4 series and onward.
 */
 
 #define _XOPEN_SOURCE 600
@@ -45,6 +45,7 @@
 #include <signal.h>
 #include <ctype.h>
 #include <errno.h>
+#include <time.h>               /* for clock_gettime() */
 #include <limits.h>
 #define __STDC_FORMAT_MACROS 1
 #include <inttypes.h>
@@ -56,12 +57,18 @@
 #ifndef major
 #include <sys/types.h>
 #endif
-#include <linux/major.h>        /* for MEM_MAJOR, SCSI_GENERIC_MAJOR, etc */
-#include <linux/fs.h>           /* for BLKSSZGET and friends */
 
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
+
+#ifdef HAVE_LINUX_MAJOR_H
+#include <linux/major.h>
+#include <linux/fs.h>           /* for BLKSSZGET and friends */
+#else
+#include "sg_pt_linux_missing.h"
+#endif
+
 #include "sg_lib.h"
 #include "sg_cmds_basic.h"
 #include "sg_io_linux.h"
@@ -69,15 +76,18 @@
 #include "sg_pr2serr.h"
 
 
-static const char * version_str = "1.67 20200510";
+static const char * version_str = "1.25 20230717";
+
+static const char * my_name = "sgm_dd: ";
 
 #define DEF_BLOCK_SIZE 512
 #define DEF_BLOCKS_PER_TRANSFER 128
 #define DEF_BLOCKS_PER_2048TRANSFER 32
 #define DEF_SCSI_CDBSZ 10
 #define MAX_SCSI_CDBSZ 16
+#define MAX_BPT_VALUE (1 << 24)         /* used for maximum bs as well */
+#define MAX_COUNT_SKIP_SEEK (1LL << 48) /* coverity wants upper bound */
 
-#define ME "sgm_dd: "
 
 
 #ifndef SG_FLAG_MMAP_IO
@@ -116,6 +126,7 @@ static int64_t out_full = 0;
 static int out_partial = 0;
 static int verbose = 0;
 static int dry_run = 0;
+static int progress = 0;        /* accept --progress or -p, does nothing */
 
 static bool do_time = false;
 static bool start_tm_valid = false;
@@ -123,7 +134,7 @@ static struct timeval start_tm;
 static int blk_sz = 0;
 static uint32_t glob_pack_id = 0;       /* pre-increment */
 
-static const char * proc_allow_dio = "/proc/scsi/sg/allow_dio";
+static const char * sg_allow_dio = "/sys/module/sg/parameters/allow_dio";
 
 struct flags_t {
     bool append;
@@ -160,6 +171,97 @@ print_stats()
             out_partial);
 }
 
+/* Note that duration measurements may be effected by "discontinuous jumps
+ * in the system time". */
+static void
+calc_duration_throughput(bool contin)
+{
+    int n, elapsed_secs;
+    int64_t blks;
+    double a, b, r, da, db;
+    char f[128];
+    struct timeval end_tm, res_tm, delta_tm;
+    static const int flen = sizeof(f);
+    static bool prev_valid = false;
+    static struct timeval prev_tm;
+    static int64_t prev_blks;
+
+    f[0] = '\0';
+    if (start_tm_valid && (start_tm.tv_sec || start_tm.tv_usec)) {
+        blks = (in_full > out_full) ? in_full : out_full;
+        gettimeofday(&end_tm, NULL);
+        res_tm.tv_sec = end_tm.tv_sec - start_tm.tv_sec;
+        res_tm.tv_usec = end_tm.tv_usec - start_tm.tv_usec;
+        if (res_tm.tv_usec < 0) {
+            --res_tm.tv_sec;
+            res_tm.tv_usec += 1000000;
+        }
+        elapsed_secs = res_tm.tv_sec;
+        a = res_tm.tv_sec;
+        a += (0.000001 * res_tm.tv_usec);
+        if (prev_valid) {
+            delta_tm.tv_sec = end_tm.tv_sec - prev_tm.tv_sec;
+            delta_tm.tv_usec = end_tm.tv_usec - prev_tm.tv_usec;
+            if (delta_tm.tv_usec < 0) {
+                --delta_tm.tv_sec;
+                delta_tm.tv_usec += 1000000;
+            }
+            da = delta_tm.tv_sec;
+            da += (0.000001 * delta_tm.tv_usec);
+        } else
+            da = 0.0000001;
+
+        b = (double)blk_sz * blks;
+        n = sg_scnpr(f, flen, "time to copy data%s: %d.%06d secs",
+                     (contin ? " so far" : ""),
+                     (int)res_tm.tv_sec, (int)res_tm.tv_usec);
+        r = 0.0;
+        if ((a > 0.00001) && (b > 511)) {
+            r = b / (a * 1000000.0);
+            if (r < 1.0)
+                n += sg_scn3pr(f, flen, n, " at %.1f kB/sec", r * 1000);
+            else
+                n += sg_scn3pr(f, flen, n, " at %.2f MB/sec", r);
+        }
+        if (prev_valid && (da > 0.00001)) {
+            db = (double)blk_sz * (blks - prev_blks);
+            if (db > 511) {
+                double dr = db / (da * 1000000.0);
+
+                if (dr < 1.0)
+                    sg_scn3pr(f, flen, n, " (delta %.1f KB/sec)", dr * 1000);
+                else
+                    sg_scn3pr(f, flen, n, " (delta %.2f MB/sec)", dr);
+            }
+        }
+        pr2serr("%s\n", f);
+        if (contin && (r > 0.01) && (dd_count > 100)) {
+            int secs = (int)(((double)blk_sz * dd_count) / (r * 1000000));
+            int h, m;
+
+            if (secs > 10) {
+                n = sg_scnpr(f, flen, "%d%% complete, ",
+                             (100 * elapsed_secs) / (secs + elapsed_secs));
+                h = secs / 3600;
+                secs = secs - (h * 3600);
+                m = secs / 60;
+                secs = secs - (m * 60);
+                n += sg_scn3pr(f, flen, n, "estimated time remaining: ");
+                if (h > 0)
+                    sg_scn3pr(f, flen, n, "%d:%02d:%02d", h, m, secs);
+                else
+                    sg_scn3pr(f, flen, n, "%d:%02d", m, secs);
+                pr2serr("%s\n", f);
+            }
+        }
+        prev_tm = end_tm;
+        prev_blks = blks;
+        if (! prev_valid)
+            prev_valid = true;
+    }
+}
+
+#if 0
 static void
 calc_duration_throughput(bool contin)
 {
@@ -186,6 +288,7 @@ calc_duration_throughput(bool contin)
             pr2serr("\n");
     }
 }
+#endif
 
 static void
 interrupt_handler(int sig)
@@ -239,24 +342,24 @@ dd_filetype(const char * filename)
 }
 
 static char *
-dd_filetype_str(int ft, char * buff)
+dd_filetype_str(int ft, char * buff, int bufflen)
 {
     int off = 0;
 
     if (FT_DEV_NULL & ft)
-        off += sg_scnpr(buff + off, 32, "null device ");
+        off += sg_scn3pr(buff, bufflen, off, "null device ");
     if (FT_SG & ft)
-        off += sg_scnpr(buff + off, 32, "SCSI generic (sg) device ");
+        off += sg_scn3pr(buff, bufflen, off, "SCSI generic (sg) device ");
     if (FT_BLOCK & ft)
-        off += sg_scnpr(buff + off, 32, "block device ");
+        off += sg_scn3pr(buff, bufflen, off, "block device ");
     if (FT_ST & ft)
-        off += sg_scnpr(buff + off, 32, "SCSI tape device ");
+        off += sg_scn3pr(buff, bufflen, off, "SCSI tape device ");
     if (FT_RAW & ft)
-        off += sg_scnpr(buff + off, 32, "raw device ");
+        off += sg_scn3pr(buff, bufflen, off, "raw device ");
     if (FT_OTHER & ft)
-        off += sg_scnpr(buff + off, 32, "other (perhaps ordinary file) ");
+        off += sg_scn3pr(buff, bufflen, off, "other (perhaps ordinary file) ");
     if (FT_ERROR & ft)
-        sg_scnpr(buff + off, 32, "unable to 'stat' file ");
+        sg_scn3pr(buff, bufflen, off, "unable to 'stat' file ");
     return buff;
 }
 
@@ -271,7 +374,7 @@ usage()
     pr2serr("               [bpt=BPT] [cdbsz=6|10|12|16] [dio=0|1] "
             "[fua=0|1|2|3]\n"
             "               [sync=0|1] [time=0|1] [verbose=VERB] "
-            "[--dry-run] [--verbose]\n\n"
+            "[--dry-run] [--progress] [--verbose]\n\n"
             "  where:\n"
             "    bpt         is blocks_per_transfer (default is 128)\n"
             "    bs          must be device logical block size (default "
@@ -303,6 +406,7 @@ usage()
             "etc\n"
             "    --dry-run|-d    prepare but bypass copy/read\n"
             "    --help|-h       print usage message then exit\n"
+            "    --progress|-p    outputs progress report every 2 minutes\n"
             "    --verbose|-v    increase verbosity\n"
             "    --version|-V    print version information then exit\n\n"
             "Copy from IFILE to OFILE, similar to dd command\n"
@@ -338,7 +442,7 @@ scsi_read_capacity(int sg_fd, int64_t * num_sect, int * sect_sz)
         *num_sect = (int64_t)ui + 1;
         *sect_sz = sg_get_unaligned_be32(rcBuff + 4);
     }
-    if (verbose)
+    if (verb)
         pr2serr("      number of blocks=%" PRId64 " [0x%" PRIx64 "], block "
                 "size=%d\n", *num_sect, *num_sect, *sect_sz);
     return 0;
@@ -363,7 +467,7 @@ read_blkdev_capacity(int sg_fd, int64_t * num_sect, int * sect_sz)
             return -1;
         }
         *num_sect = ((int64_t)ull / (int64_t)*sect_sz);
-        if (verbose)
+        if (verbose > 1)
             pr2serr("      [bgs64] number of blocks=%" PRId64 " [0x%" PRIx64
                     "], logical block size=%d\n", *num_sect, *num_sect,
                     *sect_sz);
@@ -375,7 +479,7 @@ read_blkdev_capacity(int sg_fd, int64_t * num_sect, int * sect_sz)
             return -1;
         }
         *num_sect = (int64_t)ul;
-        if (verbose)
+        if (verbose > 1)
             pr2serr("      [bgs] number of blocks=%" PRId64 " [0x%" PRIx64
                     "], logical block size=%d\n", *num_sect, *num_sect,
                     *sect_sz);
@@ -383,6 +487,7 @@ read_blkdev_capacity(int sg_fd, int64_t * num_sect, int * sect_sz)
     }
     return 0;
 #else
+    if (sg_fd) { ; }      /* unused, suppress warning */
     if (verbose)
         pr2serr("      BLKSSZGET+BLKGETSIZE ioctl not available\n");
     *num_sect = 0;
@@ -412,18 +517,18 @@ sg_build_scsi_cdb(uint8_t * cdbp, int cdb_sz, unsigned int blocks,
         sg_put_unaligned_be24(0x1fffff & start_block, cdbp + 1);
         cdbp[4] = (256 == blocks) ? 0 : (uint8_t)blocks;
         if (blocks > 256) {
-            pr2serr(ME "for 6 byte commands, maximum number of blocks is "
-                    "256\n");
+            pr2serr("%sfor 6 byte commands, maximum number of blocks is "
+                    "256\n", my_name);
             return 1;
         }
         if ((start_block + blocks - 1) & (~0x1fffff)) {
-            pr2serr(ME "for 6 byte commands, can't address blocks beyond "
-                    "%d\n", 0x1fffff);
+            pr2serr("%sfor 6 byte commands, can't address blocks beyond "
+                    "%d\n", my_name, 0x1fffff);
             return 1;
         }
         if (dpo || fua) {
-            pr2serr(ME "for 6 byte commands, neither dpo nor fua bits "
-                    "supported\n");
+            pr2serr("%sfor 6 byte commands, neither dpo nor fua bits "
+                    "supported\n", my_name);
             return 1;
         }
         break;
@@ -434,8 +539,8 @@ sg_build_scsi_cdb(uint8_t * cdbp, int cdb_sz, unsigned int blocks,
         sg_put_unaligned_be32((uint32_t)start_block, cdbp + 2);
         sg_put_unaligned_be16((uint16_t)blocks, cdbp + 7);
         if (blocks & (~0xffff)) {
-            pr2serr(ME "for 10 byte commands, maximum number of blocks is "
-                    "%d\n", 0xffff);
+            pr2serr("%sfor 10 byte commands, maximum number of blocks is "
+                    "%d\n", my_name, 0xffff);
             return 1;
         }
         break;
@@ -454,8 +559,8 @@ sg_build_scsi_cdb(uint8_t * cdbp, int cdb_sz, unsigned int blocks,
         sg_put_unaligned_be32((uint32_t)blocks, cdbp + 10);
         break;
     default:
-        pr2serr(ME "expected cdb size of 6, 10, 12, or 16 but got %d\n",
-                cdb_sz);
+        pr2serr("%sexpected cdb size of 6, 10, 12, or 16 but got %d\n",
+                my_name, cdb_sz);
         return 1;
     }
     return 0;
@@ -470,13 +575,13 @@ sg_read(int sg_fd, uint8_t * buff, int blocks, int64_t from_block,
     bool print_cdb_after = false;
     int res;
     uint8_t rdCmd[MAX_SCSI_CDBSZ];
-    uint8_t senseBuff[SENSE_BUFF_LEN];
+    uint8_t senseBuff[SENSE_BUFF_LEN] SG_C_CPP_ZERO_INIT;
     struct sg_io_hdr io_hdr;
 
     if (sg_build_scsi_cdb(rdCmd, cdbsz, blocks, from_block, false, fua,
                           dpo)) {
-        pr2serr(ME "bad rd cdb build, from_block=%" PRId64 ", blocks=%d\n",
-                from_block, blocks);
+        pr2serr("%sbad rd cdb build, from_block=%" PRId64 ", blocks=%d\n",
+                my_name, from_block, blocks);
         return SG_LIB_SYNTAX_ERROR;
     }
     memset(&io_hdr, 0, sizeof(struct sg_io_hdr));
@@ -505,7 +610,10 @@ sg_read(int sg_fd, uint8_t * buff, int blocks, int64_t from_block,
            ((EINTR == errno) || (EAGAIN == errno) || (EBUSY == errno)))
         sleep(1);
     if (res < 0) {
-        perror(ME "SG_IO error (sg_read)");
+        char e[64];
+
+        snprintf(e, sizeof(e), "%s%s: SG_IO error", my_name, __func__);
+        perror(e);
         return -1;
     }
 #else
@@ -567,16 +675,15 @@ sg_write(int sg_fd, uint8_t * buff, int blocks, int64_t to_block,
     bool print_cdb_after = false;
     int res;
     uint8_t wrCmd[MAX_SCSI_CDBSZ];
-    uint8_t senseBuff[SENSE_BUFF_LEN];
-    struct sg_io_hdr io_hdr;
+    uint8_t senseBuff[SENSE_BUFF_LEN] SG_C_CPP_ZERO_INIT;
+    struct sg_io_hdr io_hdr SG_C_CPP_ZERO_INIT;
 
     if (sg_build_scsi_cdb(wrCmd, cdbsz, blocks, to_block, true, fua, dpo)) {
-        pr2serr(ME "bad wr cdb build, to_block=%" PRId64 ", blocks=%d\n",
-                to_block, blocks);
+        pr2serr("%sbad wr cdb build, to_block=%" PRId64 ", blocks=%d\n",
+                my_name, to_block, blocks);
         return SG_LIB_SYNTAX_ERROR;
     }
 
-    memset(&io_hdr, 0, sizeof(struct sg_io_hdr));
     io_hdr.interface_id = 'S';
     io_hdr.cmd_len = cdbsz;
     io_hdr.cmdp = wrCmd;
@@ -604,7 +711,10 @@ sg_write(int sg_fd, uint8_t * buff, int blocks, int64_t to_block,
            ((EINTR == errno) || (EAGAIN == errno) || (EBUSY == errno)))
         sleep(1);
     if (res < 0) {
-        perror(ME "SG_IO error (sg_write)");
+        char e[64];
+
+        snprintf(e, sizeof(e), "%s%s: SG_IO error", my_name, __func__);
+        perror(e);
         return -1;
     }
 #else
@@ -713,6 +823,136 @@ num_chs_in_str(const char * s, int slen, int ch)
     return res;
 }
 
+#define PROGRESS_TRIGGER_MS 120000      /* milliseconds: 2 minutes */
+#define PROGRESS2_TRIGGER_MS 60000      /* milliseconds: 1 minute */
+#define PROGRESS3_TRIGGER_MS 30000      /* milliseconds: 30 seconds */
+
+/* Returns true when it time to output a progress report; else false. */
+static bool
+check_progress()
+{
+#if defined(HAVE_CLOCK_GETTIME) && defined(CLOCK_MONOTONIC)
+    static bool have_prev, measure;
+    static struct timespec prev_true_tm;
+    static int count, threshold;
+    bool res = false;
+    uint32_t elapsed_ms, ms;
+    struct timespec now_tm, res_tm;
+
+    if (progress) {
+        if (! have_prev) {
+            have_prev = true;
+            measure = true;
+            clock_gettime(CLOCK_MONOTONIC, &prev_true_tm);
+            return false;       /* starting reference */
+        }
+        if (! measure) {
+            if (++count >= threshold)
+                count = 0;
+            else
+                return false;
+        }
+        clock_gettime(CLOCK_MONOTONIC, &now_tm);
+        res_tm.tv_sec = now_tm.tv_sec - prev_true_tm.tv_sec;
+        res_tm.tv_nsec = now_tm.tv_nsec - prev_true_tm.tv_nsec;
+        if (res_tm.tv_nsec < 0) {
+            --res_tm.tv_sec;
+            res_tm.tv_nsec += 1000000000;
+        }
+        elapsed_ms = (1000 * res_tm.tv_sec) + (res_tm.tv_nsec / 1000000);
+        if (measure) {
+            ++threshold;
+            if (elapsed_ms > 80)        /* 80 milliseconds */
+                measure = false;
+        }
+        if (elapsed_ms >= PROGRESS3_TRIGGER_MS) {
+            if (elapsed_ms >= PROGRESS2_TRIGGER_MS) {
+                if (elapsed_ms >= PROGRESS_TRIGGER_MS) {
+                    ms = PROGRESS_TRIGGER_MS;
+                    res = true;
+                } else if (progress > 1) {
+                    ms = PROGRESS2_TRIGGER_MS;
+                    res = true;
+                }
+            } else if (progress > 2) {
+                ms = PROGRESS3_TRIGGER_MS;
+                res = true;
+            }
+        }
+        if (res) {
+            prev_true_tm.tv_sec += (ms / 1000);
+            prev_true_tm.tv_nsec += (ms % 1000) * 1000000;
+            if (prev_true_tm.tv_nsec >= 1000000000) {
+                ++prev_true_tm.tv_sec;
+                prev_true_tm.tv_nsec -= 1000000000;
+            }
+        }
+    }
+    return res;
+
+#elif defined(HAVE_GETTIMEOFDAY)
+    static bool have_prev, measure;
+    static struct timeval prev_true_tm;
+    static int count, threshold;
+    bool res = false;
+    uint32_t elapsed_ms, ms;
+    struct timeval now_tm, res_tm;
+
+    if (progress) {
+        if (! have_prev) {
+            have_prev = true;
+            gettimeofday(&prev_true_tm, NULL);
+            return false;       /* starting reference */
+        }
+        if (! measure) {
+            if (++count >= threshold)
+                count = 0;
+            else
+                return false;
+        }
+        gettimeofday(&now_tm, NULL);
+        res_tm.tv_sec = now_tm.tv_sec - prev_true_tm.tv_sec;
+        res_tm.tv_usec = now_tm.tv_usec - prev_true_tm.tv_usec;
+        if (res_tm.tv_usec < 0) {
+            --res_tm.tv_sec;
+            res_tm.tv_usec += 1000000;
+        }
+        elapsed_ms = (1000 * res_tm.tv_sec) + (res_tm.tv_usec / 1000);
+        if (measure) {
+            ++threshold;
+            if (elapsed_ms > 80)        /* 80 milliseconds */
+                measure = false;
+        }
+        if (elapsed_ms >= PROGRESS3_TRIGGER_MS) {
+            if (elapsed_ms >= PROGRESS2_TRIGGER_MS) {
+                if (elapsed_ms >= PROGRESS_TRIGGER_MS) {
+                    ms = PROGRESS_TRIGGER_MS;
+                    res = true;
+                } else if (progress > 1) {
+                    ms = PROGRESS2_TRIGGER_MS;
+                    res = true;
+                }
+            } else if (progress > 2) {
+                ms = PROGRESS3_TRIGGER_MS;
+                res = true;
+            }
+        }
+        if (res) {
+            prev_true_tm.tv_sec += (ms / 1000);
+            prev_true_tm.tv_usec += (ms % 1000) * 1000;
+            if (prev_true_tm.tv_usec >= 1000000) {
+                ++prev_true_tm.tv_sec;
+                prev_true_tm.tv_usec -= 1000000;
+            }
+        }
+    }
+    return res;
+
+#else   /* no clock reading functions available */
+    return false;
+#endif
+}
+
 
 #define STR_SZ 1024
 #define INOUTF_SZ 512
@@ -758,6 +998,8 @@ main(int argc, char * argv[])
     char b[80];
     struct flags_t in_flags;
     struct flags_t out_flags;
+    static const char * bat_s = "bad argument to";
+    static const int blen = sizeof(b);
 
 #if defined(HAVE_SYSCONF) && defined(_SC_PAGESIZE)
     psz = sysconf(_SC_PAGESIZE); /* POSIX.1 (was getpagesize()) */
@@ -768,6 +1010,8 @@ main(int argc, char * argv[])
     outf[0] = '\0';
     memset(&in_flags, 0, sizeof(in_flags));
     memset(&out_flags, 0, sizeof(out_flags));
+    if (getenv("SG3_UTILS_INVOCATION"))
+        sg_rep_invocation(my_name, version_str, argc, argv, stderr);
 
     for (k = 1; k < argc; k++) {
         if (argv[k])
@@ -782,18 +1026,22 @@ main(int argc, char * argv[])
         if (0 == strcmp(key,"bpt")) {
             bpt = sg_get_num(buf);
             if (-1 == bpt) {
-                pr2serr(ME "bad argument to 'bpt'\n");
+                pr2serr("%s%s 'bpt'\n", my_name, bat_s);
                 return SG_LIB_SYNTAX_ERROR;
             }
             bpt_given = true;
         } else if (0 == strcmp(key,"bs")) {
             blk_sz = sg_get_num(buf);
             if (-1 == blk_sz) {
-                pr2serr(ME "bad argument to 'bs'\n");
+                pr2serr("%s%s 'bs'\n", my_name, bat_s);
                 return SG_LIB_SYNTAX_ERROR;
             }
         } else if (0 == strcmp(key,"cdbsz")) {
             scsi_cdbsz_in = sg_get_num(buf);
+            if ((scsi_cdbsz_in < 6) || (scsi_cdbsz_in > 32)) {
+                pr2serr("%s'cdbsz' expects 6, 10, 12, 16 or 32\n", my_name);
+                return SG_LIB_SYNTAX_ERROR;
+            }
             scsi_cdbsz_out = scsi_cdbsz_in;
             cdbsz_given = true;
         } else if (0 == strcmp(key,"coe")) {
@@ -802,8 +1050,8 @@ main(int argc, char * argv[])
         } else if (0 == strcmp(key,"count")) {
             if (0 != strcmp("-1", buf)) {
                 dd_count = sg_get_llnum(buf);
-                if (-1LL == dd_count) {
-                    pr2serr(ME "bad argument to 'count'\n");
+                if ((dd_count < 0) || (dd_count > MAX_COUNT_SKIP_SEEK)) {
+                    pr2serr("%s%s 'count'\n", my_name, bat_s);
                     return SG_LIB_SYNTAX_ERROR;
                 }
             }   /* treat 'count=-1' as calculate count (same as not given) */
@@ -817,8 +1065,8 @@ main(int argc, char * argv[])
                 in_flags.fua = true;
         } else if (0 == strcmp(key,"ibs")) {
             ibs = sg_get_num(buf);
-            if (-1 == ibs) {
-                pr2serr(ME "bad argument to 'ibs'\n");
+            if ((ibs < 0) || (ibs > MAX_BPT_VALUE)) {
+                pr2serr("%s%s 'ibs'\n", my_name, bat_s);
                 return SG_LIB_SYNTAX_ERROR;
             }
         } else if (strcmp(key,"if") == 0) {
@@ -831,7 +1079,7 @@ main(int argc, char * argv[])
             }
         } else if (0 == strcmp(key, "iflag")) {
             if (process_flags(buf, &in_flags)) {
-                pr2serr(ME "bad argument to 'iflag'\n");
+                pr2serr("%s%s 'iflag'\n", my_name, bat_s);
                 return SG_LIB_SYNTAX_ERROR;
             }
         } else if (strcmp(key,"of") == 0) {
@@ -844,25 +1092,25 @@ main(int argc, char * argv[])
             }
         } else if (0 == strcmp(key, "oflag")) {
             if (process_flags(buf, &out_flags)) {
-                pr2serr(ME "bad argument to 'oflag'\n");
+                pr2serr("%s%s 'oflag'\n", my_name, bat_s);
                 return SG_LIB_SYNTAX_ERROR;
             }
         } else if (0 == strcmp(key,"obs")) {
             obs = sg_get_num(buf);
-            if (-1 == obs) {
-                pr2serr(ME "bad argument to 'obs'\n");
+            if ((obs < 0) || (obs > MAX_BPT_VALUE)) {
+                pr2serr("%s%s 'obs'\n", my_name, bat_s);
                 return SG_LIB_SYNTAX_ERROR;
             }
         } else if (0 == strcmp(key,"seek")) {
             seek = sg_get_llnum(buf);
-            if (-1LL == seek) {
-                pr2serr(ME "bad argument to 'seek'\n");
+            if ((seek < 0) || (seek > MAX_COUNT_SKIP_SEEK)) {
+                pr2serr("%s%s 'seek'\n", my_name, bat_s);
                 return SG_LIB_SYNTAX_ERROR;
             }
         } else if (0 == strcmp(key,"skip")) {
             skip = sg_get_llnum(buf);
-            if (-1LL == skip) {
-                pr2serr(ME "bad argument to 'skip'\n");
+            if ((skip < 0) || (skip > MAX_COUNT_SKIP_SEEK)) {
+                pr2serr("%s%s 'skip'\n", my_name, bat_s);
                 return SG_LIB_SYNTAX_ERROR;
             }
         } else if (0 == strcmp(key,"sync"))
@@ -881,6 +1129,9 @@ main(int argc, char * argv[])
                 usage();
                 return 0;
             }
+            n = num_chs_in_str(key + 1, keylen - 1, 'p');
+            progress += n;
+            res += n;
             n = num_chs_in_str(key + 1, keylen - 1, 'v');
             verbose += n;
             res += n;
@@ -900,7 +1151,9 @@ main(int argc, char * argv[])
                  (0 == strcmp(key, "-?"))) {
             usage();
             return 0;
-        } else if (0 == strncmp(key, "--verb", 6))
+        } else if (0 == strncmp(key, "--prog", 6))
+            ++progress;
+        else if (0 == strncmp(key, "--verb", 6))
             ++verbose;
         else if (0 == strncmp(key, "--vers", 6))
             version_given = true;
@@ -910,11 +1163,13 @@ main(int argc, char * argv[])
             return SG_LIB_SYNTAX_ERROR;
         }
     }
+    if (progress > 0)
+        do_time=true;
 #ifdef DEBUG
     pr2serr("In DEBUG mode, ");
     if (verbose_given && version_given) {
         pr2serr("but override: '-vV' given, zero verbose and continue\n");
-        verbose_given = false;
+        /* verbose_given = false; */
         version_given = false;
         verbose = 0;
     } else if (! verbose_given) {
@@ -927,7 +1182,7 @@ main(int argc, char * argv[])
         pr2serr("Not in DEBUG mode, so '-vV' has no special action\n");
 #endif
     if (version_given) {
-        pr2serr(ME ": %s\n", version_str);
+        pr2serr("%s%s\n", my_name, version_str);
         return 0;
     }
 
@@ -949,8 +1204,8 @@ main(int argc, char * argv[])
         pr2serr("Can't use both append and seek switches\n");
         return SG_LIB_CONTRADICT;
     }
-    if (bpt < 1) {
-        pr2serr("bpt must be greater than 0\n");
+    if ((bpt < 1) || (bpt > MAX_BPT_VALUE)) {
+        pr2serr("bpt must be > 0 and <= %d\n", MAX_BPT_VALUE);
         return SG_LIB_SYNTAX_ERROR;
     }
     /* defaulting transfer size to 128*2048 for CD/DVDs is too large
@@ -960,8 +1215,8 @@ main(int argc, char * argv[])
         bpt = DEF_BLOCKS_PER_2048TRANSFER;
 
 #ifdef DEBUG
-    pr2serr(ME "if=%s skip=%" PRId64 " of=%s seek=%" PRId64 " count=%" PRId64
-            "\n", inf, skip, outf, seek, dd_count);
+    pr2serr("%sif=%s skip=%" PRId64 " of=%s seek=%" PRId64 " count=%" PRId64
+            "\n", my_name, inf, skip, outf, seek, dd_count);
 #endif
     install_handler (SIGINT, interrupt_handler);
     install_handler (SIGQUIT, interrupt_handler);
@@ -972,15 +1227,15 @@ main(int argc, char * argv[])
     outfd = STDOUT_FILENO;
     if (inf[0] && ('-' != inf[0])) {
         in_type = dd_filetype(inf);
-        if (verbose)
+        if (verbose > 1)
             pr2serr(" >> Input file type: %s\n",
-                    dd_filetype_str(in_type, ebuff));
+                    dd_filetype_str(in_type, ebuff, sizeof(ebuff)));
 
         if (FT_ERROR == in_type) {
-            pr2serr(ME "unable to access %s\n", inf);
+            pr2serr("%sunable to access %s\n", my_name, inf);
             return SG_LIB_FILE_ERROR;
         } else if (FT_ST == in_type) {
-            pr2serr(ME "unable to use scsi tape device %s\n", inf);
+            pr2serr("%sunable to use scsi tape device %s\n", my_name, inf);
             return SG_LIB_FILE_ERROR;
         } else if (FT_SG == in_type) {
             flags = O_RDWR | O_NONBLOCK;
@@ -993,13 +1248,13 @@ main(int argc, char * argv[])
             if ((infd = open(inf, flags)) < 0) {
                 err = errno;
                 snprintf(ebuff, EBUFF_SZ,
-                         ME "could not open %s for sg reading", inf);
+                         "%scould not open %s for sg reading", my_name, inf);
                 perror(ebuff);
                 return sg_convert_errno(err);
             }
             res = ioctl(infd, SG_GET_VERSION_NUM, &t);
             if ((res < 0) || (t < 30122)) {
-                pr2serr(ME "sg driver prior to 3.1.22\n");
+                pr2serr("%ssg driver prior to 3.1.22\n", my_name);
                 return SG_LIB_FILE_ERROR;
             }
             in_res_sz = blk_sz * bpt;
@@ -1007,7 +1262,9 @@ main(int argc, char * argv[])
                 in_res_sz = ((in_res_sz / psz) + 1) * psz;
             if (ioctl(infd, SG_GET_RESERVED_SIZE, &t) < 0) {
                 err = errno;
-                perror(ME "SG_GET_RESERVED_SIZE error");
+                snprintf(ebuff, EBUFF_SZ, "%sSG_GET_RESERVED_SIZE error",
+                         my_name);
+                perror(ebuff);
                 return sg_convert_errno(err);
             }
             if (t < MIN_RESERVED_SIZE)
@@ -1015,7 +1272,9 @@ main(int argc, char * argv[])
             if (in_res_sz > t) {
                 if (ioctl(infd, SG_SET_RESERVED_SIZE, &in_res_sz) < 0) {
                     err = errno;
-                    perror(ME "SG_SET_RESERVED_SIZE error");
+                    snprintf(ebuff, EBUFF_SZ, "%sSG_SET_RESERVED_SIZE error",
+                             my_name);
+                    perror(ebuff);
                     return sg_convert_errno(err);
                 }
             }
@@ -1024,7 +1283,7 @@ main(int argc, char * argv[])
             if (MAP_FAILED == wrkMmap) {
                 err = errno;
                 snprintf(ebuff, EBUFF_SZ,
-                         ME "error using mmap() on file: %s", inf);
+                         "%serror using mmap() on file: %s", my_name, inf);
                 perror(ebuff);
                 return sg_convert_errno(err);
             }
@@ -1039,7 +1298,7 @@ main(int argc, char * argv[])
             if ((infd = open(inf, flags)) < 0) {
                 err = errno;
                 snprintf(ebuff, EBUFF_SZ,
-                         ME "could not open %s for reading", inf);
+                         "%scould not open %s for reading", my_name, inf);
                 perror(ebuff);
                 return sg_convert_errno(err);
             }
@@ -1049,12 +1308,12 @@ main(int argc, char * argv[])
                 offset *= blk_sz;       /* could exceed 32 bits here! */
                 if (lseek64(infd, offset, SEEK_SET) < 0) {
                     err = errno;
-                    snprintf(ebuff, EBUFF_SZ, ME "couldn't skip to "
-                             "required position on %s", inf);
+                    snprintf(ebuff, EBUFF_SZ, "%scouldn't skip to "
+                             "required position on %s", my_name, inf);
                     perror(ebuff);
                     return sg_convert_errno(err);
                 }
-                if (verbose)
+                if (verbose > 1)
                     pr2serr("  >> skip: lseek64 SEEK_SET, byte offset=0x%"
                             PRIx64 "\n", (uint64_t)offset);
             }
@@ -1063,12 +1322,12 @@ main(int argc, char * argv[])
 
     if (outf[0] && ('-' != outf[0])) {
         out_type = dd_filetype(outf);
-        if (verbose)
+        if (verbose > 1)
             pr2serr(" >> Output file type: %s\n",
-                    dd_filetype_str(out_type, ebuff));
+                    dd_filetype_str(out_type, ebuff, sizeof(ebuff)));
 
         if (FT_ST == out_type) {
-            pr2serr(ME "unable to use scsi tape device %s\n", outf);
+            pr2serr("%sunable to use scsi tape device %s\n", my_name, outf);
             return SG_LIB_FILE_ERROR;
         }
         else if (FT_SG == out_type) {
@@ -1081,19 +1340,21 @@ main(int argc, char * argv[])
                 flags |= O_SYNC;
             if ((outfd = open(outf, flags)) < 0) {
                 err = errno;
-                snprintf(ebuff, EBUFF_SZ, ME "could not open %s for "
-                         "sg writing", outf);
+                snprintf(ebuff, EBUFF_SZ, "%scould not open %s for "
+                         "sg writing", my_name, outf);
                 perror(ebuff);
                 return sg_convert_errno(err);
             }
             res = ioctl(outfd, SG_GET_VERSION_NUM, &t);
             if ((res < 0) || (t < 30122)) {
-                pr2serr(ME "sg driver prior to 3.1.22\n");
+                pr2serr("%ssg driver prior to 3.1.22\n", my_name);
                 return SG_LIB_FILE_ERROR;
             }
             if (ioctl(outfd, SG_GET_RESERVED_SIZE, &t) < 0) {
                 err = errno;
-                perror(ME "SG_GET_RESERVED_SIZE error");
+                snprintf(ebuff, EBUFF_SZ, "%sSG_GET_RESERVED_SIZE error",
+                         my_name);
+                perror(ebuff);
                 return sg_convert_errno(err);
             }
            if (t < MIN_RESERVED_SIZE)
@@ -1102,7 +1363,9 @@ main(int argc, char * argv[])
             if (out_res_sz > t) {
                 if (ioctl(outfd, SG_SET_RESERVED_SIZE, &out_res_sz) < 0) {
                     err = errno;
-                    perror(ME "SG_SET_RESERVED_SIZE error");
+                    snprintf(ebuff, EBUFF_SZ, "%sSG_SET_RESERVED_SIZE error",
+                             my_name);
+                    perror(ebuff);
                     return sg_convert_errno(err);
                 }
             }
@@ -1111,8 +1374,8 @@ main(int argc, char * argv[])
                                 PROT_READ | PROT_WRITE, MAP_SHARED, outfd, 0);
                 if (MAP_FAILED == wrkMmap) {
                     err = errno;
-                    snprintf(ebuff, EBUFF_SZ,
-                             ME "error using mmap() on file: %s", outf);
+                    snprintf(ebuff, EBUFF_SZ, "%serror using mmap() on "
+                             "file: %s", my_name, outf);
                     perror(ebuff);
                     return sg_convert_errno(err);
                 }
@@ -1133,8 +1396,8 @@ main(int argc, char * argv[])
                     flags |= O_APPEND;
                 if ((outfd = open(outf, flags, 0666)) < 0) {
                     err = errno;
-                    snprintf(ebuff, EBUFF_SZ,
-                             ME "could not open %s for writing", outf);
+                    snprintf(ebuff, EBUFF_SZ, "%scould not open %s for "
+                             "writing", my_name, outf);
                     perror(ebuff);
                     return sg_convert_errno(err);
                 }
@@ -1142,8 +1405,8 @@ main(int argc, char * argv[])
             else {
                 if ((outfd = open(outf, O_WRONLY)) < 0) {
                     err = errno;
-                    snprintf(ebuff, EBUFF_SZ, ME "could not open %s "
-                             "for raw writing", outf);
+                    snprintf(ebuff, EBUFF_SZ, "%scould not open %s "
+                             "for raw writing", my_name, outf);
                     perror(ebuff);
                     return sg_convert_errno(err);
                 }
@@ -1154,12 +1417,12 @@ main(int argc, char * argv[])
                 offset *= blk_sz;       /* could exceed 32 bits here! */
                 if (lseek64(outfd, offset, SEEK_SET) < 0) {
                     err = errno;
-                    snprintf(ebuff, EBUFF_SZ, ME "couldn't seek to "
-                             "required position on %s", outf);
+                    snprintf(ebuff, EBUFF_SZ, "%scouldn't seek to "
+                             "required position on %s", my_name, outf);
                     perror(ebuff);
                     return sg_convert_errno(err);
                 }
-                if (verbose)
+                if (verbose > 1)
                     pr2serr("   >> seek: lseek64 SEEK_SET, byte offset=0x%"
                             PRIx64 "\n", (uint64_t)offset);
             }
@@ -1183,7 +1446,7 @@ main(int argc, char * argv[])
                 res = scsi_read_capacity(infd, &in_num_sect, &in_sect_sz);
             }
             if (0 != res) {
-                sg_get_category_sense_str(res, sizeof(b), b, verbose);
+                sg_get_category_sense_str(res, blen, b, verbose);
                 pr2serr("Read capacity (if=%s): %s\n", inf, b);
                 in_num_sect = -1;
             }
@@ -1212,7 +1475,7 @@ main(int argc, char * argv[])
                 res = scsi_read_capacity(outfd, &out_num_sect, &out_sect_sz);
             }
             if (0 != res) {
-                sg_get_category_sense_str(res, sizeof(b), b, verbose);
+                sg_get_category_sense_str(res, blen, b, verbose);
                 pr2serr("Read capacity (of=%s): %s\n", inf, b);
                 out_num_sect = -1;
             }
@@ -1274,11 +1537,11 @@ main(int argc, char * argv[])
         int fd;
         char c;
 
-        if ((fd = open(proc_allow_dio, O_RDONLY)) >= 0) {
+        if ((fd = open(sg_allow_dio, O_RDONLY)) >= 0) {
             if (1 == read(fd, &c, 1)) {
                 if ('0' == c)
                     pr2serr(">>> %s set to '0' but should be set to '1' for "
-                            "direct IO\n", proc_allow_dio);
+                            "direct IO\n", sg_allow_dio);
             }
             close(fd);
         }
@@ -1316,7 +1579,7 @@ main(int argc, char * argv[])
         pr2serr("Since both 'if' and 'of' are sg devices, only do mmap-ed "
                 "transfers on 'if'\n");
 
-    while (dd_count > 0) {
+    while (dd_count > 0) {      /* start of main copy loop */
         blocks = (dd_count > blocks_per) ? blocks_per : dd_count;
         if (FT_SG == in_type) {
             ret = sg_read(infd, wrkPos, blocks, skip, blk_sz, scsi_cdbsz_in,
@@ -1345,8 +1608,8 @@ main(int argc, char * argv[])
                 pr2serr("read(unix): count=%d, res=%d\n", blocks * blk_sz,
                         res);
             if (ret < 0) {
-                snprintf(ebuff, EBUFF_SZ, ME "reading, skip=%" PRId64 " ",
-                         skip);
+                snprintf(ebuff, EBUFF_SZ, "%sreading, skip=%" PRId64 " ",
+                         my_name, skip);
                 perror(ebuff);
                 ret = -1;
                 break;
@@ -1400,8 +1663,8 @@ main(int argc, char * argv[])
                 pr2serr("write(unix): count=%d, res=%d\n", blocks * blk_sz,
                         res);
             if (res < 0) {
-                snprintf(ebuff, EBUFF_SZ, ME "writing, seek=%" PRId64 " ",
-                         seek);
+                snprintf(ebuff, EBUFF_SZ, "%swriting, seek=%" PRId64 " ",
+                         my_name, seek);
                 perror(ebuff);
                 break;
             }
@@ -1420,7 +1683,13 @@ main(int argc, char * argv[])
             dd_count -= blocks;
         skip += blocks;
         seek += blocks;
-    }
+        if (progress > 0) {
+            if (check_progress()) {
+                calc_duration_throughput(true);
+                print_stats();
+            }
+        }
+    }                   /* end of main while loop */
 
     if (do_time)
         calc_duration_throughput(false);
@@ -1433,7 +1702,7 @@ main(int argc, char * argv[])
                 res = sg_ll_sync_cache_10(outfd, 0, 0, 0, 0, 0, false, 0);
             }
             if (0 != res) {
-                sg_get_category_sense_str(res, sizeof(b), b, verbose);
+                sg_get_category_sense_str(res, blen, b, verbose);
                 pr2serr("Synchronize cache(out): %s\n", b);
             }
         }
@@ -1442,10 +1711,12 @@ main(int argc, char * argv[])
 fini:
     if (wrkBuff)
         free(wrkBuff);
-    if (STDIN_FILENO != infd)
+    if ((STDIN_FILENO != infd) && (infd >= 0))
         close(infd);
-    if ((STDOUT_FILENO != outfd) && (FT_DEV_NULL != out_type))
-        close(outfd);
+    if ((STDOUT_FILENO != outfd) && (FT_DEV_NULL != out_type)) {
+        if (outfd >= 0)
+            close(outfd);
+    }
     if ((0 != dd_count) && (0 == dry_run)) {
         pr2serr("Some error occurred,");
         if (0 == ret)
