@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2018-2019 Douglas Gilbert
+ * Copyright (c) 2018-2021 Douglas Gilbert
  * All rights reserved.
  * Use of this source code is governed by a BSD-style
  * license that can be found in the BSD_LICENSE file.
@@ -39,7 +39,7 @@
 #include "sg_unaligned.h"
 #include "sg_pr2serr.h"
 
-static const char * version_str = "1.06 20190113";
+static const char * version_str = "1.07 20210225";
 
 
 #define ME "sg_tst_nvme: "
@@ -116,7 +116,7 @@ static int next_dev_name_pos = 0;
 static void
 usage()
 {
-    pr2serr("Usage: sg_tst_nvme [--ctl] [dev-id] [--help] [--long] "
+    pr2serr("Usage: sg_tst_nvme [--ctl] [--dev-id] [--help] [--long] "
             "[--maxlen=LEN]\n"
             "                   [--nsid=ID] [--self-test=ST] [--to-ms=TO] "
             "[--verbose]\n"
@@ -381,7 +381,7 @@ nvme_din_admin_cmd(struct sg_pt_base * ptvp, const uint8_t *cmdp,
     int res, k;
     uint16_t sct_sc = 0;
     uint32_t result, clen;
-    uint8_t sense_b[SENSE_BUFF_NVME_LEN];
+    uint8_t sense_b[SENSE_BUFF_NVME_LEN] SG_C_CPP_ZERO_INIT;
     uint8_t ucmd[128];
     char b[32];
 
@@ -514,7 +514,7 @@ sg_scsi_inquiry(struct sg_pt_base * ptvp, bool evpd, int pg_op, void * resp,
 {
     int res, ret, k, sense_cat, resid;
     uint8_t inq_cdb[INQUIRY_CMDLEN] = {INQUIRY_CMD, 0, 0, 0, 0, 0};
-    uint8_t sense_b[SENSE_BUFF_LEN];
+    uint8_t sense_b[SENSE_BUFF_LEN] SG_C_CPP_ZERO_INIT;
     uint8_t * up;
 
     if (evpd)
