@@ -14,18 +14,18 @@
  * Later extensions (versions 2, 3 and 4) to driver:
  *   Copyright (C) 1998 - 2021 Douglas Gilbert
  *
- * Version 4.0.46 (20210111)
+ * Version 4.0.47 (20210605)
  *  This version is for Linux 4 and 5 series kernels.
  *
  * Documentation
  * =============
  * A web site for the SG device driver can be found at:
- *   http://sg.danny.cz/sg  [alternatively check the MAINTAINERS file]
+ *   https://sg.danny.cz/sg  [alternatively check the MAINTAINERS file]
  * The documentation for the sg version 3 driver can be found at:
- *   http://sg.danny.cz/sg/p/sg_v3_ho.html
+ *   https://sg.danny.cz/sg/p/sg_v3_ho.html
  * Also see: <kernel_source>/Documentation/scsi/scsi-generic.txt
  *
- * For utility and test programs see: http://sg.danny.cz/sg/sg3_utils.html
+ * For utility and test programs see: https://sg.danny.cz/sg/sg3_utils.html
  */
 
 #include <stddef.h>
@@ -114,12 +114,13 @@ typedef struct sg_io_hdr {
 #define SGV4_FLAG_YIELD_TAG 0x8  /* sg_io_v4::generated_tag set after SG_IOS */
 #define SGV4_FLAG_Q_AT_TAIL SG_FLAG_Q_AT_TAIL
 #define SGV4_FLAG_Q_AT_HEAD SG_FLAG_Q_AT_HEAD
-#define SGV4_FLAG_NO_WAITQ  0x40	/* need to poll for completion */
-#define SGV4_FLAG_DOUT_OFFSET  0x80	/* dout byte offset in v4::spare_in */
+#define SGV4_FLAG_DOUT_OFFSET  0x40	/* dout byte offset in v4::spare_in */
+#define SGV4_FLAG_EVENTFD 0x80		/* signal completion on ... */
 #define SGV4_FLAG_COMPLETE_B4  0x100	/* mrq: complete this rq before next */
 #define SGV4_FLAG_SIGNAL 0x200	/* v3: ignored; v4 signal on completion */
 #define SGV4_FLAG_IMMED 0x400   /* issue request and return immediately ... */
-#define SGV4_FLAG_HIPRI 0x800 /* request will use blk_poll to complete */
+#define SGV4_FLAG_HIPRI 0x800 /* use blk_poll (deprecated name, use POLLED) */
+#define SGV4_FLAG_POLLED 0x800 /* request will use blk_poll to complete */
 #define SGV4_FLAG_STOP_IF 0x1000	/* Stops sync mrq if error or warning */
 #define SGV4_FLAG_DEV_SCOPE 0x2000 /* permit SG_IOABORT to have wider scope */
 #define SGV4_FLAG_SHARE 0x4000	/* share IO buffer; needs SG_SEIM_SHARE_FD */
@@ -127,9 +128,9 @@ typedef struct sg_io_hdr {
 #define SGV4_FLAG_NO_DXFER SG_FLAG_NO_DXFER /* but keep dev<-->kernel xfr */
 #define SGV4_FLAG_KEEP_SHARE 0x20000  /* ... buffer for another dout command */
 #define SGV4_FLAG_MULTIPLE_REQS 0x40000	/* 1 or more sg_io_v4-s in data-in */
-#define SGV4_FLAG_EVENTFD 0x80000	/* signal completion on ... */
-#define SGV4_FLAG_ORDERED_WR 0x100000	/* svb: issue in-order writes */
-#define SGV4_FLAG_REC_ORDER 0x200000 /* receive order in v4:request_priority */
+#define SGV4_FLAG_ORDERED_WR 0x80000	/* svb: issue in-order writes */
+#define SGV4_FLAG_REC_ORDER 0x100000 /* receive order in v4:request_priority */
+#define SGV4_FLAG_META_OUT_IF 0x200000  /* ... there is something to report */
 
 /* Output (potentially OR-ed together) in v3::info or v4::info field */
 #define SG_INFO_OK_MASK 0x1
@@ -166,7 +167,7 @@ typedef struct sg_scsi_id {
 typedef struct sg_req_info {	/* used by SG_GET_REQUEST_TABLE ioctl() */
 	char req_state;	/* See 'enum sg_rq_state' definition in v4 driver */
 	char orphan;	/* 0 -> normal request, 1 -> from interrupted SG_IO */
-	/* sg_io_owned set imples synchronous, clear implies asynchronous */
+	/* sg_io_owned set implies synchronous, clear implies asynchronous */
 	char sg_io_owned;/* 0 -> complete with read(), 1 -> owned by SG_IO */
 	char problem;	/* 0 -> no problem detected, 1 -> error to report */
 	/* If SG_CTL_FLAGM_TAG_FOR_PACK_ID set on fd then next field is tag */
@@ -220,8 +221,7 @@ typedef struct sg_req_info {	/* used by SG_GET_REQUEST_TABLE ioctl() */
 #define SG_CTL_FLAGM_EXCL_WAITQ 0x1000	/* only 1 wake up per response */
 #define SG_CTL_FLAGM_SNAP_DEV	0x2000	/* output to debugfs::snapped */
 #define SG_CTL_FLAGM_RM_EVENTFD	0x4000	/* only if new eventfd wanted */
-#define SG_CTL_FLAGM_NO_WAIT_POLL 0x8000 /* POLLERR on poll(2)s that wait */
-#define SG_CTL_FLAGM_ALL_BITS	0xffff	/* should be OR of previous items */
+#define SG_CTL_FLAGM_ALL_BITS	0x7fff	/* should be OR of previous items */
 
 /* Write one of the following values to sg_extended_info::read_value, get... */
 #define SG_SEIRV_INT_MASK	0x0	/* get SG_SEIM_ALL_BITS */
@@ -443,8 +443,8 @@ struct sg_header {
 #define SG_SET_COMMAND_Q 0x2271   /* Change queuing state with 0 or 1 */
 
 /*
- * Turn on/off error sense trace (1 and 0 respectively, default is off).
- * Try using: "# cat /proc/scsi/sg/debug" instead in the v3 driver
+ * Turn on/off error sense trace in the kernel log (1 and 0 respectively, default is
+ * off).
  */
 #define SG_SET_DEBUG 0x227e    /* 0 -> turn off debug */
 
